/***************************************************************************
                        audio_block.c  -  description
                             -------------------
   Functions for handling AC3 audio blocks.
                             -------------------
    begin                : Fri Aug 29 2005
    copyright            : (C) 2005 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*#define AC3_AUDIO_BLOCK_TRACE*/
#ifdef AC3_AUDIO_BLOCK_TRACE
# include <stdio.h>
#endif
#include <stddef.h>
#include <time.h>

#include "audio_block.h"
#include "AC3_tables.h"
#include "bit_alloc.h"
#include "uncouple_channels.h"
#include "get_coeffs.h"
#include "bitstream.h"

#define DATA_BYTE       data
#define BIT_SHIFT_BYTE  bit_shift
#define DATA_PTR        ptr
#define END_DATA_PTR    end_pos
#define PARSER_STATE    frame->state


/* Read the channel config info in the audio block..
      INPUT: frame - Pointer to an AC3_SYNC_FRAME. */
void AUDIO_BLOCK_read_config(AC3_SYNC_FRAME *frame,
                             AUDIO_BLOCK    *blk)
{
   const u8 *ptr       = frame->payload,
            *end_pos   = frame->payload + frame->payload_len16;
   u8        bit_shift = frame->bit_shift,
             data      = frame->data,
             bit_shift_inc;
#undef  STOP_PARSING
#define STOP_PARSING    goto AUDIO_BLOCK_read_config_END

#define GET_DITHER               (AUDIO_BLOCK_read_config_STATE+ 1)
#define GET_DYN_RANGE_GAIN       (AUDIO_BLOCK_read_config_STATE+ 2)
#define GET_DYN_RANGE_GAIN2      (AUDIO_BLOCK_read_config_STATE+ 3)
#define GET_DYN_RANGE_GAIN2b     (AUDIO_BLOCK_read_config_STATE+ 4)
#define GET_COUPLING_INFO        (AUDIO_BLOCK_read_config_STATE+ 5)
#define GET_COUPLED_SBAND        (AUDIO_BLOCK_read_config_STATE+ 6)
#define GET_COUPLED_SBANDS1      (AUDIO_BLOCK_read_config_STATE+ 7)
#define GET_COUPLED_SBANDS2      (AUDIO_BLOCK_read_config_STATE+ 8)
#define GET_COUPLING_COORD1      (AUDIO_BLOCK_read_config_STATE+ 9)
#define GET_COUPLING_COORD2      (AUDIO_BLOCK_read_config_STATE+10)
#define GET_COUPLING_EXPMAN      (AUDIO_BLOCK_read_config_STATE+11)
#define GET_PHASE_INVERT         (AUDIO_BLOCK_read_config_STATE+12)
#define GET_REMATRIX_INFO        (AUDIO_BLOCK_read_config_STATE+13)
#define SIDE_INFO_DONE           (AUDIO_BLOCK_read_config_STATE+14)
   if (frame->state > AUDIO_BLOCK_read_config_STATE) {
      switch (frame->state) {
      case GET_DITHER:
         goto AUDIO_BLOCK_read_config_GET_DITHER;
      case GET_DYN_RANGE_GAIN:
         goto AUDIO_BLOCK_read_config_GET_DYN_RANGE_GAIN;
      case GET_DYN_RANGE_GAIN2:
         goto AUDIO_BLOCK_read_config_GET_DYN_RANGE_GAIN2;
      case GET_DYN_RANGE_GAIN2b:
         goto AUDIO_BLOCK_read_config_GET_DYN_RANGE_GAIN2b;
      case GET_COUPLING_INFO:
         goto AUDIO_BLOCK_read_config_GET_COUPLING_INFO;
      case GET_COUPLED_SBAND:
         goto AUDIO_BLOCK_read_config_GET_COUPLED_SBAND;
      case GET_COUPLED_SBANDS1:
         goto AUDIO_BLOCK_read_config_GET_COUPLED_SBANDS1;
      case GET_COUPLED_SBANDS2:
         goto AUDIO_BLOCK_read_config_GET_COUPLED_SBANDS2;
      case GET_COUPLING_COORD1:
         goto AUDIO_BLOCK_read_config_GET_COUPLING_COORD1;
      case GET_COUPLING_COORD2:
         goto AUDIO_BLOCK_read_config_GET_COUPLING_COORD2;
      case GET_COUPLING_EXPMAN:
         goto AUDIO_BLOCK_read_config_GET_COUPLING_EXPMAN;
      case GET_PHASE_INVERT:
         goto AUDIO_BLOCK_read_config_GET_PHASE_INVERT;
      case GET_REMATRIX_INFO:
         goto AUDIO_BLOCK_read_config_GET_REMATRIX_INFO;
      case SIDE_INFO_DONE:
         goto AUDIO_BLOCK_read_config_DONE;
      }
   }
#ifdef AC3_AUDIO_BLOCK_TRACE
   printf("  AUDIO_BLOCK_%u:\n    Config Info:\n", blk->num);
/*   printf("START> 0x%02hX,%hu (%02hX %02hX)\n", (u16)DATA_BYTE, 
                                                (u16)BIT_SHIFT_BYTE, 
                                                (u16)*ptr, 
                                                (u16)*(ptr+1));*/
#endif
   blk->blk_switch  = blk->dither = 0xFF << (8-frame->num_ch);
   blk->blk_switch &= DATA_BYTE;
#ifdef AC3_AUDIO_BLOCK_TRACE
   printf("      blk_switch=0x%02hX\n", (u16)blk->blk_switch);
#endif
   ALIGN_DATA_BYTE(GET_DITHER, frame->num_ch);

AUDIO_BLOCK_read_config_GET_DITHER:
   GET_LSB;
   blk->dither &= DATA_BYTE;
#ifdef AC3_AUDIO_BLOCK_TRACE
   printf("      dither=0x%02hX\n", (u16)blk->dither);
#endif
   blk->i = DATA_BYTE & (0x80 >> frame->num_ch);
   ALIGN_DATA_BYTE(GET_DYN_RANGE_GAIN, (frame->num_ch+1));

AUDIO_BLOCK_read_config_GET_DYN_RANGE_GAIN:
   if (blk->i) {
/*      GET_LSB;
      blk->dyn_range_gain[0] = DATA_BYTE; */
#ifdef AC3_AUDIO_BLOCK_TRACE
      GET_LSB;
      printf("      dyn_range_gain[0]=%hu (ignored)\n",
             (u16)DATA_BYTE);
#endif
      ALIGN8_DATA_BYTE(GET_DYN_RANGE_GAIN2);
   }
/*   else if (!blk->num)
      blk->dyn_range_gain[0] = 0;*/
   
AUDIO_BLOCK_read_config_GET_DYN_RANGE_GAIN2:
   if (frame->mode < AC3_AUDIO_AC_10) {
      GET_LSB;
      blk->i = DATA_BYTE;
      ALIGN_DATA_BYTE(GET_DYN_RANGE_GAIN2b, 1);
AUDIO_BLOCK_read_config_GET_DYN_RANGE_GAIN2b:
      if (blk->i >= 0x80) {
/*         GET_LSB;
         blk->dyn_range_gain[1] = DATA_BYTE;*/
#ifdef AC3_AUDIO_BLOCK_TRACE
         GET_LSB;
         printf("      dyn_range_gain[1]=%hu (ignored)\n",
                (u16)DATA_BYTE);
#endif
         ALIGN8_DATA_BYTE(GET_COUPLING_INFO);
      }
/*      else if (!blk->num)
         blk->dyn_range_gain[1] = 0;*/
   }

AUDIO_BLOCK_read_config_GET_COUPLING_INFO:
   GET_LSB;
   if (DATA_BYTE >= 0x80) {
      bit_shift_inc = 2;
      if (DATA_BYTE < 0xC0)
         blk->coupled_ch = 0;
      else {
         bit_shift_inc    += frame->num_ch;
         blk->coupled_ch   = DATA_BYTE & (0xFF << (6-frame->num_ch));
         blk->coupled_ch <<= 2;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      coupled_ch=0x%02hX\n", (u16)blk->coupled_ch);
#endif
         if (AC3_AUDIO_AC(frame) == AC3_AUDIO_AC_20) {
            blk->phase_invert = (u32)(DATA_BYTE & 0x08);
#ifdef AC3_AUDIO_BLOCK_TRACE
            printf("      phsflginu=%u\n", (blk->phase_invert >> 3));
#endif
            bit_shift_inc++;
         }
         else
            blk->phase_invert = 0;
         ALIGN_DATA_BYTE(GET_COUPLED_SBAND, bit_shift_inc);
AUDIO_BLOCK_read_config_GET_COUPLED_SBAND:
         GET_LSB;
         blk->cpl_data.bin_start =  DATA_BYTE >> 4;
         blk->num_coupled_bands  = (DATA_BYTE &  0x0F) + 3 -
                                   blk->cpl_data.bin_start;
         blk->cpl_data.num_bins  = 12 * blk->num_coupled_bands;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      cplbegf          =%hu\n"
                "      cplendf          =%hu\n"
                "      ncplsubnd        =%hu\n", 
                (u16)blk->cpl_data.bin_start,
                (u16)((DATA_BYTE & 0x0F) + 3),
                (u16)blk->num_coupled_bands);
#endif
         blk->cpl_data.bin_start *= 12;
         blk->cpl_data.bin_start += 37;
         ALIGN8_DATA_BYTE(GET_COUPLED_SBANDS1);

AUDIO_BLOCK_read_config_GET_COUPLED_SBANDS1:
         blk->j              = blk->num_coupled_bands - 1;
         blk->coupled_sbands = 0;
         if (blk->j >= AC3_NUM_BND)
            goto AUDIO_BLOCK_read_config_ERROR;
         else if (!blk->j)
            goto AUDIO_BLOCK_read_config_GET_COUPLING_COORD1;
         else {
AUDIO_BLOCK_read_config_GET_COUPLED_SBANDS2:
            while (blk->j > 8) {
               blk->j -= 8;
               GET_LSB;
               blk->coupled_sbands |= (u32)DATA_BYTE << blk->j;
               ALIGN8_DATA_BYTE(GET_COUPLED_SBANDS2);
            }
            bit_shift_inc = blk->j;
            GET_LSB;
            blk->coupled_sbands  |= (u32)DATA_BYTE >> (8-bit_shift_inc);
            blk->coupled_sbands <<= 32 - blk->num_coupled_bands;
            frame->tmp_data       = blk->coupled_sbands;
            do {
               frame->tmp_data <<= 1;
               if (frame->tmp_data >= 0x80000000)
                  blk->num_coupled_bands--;
            } while (frame->tmp_data);
         }
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      num_coupled_bands=%hu\n"
                "      coupled_sbands   =0x%08X\n",
                (u16)blk->num_coupled_bands,
                blk->coupled_sbands);
#endif
      }
   }
   else
      bit_shift_inc = 1;
   ALIGN_DATA_BYTE(GET_COUPLING_COORD1, bit_shift_inc);

AUDIO_BLOCK_read_config_GET_COUPLING_COORD1:
   blk->cpl_data.ch = 0xFF;
   if (blk->coupled_ch) {
      blk->j       = blk->coupled_ch;
      blk->ch      = 0;
      blk->u8_data = blk->ch_data[0].cpl_coord;
      while (1) {
         if (blk->j >= 0x80) {
            if (blk->cpl_data.ch == 0xFF)
               blk->cpl_data.ch = blk->ch;
            *(blk->u8_data) = 0x40;          /*  chincpl */
            GET_LSB;
            if (DATA_BYTE >= 0x80) {
               *(blk->u8_data++) |= 0x20;    /* cplcoe */
               *(blk->u8_data++)  = (DATA_BYTE >> 5) & 0x03;   /* mstrcplco */
               blk->i             = blk->num_coupled_bands;
               ALIGN_DATA_BYTE(GET_COUPLING_EXPMAN, 3);
AUDIO_BLOCK_read_config_GET_COUPLING_EXPMAN:
               while (blk->i--) {
                  GET_LSB;
                  *(blk->u8_data++) = DATA_BYTE;   /* cplcoexp, cplcomant */
                  ALIGN8_DATA_BYTE(GET_COUPLING_EXPMAN);
               }
               blk->u8_data -= blk->num_coupled_bands + 2;
            }
            else {
               ALIGN_DATA_BYTE(GET_COUPLING_COORD2, 1);
            }
AUDIO_BLOCK_read_config_GET_COUPLING_COORD2:
            if (blk->j == 0x80) {
               *(blk->u8_data) |= 0x80;   /* last_cpl_ch */
               break;
            }
         }
         else {
            *(blk->u8_data) = 0;
            if (!blk->j)
               break;
         }
         blk->u8_data += sizeof(CH_STRUCT);
         blk->ch++;
         blk->j <<= 1;
      }

#ifdef AC3_AUDIO_BLOCK_TRACE
      if (blk->cpl_data.ch != 0xFF) {
         blk->ch      = blk->cpl_data.ch;
         blk->u8_data = blk->ch_data[blk->ch].cpl_coord;
         printf("      coupled_channels =%02hX", 
                (u16)(*(blk->u8_data)|blk->ch));
         while (*(blk->u8_data) < 0x80) {
            blk->ch++;
            blk->u8_data += sizeof(CH_STRUCT);
            printf(",%02hX", (u16)(*(blk->u8_data)|blk->ch));
         }
         printf("\n");
      
         blk->ch      = blk->cpl_data.ch;
         blk->u8_data = blk->ch_data[blk->ch].cpl_coord;
         do {
            blk->i = *(blk->u8_data);
            if (blk->i & 0x20) {
               printf("      coupled_gain  [%hu]=%hu\n"
                      "      coupled_expman[%hu]=",
                      (u16)blk->ch, 
                      (u16)blk->u8_data[1],
                      (u16)blk->ch);
               blk->j = 2;
               do {
                  printf("%02hX ", (u16)blk->u8_data[blk->j]);
               } while (blk->j++ <= blk->num_coupled_bands);
               printf("\n");
            }
            blk->ch++;
            blk->u8_data += sizeof(CH_STRUCT);
         } while (blk->i < 0x80);
      }
#endif
      
      if ( (blk->phase_invert)&&(blk->cpl_data.ch < 2) ) {
         blk->j            = blk->num_coupled_bands;
         blk->phase_invert = 0;
AUDIO_BLOCK_read_config_GET_PHASE_INVERT:
         while (blk->j > 8) {
            blk->j -= 8;
            GET_LSB;
            blk->phase_invert |= (u32)DATA_BYTE << blk->j;
            ALIGN8_DATA_BYTE(GET_PHASE_INVERT);
         }
         GET_LSB;
         blk->phase_invert  |= (u32)DATA_BYTE >> (8-blk->j);
         blk->phase_invert <<= 32 - blk->num_coupled_bands;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      phase_invert=0x%08X\n", blk->phase_invert);
#endif
         ALIGN_DATA_BYTE(GET_REMATRIX_INFO, blk->j);
      }
   }

   if (AC3_AUDIO_AC(frame) == AC3_AUDIO_AC_20) {
AUDIO_BLOCK_read_config_GET_REMATRIX_INFO:
      GET_LSB;
      bit_shift_inc = 1;
      if (DATA_BYTE >= 0x80) {
         if ( (blk->cpl_data.bin_start > 61)||
              (!blk->coupled_ch) )
            bit_shift_inc += 4;
         else if (blk->cpl_data.bin_start > 37)
            bit_shift_inc += 3;
         else
            bit_shift_inc += 2;
         blk->rematrix = DATA_BYTE & (0xFF << (8-bit_shift_inc));
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      rematrix=0x%02hX\n", (u16)blk->rematrix);
#endif
      }
      ALIGN_DATA_BYTE(SIDE_INFO_DONE, bit_shift_inc);
   }
   else
      blk->rematrix = 0;

AUDIO_BLOCK_read_config_DONE:
   frame->bit_shift      = bit_shift;
   frame->data           = data;
   frame->state          = AUDIO_BLOCK_read_config_END_STATE;
   frame->payload_len16 -= (u16)(ptr - frame->payload);
   frame->payload        = ptr;
   return;

AUDIO_BLOCK_read_config_ERROR:
   frame->state          = AC3_AUDIO_next_frame_STATE;
   frame->payload_len16 -= (u16)(ptr - frame->payload);
   frame->payload        = ptr;
   return;

AUDIO_BLOCK_read_config_END:
   frame->bit_shift     = bit_shift;
   frame->data          = data;
   frame->payload_len16 = 0;
#undef GET_DITHER
#undef GET_DYN_RANGE_GAIN
#undef GET_DYN_RANGE_GAIN2
#undef GET_DYN_RANGE_GAIN2b
#undef GET_COUPLING_INFO
#undef GET_COUPLED_SBAND
#undef GET_COUPLED_SBANDS1
#undef GET_COUPLED_SBANDS2
#undef GET_COUPLING_COORD1
#undef GET_COUPLING_COORD2
#undef GET_COUPLING_EXPMAN
#undef GET_PHASE_INVERT
#undef GET_REMATRIX_INFO
#undef SIDE_INFO_DONE
}

/* Read the exponents in the audio block..
      INPUT: frame - Pointer to an AC3_SYNC_FRAME. */
void AUDIO_BLOCK_read_exp(AC3_SYNC_FRAME *frame,
                          AUDIO_BLOCK    *blk)
{
   const u8 *ptr       = frame->payload,
            *end_pos   = frame->payload + frame->payload_len16;
   u8        bit_shift = frame->bit_shift,
             data      = frame->data;
#undef  STOP_PARSING
#define STOP_PARSING    goto AUDIO_BLOCK_read_exp_END

#define GET_MANT_PER_EXP   (AUDIO_BLOCK_read_exp_STATE+ 1)
#define GET_NUM_BINS1      (AUDIO_BLOCK_read_exp_STATE+ 2)
#define GET_NUM_BINS2      (AUDIO_BLOCK_read_exp_STATE+ 3)
#define GET_EXP1           (AUDIO_BLOCK_read_exp_STATE+ 4)
#define GET_EXP2           (AUDIO_BLOCK_read_exp_STATE+ 5)
#define GET_EXP3           (AUDIO_BLOCK_read_exp_STATE+ 6)
   if (frame->state > AUDIO_BLOCK_read_exp_STATE) {
      switch (frame->state) {
      case GET_MANT_PER_EXP:
         goto AUDIO_BLOCK_read_exp_GET_MANT_PER_EXP;
      case GET_NUM_BINS1:
         goto AUDIO_BLOCK_read_exp_GET_NUM_BINS1;
      case GET_NUM_BINS2:
         goto AUDIO_BLOCK_read_exp_GET_NUM_BINS2;
      case GET_EXP1:
         goto AUDIO_BLOCK_read_exp_GET_EXP1;
      case GET_EXP2:
         goto AUDIO_BLOCK_read_exp_GET_EXP2;
      case GET_EXP3:
         goto AUDIO_BLOCK_read_exp_GET_EXP3;
      }
   }
#ifdef AC3_AUDIO_BLOCK_TRACE
   printf("    Exponents:\n");
/*   printf("START> 0x%02hX,%hu (%02hX %02hX)\n", (u16)DATA_BYTE, 
                                                (u16)BIT_SHIFT_BYTE, 
                                                (u16)*ptr, 
                                                (u16)*(ptr+1));*/
#endif
   blk->lfe_ptr = (LFE_STRUCT *)&(blk->ch_data[frame->num_ch]);
   blk->u8_data = &(blk->cpl_data.mant_per_exp);
   if (!blk->coupled_ch) {
      *(blk->u8_data-1) =
      *(blk->u8_data)   = 0;
      goto AUDIO_BLOCK_read_exp_GET_MANT_PER_EXP;
   }

   do {
      GET_LSB;
      if (DATA_BYTE > 0x3F)
         *(blk->u8_data) = 0x11 << ((DATA_BYTE >> 6)-1);
      ALIGN_DATA_BYTE(GET_MANT_PER_EXP, 2);
AUDIO_BLOCK_read_exp_GET_MANT_PER_EXP:
      blk->u8_data += (blk->u8_data < (u8 *)&(blk->ch_data)) ? 
                      sizeof(CPL_STRUCT) : sizeof(CH_STRUCT);
   } while (blk->u8_data < (u8 *)blk->lfe_ptr);

   if (frame->mode & AC3_AUDIO_AC_LFE) {
      *(blk->u8_data-1) = AC3_MAX_LFE_BIN;   /* num_bins */
      GET_LSB;
      if (DATA_BYTE > 0x7F)
         *(blk->u8_data) = 0x11;             /* mant_per_exp */
      ALIGN_DATA_BYTE(GET_NUM_BINS1, 1);
   }
   else {
      *(blk->u8_data-1) =
      *(blk->u8_data)   = 0;
   }

AUDIO_BLOCK_read_exp_GET_NUM_BINS1:
   blk->u8_data = &(blk->ch_data[0].num_bins);
   blk->j       = blk->coupled_ch;
   do {
      if (*(blk->u8_data+1) > 0x10) {
         if (blk->j >= 0x80)
            *(blk->u8_data) = blk->cpl_data.bin_start;
         else {
            GET_LSB;
            *(blk->u8_data) = ((DATA_BYTE >> 2)*3) + 73;
            ALIGN_DATA_BYTE(GET_NUM_BINS2, 6);
         }
      }
AUDIO_BLOCK_read_exp_GET_NUM_BINS2:
      blk->u8_data += sizeof(CH_STRUCT);
      blk->j      <<= 1;
   } while (blk->u8_data < (u8 *)blk->lfe_ptr);

   blk->ch      = 0xFF;
   blk->u8_data = &(blk->cpl_data.num_bins);
   if (blk->u8_data[1] > 0x10) {
      blk->i = blk->u8_data[0] /
               (3 * (blk->u8_data[1] & 0x0F));
      GET_LSB;
      frame->tmp_data = offsetof(CPL_STRUCT, exp)-1;
      blk->u8_data   += offsetof(CPL_STRUCT, exp)-1;  /* exp[] */
      *(blk->u8_data) = (DATA_BYTE >> 3) & 0xFE;
      goto AUDIO_BLOCK_read_exp_GET_EXP0;
   }
   else
      goto AUDIO_BLOCK_read_exp_GET_EXP3;
   
   do {
      blk->ch++;
      if (blk->u8_data[1] > 0x10) {
         blk->i = (blk->u8_data[0] - 4) /
                  (3 * (blk->u8_data[1] & 0x0F));
         blk->i++;
         GET_LSB;
         frame->tmp_data = offsetof(CH_STRUCT, exp);
         blk->u8_data   += offsetof(CH_STRUCT, exp);  /* exp[] */
         *(blk->u8_data) = DATA_BYTE >> 4;
AUDIO_BLOCK_read_exp_GET_EXP0:
         if (*(blk->u8_data) > 24) {
#ifdef AC3_AUDIO_BLOCK_TRACE
            printf("<<EXP PARSE ERROR>>\n");
#endif
            goto AUDIO_BLOCK_read_exp_ERROR;
         }
         ALIGN_DATA_BYTE(GET_EXP1, 4);
         do {
AUDIO_BLOCK_read_exp_GET_EXP1:
            GET_LSB;
            blk->j = DATA_BYTE >> 1;
            ALIGN_DATA_BYTE(GET_EXP2, 7);
AUDIO_BLOCK_read_exp_GET_EXP2:
            blk->u8_data[1]  =  (blk->j / 25)   -2+blk->u8_data[0];
            blk->u8_data[2]  = ((blk->j % 25)/5)-2+blk->u8_data[1];
            blk->u8_data[3]  =  (blk->j %  5)   -2+blk->u8_data[2];
            if ( (blk->u8_data[1] > 24)||
                 (blk->u8_data[2] > 24)||
                 (blk->u8_data[3] > 24) ) {
#ifdef AC3_AUDIO_BLOCK_TRACE
               printf("<<EXP PARSE ERROR>>\n");
#endif
               goto AUDIO_BLOCK_read_exp_ERROR;
            }
            blk->u8_data    += 3;
            frame->tmp_data += 3;
         } while (--(blk->i));
         blk->u8_data -= frame->tmp_data; /* num_bins */

         if (blk->ch < AC3_MAX_CH) {
            GET_LSB;
/*            blk->gainrng[blk->ch] = DATA_BYTE >> 6;*/
            ALIGN_DATA_BYTE(GET_EXP3, 2);
         }
      }
AUDIO_BLOCK_read_exp_GET_EXP3:
      blk->u8_data += (blk->u8_data < (u8 *)&(blk->ch_data)) ? 
                      sizeof(CPL_STRUCT) : sizeof(CH_STRUCT);
   } while (blk->u8_data < (u8 *)blk->lfe_ptr);

   if ( (blk->u8_data == (u8 *)blk->lfe_ptr)&&
        (blk->u8_data[1] > 0x10) ) {
      blk->ch = 0xFF;
      blk->i  = 2;
      GET_LSB;
      frame->tmp_data = offsetof(LFE_STRUCT, exp);
      blk->u8_data   += offsetof(LFE_STRUCT, exp); /* exp[] */
      *(blk->u8_data) = DATA_BYTE >> 4;
      goto AUDIO_BLOCK_read_exp_GET_EXP0;
   }

#ifdef AC3_AUDIO_BLOCK_TRACE
   if (blk->coupled_ch) {
      printf("      coupled_bin_start =%hu\n"
             "      coupled_bin_end   =%hu\n"
             "      cpl_num_bins      =%hu\n"
             "      cpl_mant_per_exp  =0x%02hX\n", 
             (u16)blk->cpl_data.bin_start, 
             (u16)(blk->cpl_data.bin_start+blk->cpl_data.num_bins),
             (u16)blk->cpl_data.num_bins, 
             (u16)blk->cpl_data.mant_per_exp);
   }
   blk->ch = 0;
   do {
      printf("      ch_num_bins[%hu]    =%hu\n"
             "      ch_mant_per_exp[%hu]=0x%02hX\n", 
             (u16)blk->ch, (u16)blk->ch_data[blk->ch].num_bins,
             (u16)blk->ch, (u16)blk->ch_data[blk->ch].mant_per_exp);
   } while (++(blk->ch) < frame->num_ch);
   if (frame->mode & AC3_AUDIO_AC_LFE) {
      printf("      lfe_num_bins      =%hu\n"
             "      lfe_mant_per_exp  =0x%02hX\n",
             (u16)blk->lfe_ptr->num_bins, 
             (u16)blk->lfe_ptr->mant_per_exp);
   }

   if (blk->cpl_data.mant_per_exp > 0x10) {
      blk->i = blk->cpl_data.num_bins / 
               (3*(blk->cpl_data.mant_per_exp & 0x0F));
      blk->u8_data = blk->cpl_data.exp;
      blk->u8_data--;
      printf("      cpl_exp[  0]=%2hu  (ngrps=%hu)\n",
             (u16)blk->u8_data[0], (u16)blk->i);
      blk->i *= 3;
      blk->j  = 1;
      do {
         printf("             [%3hu]=%2hu\n",
                (u16)( ( (blk->j-1) *
                         (blk->cpl_data.mant_per_exp & 0x0F) ) + 
                       blk->cpl_data.bin_start ), 
                (u16)blk->u8_data[blk->j]);
      } while (++(blk->j) <= blk->i);
   }
   blk->ch = 0;
   do {
      if (blk->ch_data[blk->ch].mant_per_exp > 0x10) {
         frame->tmp_data = (u32)(blk->ch_data[blk->ch].mant_per_exp&0x0F);
         blk->i = blk->ch_data[blk->ch].num_bins / 
                 (3*(blk->ch_data[blk->ch].mant_per_exp & 0x0F));
         blk->u8_data = blk->ch_data[blk->ch].exp;
/*         printf("       gainrng[%hu] =%2hu\n",
                (u16)blk->ch, (u16)blk->gainrng[blk->ch]);*/
         printf("       exp[%hu][  0]=%2hu  (ngrps=%hu)\n",
                (u16)blk->ch,
                (u16)blk->u8_data[0], (u16)blk->i);
         blk->i *= 3;
         blk->j  = 1;
         do {
            printf("             [%3u]=%2hu\n",
                   ((u32)(blk->j-1) * frame->tmp_data) + 1, 
                   (u16)blk->u8_data[blk->j]);
         } while (++(blk->j) <= blk->i);
      }
   } while (++(blk->ch) < frame->num_ch);
   if (blk->lfe_ptr->mant_per_exp > 0x10) {
      blk->i = 6;
      blk->j = 1;
      blk->u8_data = blk->lfe_ptr->exp;
      printf("      lfe_exp[  0]=%2hu  (ngrps=2)\n",
             (u16)blk->u8_data[0]);
      do {
         printf("             [%3hu]=%2hu\n",
                (u16)blk->j, (u16)blk->u8_data[blk->j]);
      } while (++(blk->j) <= blk->i);
   }
#endif

   GET_LSB;
   frame->bit_shift      = bit_shift;
   frame->data           = data;
   frame->state          = AUDIO_BLOCK_read_exp_END_STATE;
   frame->payload_len16 -= (u16)(ptr - frame->payload);
   frame->payload        = ptr;
   return;

AUDIO_BLOCK_read_exp_ERROR:
   frame->state          = AC3_AUDIO_next_frame_STATE;
   frame->payload_len16 -= (u16)(ptr - frame->payload);
   frame->payload        = ptr;
   return;
AUDIO_BLOCK_read_exp_END:
   frame->bit_shift     = bit_shift;
   frame->data          = data;
   frame->payload_len16 = 0;
#undef GET_MANT_PER_EXP
#undef GET_NUM_BINS1
#undef GET_NUM_BINS2
#undef GET_EXP1
#undef GET_EXP2
#undef GET_EXP3
}

/* Read the bit allocation info in the audio block..
      INPUT: frame - Pointer to an AC3_SYNC_FRAME. */
void AUDIO_BLOCK_read_bit_alloc(AC3_SYNC_FRAME *frame,
                                AUDIO_BLOCK    *blk)
{
   const u8 *ptr       = frame->payload,
            *end_pos   = frame->payload + frame->payload_len16;
   u8        bit_shift = frame->bit_shift,
             data      = frame->data,
             bit_shift_inc;
#undef  STOP_PARSING
#define STOP_PARSING    goto AUDIO_BLOCK_read_bit_alloc_END

#define GET_BAI               (AUDIO_BLOCK_read_bit_alloc_STATE+ 1)
#define GET_SNR_INFO          (AUDIO_BLOCK_read_bit_alloc_STATE+ 2)
#define GET_SNR_GAIN1         (AUDIO_BLOCK_read_bit_alloc_STATE+ 3)
#define GET_SNR_GAIN2         (AUDIO_BLOCK_read_bit_alloc_STATE+ 4)
#define GET_COUPLED_FS_LEAK   (AUDIO_BLOCK_read_bit_alloc_STATE+ 5)
#define GET_DELT_INFO         (AUDIO_BLOCK_read_bit_alloc_STATE+ 6)
#define GET_DELTBAE           (AUDIO_BLOCK_read_bit_alloc_STATE+ 7)
#define GET_DELTNSEG          (AUDIO_BLOCK_read_bit_alloc_STATE+ 8)
#define GET_DELTOFFST         (AUDIO_BLOCK_read_bit_alloc_STATE+ 9)
#define GET_DELTLENBA         (AUDIO_BLOCK_read_bit_alloc_STATE+10)
#define SKIP_BYTES1           (AUDIO_BLOCK_read_bit_alloc_STATE+11)
#define SKIP_BYTES2           (AUDIO_BLOCK_read_bit_alloc_STATE+12)
#define SKIP_BYTES3           (AUDIO_BLOCK_read_bit_alloc_STATE+13)
#define SKIP_BYTES4           (AUDIO_BLOCK_read_bit_alloc_STATE+14)
#define BIT_ALLOC_DONE        (AUDIO_BLOCK_read_bit_alloc_STATE+15)
   if (frame->state > AUDIO_BLOCK_read_bit_alloc_STATE) {
      switch (frame->state) {
      case GET_BAI:
         goto AUDIO_BLOCK_read_bit_alloc_GET_BAI;
      case GET_SNR_INFO:
         goto AUDIO_BLOCK_read_bit_alloc_GET_SNR_INFO;
      case GET_SNR_GAIN1:
         goto AUDIO_BLOCK_read_bit_alloc_GET_SNR_GAIN1;
      case GET_SNR_GAIN2:
         goto AUDIO_BLOCK_read_bit_alloc_GET_SNR_GAIN2;
      case GET_COUPLED_FS_LEAK:
         goto AUDIO_BLOCK_read_bit_alloc_GET_COUPLED_FS_LEAK;
      case GET_DELT_INFO:
         goto AUDIO_BLOCK_read_bit_alloc_GET_DELT_INFO;
      case GET_DELTBAE:
         goto AUDIO_BLOCK_read_bit_alloc_GET_DELTBAE;
      case GET_DELTNSEG:
         goto AUDIO_BLOCK_read_bit_alloc_GET_DELTNSEG;
      case GET_DELTOFFST:
         goto AUDIO_BLOCK_read_bit_alloc_GET_DELTOFFST;
      case GET_DELTLENBA:
         goto AUDIO_BLOCK_read_bit_alloc_GET_DELTLENBA;
      case SKIP_BYTES1:
         goto AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES1;
      case SKIP_BYTES2:
         goto AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES2;
      case SKIP_BYTES3:
         goto AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES3;
      case SKIP_BYTES4:
         goto AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES4;
      case BIT_ALLOC_DONE:
         goto AUDIO_BLOCK_read_bit_alloc_DONE;
      }
   }
#ifdef AC3_AUDIO_BLOCK_TRACE
   printf("    Bit Allocation:\n");
/*   printf("START> 0x%02hX,%hu (%02hX %02hX)\n", (u16)DATA_BYTE, 
                                                (u16)BIT_SHIFT_BYTE, 
                                                (u16)*ptr, 
                                                (u16)*(ptr+1));*/
#endif
   blk->new_bai = 0;
   if (DATA_BYTE >= 0x80) {
      blk->new_bai = 1;
      blk->cpl_data.sdecay = 0x0F+ ((DATA_BYTE >> 4)&0x06);
      blk->cpl_data.fdecay = 0x3F+(((DATA_BYTE >> 3)&0x03) * 0x14);
      ALIGN_DATA_BYTE(GET_BAI, 5);
AUDIO_BLOCK_read_bit_alloc_GET_BAI:
      GET_LSB;
      blk->cpl_data.bai = DATA_BYTE >> 1;
#ifdef AC3_AUDIO_BLOCK_TRACE
      printf("      sdecay  =0x%02hX\n"
             "      fdecay  =0x%02hX\n"
             "      sgaincod=%2hu\n"
             "      dbpbcod =%2hu\n"
             "      floorcod=%2hu\n", 
             (u16)blk->cpl_data.sdecay,
             (u16)blk->cpl_data.fdecay,
             (u16)(blk->cpl_data.bai >> 5),
             (u16)(blk->cpl_data.bai >> 3)&0x03,
             (u16)(blk->cpl_data.bai & 0x07));
#endif
      bit_shift_inc = 7;
   }
   else
      bit_shift_inc = 1;
   ALIGN_DATA_BYTE(GET_SNR_INFO, bit_shift_inc);
   
AUDIO_BLOCK_read_bit_alloc_GET_SNR_INFO:
   GET_LSB;
   if (DATA_BYTE >= 0x80) {
      blk->new_bai = 1;
      blk->cpl_data.csnroffst = (DATA_BYTE >> 1) & 0x3F;
#ifdef AC3_AUDIO_BLOCK_TRACE
      printf("      csnroffst      =%2hu\n", 
             (u16)blk->cpl_data.csnroffst);
#endif
      ALIGN_DATA_BYTE(GET_SNR_GAIN1, 7);

AUDIO_BLOCK_read_bit_alloc_GET_SNR_GAIN1:
      blk->u8_data = &(blk->cpl_data.num_bins);
      if (!blk->coupled_ch)
         goto AUDIO_BLOCK_read_bit_alloc_GET_SNR_GAIN2;

      do {
         GET_LSB;
         blk->u8_data[2] = DATA_BYTE;  /* snr_gain */
         ALIGN_DATA_BYTE(GET_SNR_GAIN2, 7);
AUDIO_BLOCK_read_bit_alloc_GET_SNR_GAIN2:
         blk->u8_data += (blk->u8_data < (u8 *)&(blk->ch_data)) ? 
                         sizeof(CPL_STRUCT) : sizeof(CH_STRUCT);
      } while ( (blk->u8_data <= (u8 *)blk->lfe_ptr)&&
                (blk->u8_data[0]) );
   }
   else {
      ALIGN_DATA_BYTE(GET_COUPLED_FS_LEAK, 1);
   }
#ifdef AC3_AUDIO_BLOCK_TRACE
   if (blk->coupled_ch) {
      printf("      cpl_snr_gain   =0x%02hX\n", 
             (u16)blk->cpl_data.snr_gain & 0xFE);
   }
   blk->ch = 0;
   do {
      printf("      ch_snr_gain[%hu] =0x%02hX\n", 
             (u16)blk->ch,
             (u16)blk->ch_data[blk->ch].snr_gain & 0xFE);
   } while (++(blk->ch) < frame->num_ch);
   if (frame->mode & AC3_AUDIO_AC_LFE) {
      printf("      lfe_snr_gain   =0x%02hX\n", 
             (u16)blk->lfe_ptr->snr_gain & 0xFE);
   }
#endif
   
AUDIO_BLOCK_read_bit_alloc_GET_COUPLED_FS_LEAK:
   if (blk->coupled_ch) {
      bit_shift_inc = 1;
      GET_LSB;
      if (DATA_BYTE >= 0x80) {
         blk->cpl_data.mant_per_exp |= 0x08;
         blk->cpl_data.fs_leak       =  DATA_BYTE     & 0x70;
         blk->cpl_data.fs_leak      |= (DATA_BYTE>>1) & 0x07;
         blk->cpl_data.fs_leak       = 0x99 - blk->cpl_data.fs_leak;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      coupled_fs_leak=0x%02hX  (%02hX)\n", 
                (u16)blk->cpl_data.fs_leak,
                (u16)blk->cpl_data.mant_per_exp);
#endif
         bit_shift_inc += 6;
      }
      ALIGN_DATA_BYTE(GET_DELT_INFO, bit_shift_inc);
   }

AUDIO_BLOCK_read_bit_alloc_GET_DELT_INFO:
   GET_LSB;
   if (DATA_BYTE >= 0x80) {
      blk->u8_data  = blk->cpl_data.delt;
      if (!blk->coupled_ch) {
         bit_shift_inc   = 1;
         *(blk->u8_data) = AC3_DELTBAE_NONE;
      }
      else {
         bit_shift_inc   = 3;
         blk->i          = (DATA_BYTE << 1) & 0xC0;
         if (blk->i == AC3_DELTBAE_NEW) {
            *(blk->u8_data)    = AC3_DELTBAE_NEW;
            *(blk->u8_data-2) |= 0x08; /* mant_per_exp */
         }
         else if ( (blk->i == AC3_DELTBAE_REUSE)&&(blk->num)&&
                   (AC3_DELTBAE(blk->u8_data) <= AC3_DELTBAE_NEW) ) {
            *(blk->u8_data)   &= ~AC3_DELTBAE_NEW; /* REUSE */
            *(blk->u8_data-2) |= 0x08; /* mant_per_exp */
         }
         else
            *(blk->u8_data)    = AC3_DELTBAE_NONE;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      cpldeltbae=%hu  (%02hX)\n", 
                (u16)blk->u8_data[0] >> 6,
                (u16)blk->u8_data[-2]);
#endif
      }
      blk->u8_data += sizeof(CPL_STRUCT);
#ifdef AC3_AUDIO_BLOCK_TRACE
      blk->ch = 0;
#endif
      while (1) {
         ALIGN_DATA_BYTE(GET_DELTBAE, bit_shift_inc);
AUDIO_BLOCK_read_bit_alloc_GET_DELTBAE:
         if (blk->u8_data >= (u8 *)blk->lfe_ptr)
            break;
         GET_LSB;
         blk->i = DATA_BYTE & 0xC0;
         if (blk->i == AC3_DELTBAE_NEW) {
            *(blk->u8_data)    = AC3_DELTBAE_NEW;
            *(blk->u8_data-2) |= 0x08; /* mant_per_exp */
         }
         else if ( (blk->i == AC3_DELTBAE_REUSE)&&(blk->num)&&
                   (AC3_DELTBAE(blk->u8_data) <= AC3_DELTBAE_NEW) ) {
            *(blk->u8_data)   &= ~AC3_DELTBAE_NEW; /* REUSE */
            *(blk->u8_data-2) |= 0x08; /* mant_per_exp */
         }
         else
            *(blk->u8_data)    = AC3_DELTBAE_NONE;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      deltbae[%hu]=%hu  (%02hX)\n", 
                (u16)blk->ch++, (u16)blk->u8_data[0] >> 6,
                (u16)blk->u8_data[-2]);
#endif
         blk->u8_data += sizeof(CH_STRUCT);
         bit_shift_inc = 2;
      }
      
      blk->u8_data = blk->cpl_data.delt;
      do {
         if (*(blk->u8_data) == AC3_DELTBAE_NEW) {
            GET_LSB;
            blk->j = ((DATA_BYTE >> 4) & 0x0E) + 2;
            ALIGN_DATA_BYTE(GET_DELTNSEG, 3);
AUDIO_BLOCK_read_bit_alloc_GET_DELTNSEG:
            blk->i           = 1;
            blk->u8_data[0] |= blk->j;
            do {
               GET_LSB;
               blk->u8_data[blk->i++] = DATA_BYTE >> 3;
               ALIGN_DATA_BYTE(GET_DELTOFFST, 5);
AUDIO_BLOCK_read_bit_alloc_GET_DELTOFFST:
               GET_LSB;
               blk->u8_data[blk->i]   = DATA_BYTE & 0xFE;
               ALIGN_DATA_BYTE(GET_DELTLENBA, 7);
AUDIO_BLOCK_read_bit_alloc_GET_DELTLENBA:
               blk->i++;
            } while (blk->i < blk->j);
         }
         blk->u8_data += (blk->u8_data < (u8 *)&(blk->ch_data)) ? 
                         sizeof(CPL_STRUCT) : sizeof(CH_STRUCT);
      } while (blk->u8_data < (u8 *)blk->lfe_ptr);
#ifdef AC3_AUDIO_BLOCK_TRACE
      if (AC3_DELTBAE(blk->cpl_data.delt) == AC3_DELTBAE_NEW) {
         blk->i = 0;
         blk->j = (AC3_DELTNSEG(blk->cpl_data.delt) >> 1) - 1;
         printf("      cpldeltnseg            =%hu\n"
                "      cpldelt.offst.len.ba",
                (u16)blk->j);
         blk->u8_data = blk->cpl_data.delt + 1;
         do {
            printf("[%hu]=%hu.%hu.%hu\n                          ",
                   (u16)blk->i,
                   (u16)*(blk->u8_data++),
                   (u16)*(blk->u8_data) >> 4,
                   (u16)*(blk->u8_data++)& 0x0F);
         } while (++(blk->i) <= blk->j);
         printf("\n");
      }
      blk->ch = 0;
      do {
         if (AC3_DELTBAE(blk->ch_data[blk->ch].delt) == AC3_DELTBAE_NEW) {
            blk->i = 0;
            blk->j = (AC3_DELTNSEG(blk->ch_data[blk->ch].delt) >> 1) - 1;
            printf("      deltnseg[%hu]            =%hu\n"
                   "      delt[%hu].offst.len.ba",
                   (u16)blk->ch, (u16)blk->j, (u16)blk->ch);
            blk->u8_data = blk->ch_data[blk->ch].delt + 1;
            do {
               printf("[%hu]=%hu.%hu.%hu\n                          ",
                      (u16)blk->i,
                      (u16)*(blk->u8_data++),
                      (u16)*(blk->u8_data) >> 4,
                      (u16)*(blk->u8_data++)& 0x0F);
            } while (++(blk->i) <= blk->j);
            printf("\n");
         }
      } while (++(blk->ch) < frame->num_ch);
#endif
   }
   else {
      if (!blk->num) {
         blk->u8_data = blk->cpl_data.delt;
         do {
            *(blk->u8_data) = AC3_DELTBAE_NONE;
            blk->u8_data   += (blk->u8_data < (u8 *)&(blk->ch_data)) ? 
                              sizeof(CPL_STRUCT) : sizeof(CH_STRUCT);
         } while (blk->u8_data < (u8 *)blk->lfe_ptr);
      }
      ALIGN_DATA_BYTE(SKIP_BYTES1, 1);
   }

AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES1:
   GET_LSB;
   if (DATA_BYTE >= 0x80) {
      frame->tmp_data  = (u32)(DATA_BYTE & 0x7F) << 2;
      ALIGN8_DATA_BYTE(SKIP_BYTES2);
AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES2:
      GET_LSB;
      frame->tmp_data |= (u32)(DATA_BYTE >> 6) & 0x03;
#ifdef AC3_AUDIO_BLOCK_TRACE
      printf("      skip_bytes=%u\n", frame->tmp_data);
#endif
      ALIGN_DATA_BYTE(SKIP_BYTES3, 2);
AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES3:
      while (frame->tmp_data) {
         ALIGN8_DATA_BYTE(SKIP_BYTES4);
AUDIO_BLOCK_read_bit_alloc_SKIP_BYTES4:
         frame->tmp_data--;
      }
   }
   else {
      ALIGN_DATA_BYTE(BIT_ALLOC_DONE, 1);
   }
AUDIO_BLOCK_read_bit_alloc_DONE:
   GET_LSB;

   /* update bap[] values, if necessary */
   BIT_ALLOC_update_bap(&(blk->cpl_data), 
                          blk->lfe_ptr,
                     (u32)blk->new_bai,
                        frame->sample_rate);

   frame->bit_shift      = bit_shift;
   frame->data           = data;
   frame->state          = AUDIO_BLOCK_read_bit_alloc_END_STATE;
   frame->payload_len16 -= (u16)(ptr - frame->payload);
   frame->payload        = ptr;
   return;

AUDIO_BLOCK_read_bit_alloc_END:
   frame->bit_shift     = bit_shift;
   frame->data          = data;
   frame->payload_len16 = 0;
#undef GET_BAI
#undef GET_SNR_INFO
#undef GET_SNR_GAIN1
#undef GET_SNR_GAIN2
#undef GET_COUPLED_FS_LEAK
#undef GET_DELT_INFO
#undef GET_DELTBAE
#undef GET_DELTNSEG
#undef GET_DELTOFFST
#undef GET_DELTLENBA
#undef SKIP_BYTES1
#undef SKIP_BYTES2
#undef SKIP_BYTES3
#undef SKIP_BYTES4
#undef BIT_ALLOC_DONE
}

/* Read the mantissas in the audio block..
      INPUT: frame - Pointer to an AC3_SYNC_FRAME. */
void AUDIO_BLOCK_read_mant(AC3_SYNC_FRAME *frame,
                           AUDIO_BLOCK    *blk)
{
   const u8 *ptr       = frame->payload,
            *end_pos   = frame->payload + frame->payload_len16;
   u8        bit_shift = frame->bit_shift,
             data      = frame->data,
             coeff_bits, u8_tmp;
   AC3_COEFF unit_val;
   u8       *code;
#ifdef AC3_AUDIO_BLOCK_TRACE
   u8  mant_idx, 
       ch_idx = 0;
#endif

#undef  STOP_PARSING
#define STOP_PARSING    goto AUDIO_BLOCK_read_mant_END

#define HIGH_BITS    (AUDIO_BLOCK_read_mant_STATE+1)
#define LOW_BITS     (AUDIO_BLOCK_read_mant_STATE+2)
   if (frame->state > AUDIO_BLOCK_read_mant_STATE) {
      switch (frame->state) {
      case HIGH_BITS:
         goto AUDIO_BLOCK_read_mant_HIGH_BITS;
      case LOW_BITS:
         goto AUDIO_BLOCK_read_mant_LOW_BITS;
      }
   }
#ifdef AC3_AUDIO_BLOCK_TRACE
   printf("    Mantissas:\n");
/*   printf("START> 0x%02hX,%hu (%02hX %02hX)\n", (u16)DATA_BYTE, 
                                                (u16)BIT_SHIFT_BYTE, 
                                                (u16)*ptr, 
                                                (u16)*(ptr+1));*/
#endif
   if (blk->rng == 0)
      blk->rng = (u32)time(NULL) ^ 0x12345678;
   blk->prev_code[0] = blk->prev_code[1] = 
   blk->prev_code[2] = blk->prev_code[3] = 
   blk->prev_code[4] = blk->prev_code[5] = 0xFF;
   blk->new_bai      = blk->dither;    /* store this value for later */
   
   blk->u8_data = &(blk->ch_data[0].num_bins);
   blk->ch      = blk->cpl_data.ch;
   do {
      blk->i          = *(blk->u8_data);
      blk->coeff      = (AC3_COEFF *)(blk->u8_data + offsetof(CH_STRUCT, coeff));
      blk->u8_data   += offsetof(CH_STRUCT, bap);
      frame->tmp_data = sizeof(CH_STRUCT) - offsetof(CH_STRUCT, bap);
#ifdef AC3_AUDIO_BLOCK_TRACE
      printf("      mant[%3hu]", (u16)(ch_idx++));
      mant_idx = 0;
#endif

      do {
AUDIO_BLOCK_read_mant_START:
         if (*(blk->u8_data) == 0) {
            if (blk->dither >= 0x80)
               GET_DITHER_VALUE(*(blk->coeff), blk->rng)
            else
               *(blk->coeff) = 0;
#ifdef AC3_AUDIO_BLOCK_TRACE
            if (blk->dither >= 0x80) {
               printf("[%3hu]=%g**\n               ", 
                      (u16)mant_idx, 
                      (double)*(blk->coeff)/
                      (double)AC3_COEFF_CONST(1.0));
            }
            mant_idx++;
#endif
         }
         else {
            coeff_bits = *(blk->u8_data);
            if (BAP_SYM_QUANT(coeff_bits)) {
               coeff_bits    = BAP_BITS(coeff_bits);
               *(blk->coeff) = (AC3_COEFF)(DATA_BYTE >> (8-coeff_bits));
               switch (*(blk->u8_data)) {
               case 0x43:
                  u8_tmp   = 3;
                  unit_val = AC3_COEFF_CONST(2.0/ 7.0);
                  code     = blk->prev_code + 1;
                  break;
               case 0x44:
                  u8_tmp   = 7;
                  unit_val = AC3_COEFF_CONST(2.0/15.0);
                  code     = blk->prev_code + 1;
                  break;
               case 0x87:
                  u8_tmp   = 5;
                  unit_val = AC3_COEFF_CONST(2.0/11.0);
                  code     = blk->prev_code;
                  if (*code != 0xFF) {
                     coeff_bits     = 0;
                     *(blk->coeff)  = (AC3_COEFF)(*code);
                     *code          = 0xFF;
                  }
                  else {
                     *code          = (u8)(*(blk->coeff) % 11);
                     *(blk->coeff) /= 11;
                  }
                  break;
               case 0xC5:
                  u8_tmp   = 1;
                  unit_val = AC3_COEFF_CONST(2.0/ 3.0);
                  code     = blk->prev_code + 2;
                  if (*code != 0xFF) {
                     coeff_bits     = 0;
                     *(blk->coeff)  = (AC3_COEFF)(*code);
                     *code          = *(code+1);
                     *(code+1)      = 0xFF;
                  }
                  else {
                     *code          = (u8)(*(blk->coeff) %  9) / 3;
                     *(code+1)      = (u8)(*(blk->coeff) %  3);
                     *(blk->coeff) /=  9;
                  }
                  break;
               default: /*case 0xC7:*/
                  u8_tmp   = 2;
                  unit_val = AC3_COEFF_CONST(2.0/ 5.0);
                  code     = blk->prev_code + 4;
                  if (*code != 0xFF) {
                     coeff_bits     = 0;
                     *(blk->coeff)  = (AC3_COEFF)(*code);
                     *code          = *(code+1);
                     *(code+1)      = 0xFF;
                  }
                  else {
                     *code          = (u8)(*(blk->coeff) % 25) / 5;
                     *(code+1)      = (u8)(*(blk->coeff) %  5);
                     *(blk->coeff) /= 25;
                  }
               }
               while (1) {
                  *(blk->coeff) -= (AC3_COEFF)u8_tmp;
                  *(blk->coeff) *= unit_val;
               
                  if ( (*code == 0xFF)||(blk->i == 1)||
                       (*(blk->u8_data) != *(blk->u8_data+1)) )
                     break;
#ifdef AC3_AUDIO_BLOCK_TRACE
                  printf("[%3hu]=%g\n               ", 
                         (u16)(mant_idx++), 
                         (double)*(blk->coeff)/
                         (double)AC3_COEFF_CONST(1.0));
#endif
                  frame->tmp_data--;
                  blk->i--;
                  blk->u8_data++;
                  *(++(blk->coeff)) = (AC3_COEFF)(*code);
                  *code             = *(code+1);
                  *(code+1)         = 0xFF;
               }
            }
            else {
               blk->j = coeff_bits;
               if (coeff_bits > 8) {
                  *(blk->coeff) = (AC3_COEFF)DATA_BYTE << (AC3_COEFF_SIZE-8);
                  ALIGN8_DATA_BYTE(HIGH_BITS);
AUDIO_BLOCK_read_mant_HIGH_BITS:
                  GET_LSB;
                  coeff_bits = blk->j - 8;
               }
               else
                  *(blk->coeff) = 0;
               u8_tmp         = DATA_BYTE >> (8-coeff_bits);
               *(blk->coeff) |= (AC3_COEFF)u8_tmp << (AC3_COEFF_SIZE-blk->j);
            }
            ALIGN_DATA_BYTE(LOW_BITS, coeff_bits);
AUDIO_BLOCK_read_mant_LOW_BITS:
            GET_LSB;
#ifdef AC3_AUDIO_BLOCK_TRACE
            printf("[%3hu]=%g\n               ", 
                   (u16)(mant_idx++), 
                   (double)*(blk->coeff)/
                   (double)AC3_COEFF_CONST(1.0));
#endif
         }
         blk->coeff++;
         blk->u8_data++;
         frame->tmp_data--;
      } while (--(blk->i));

      blk->u8_data += frame->tmp_data;
#ifdef AC3_AUDIO_BLOCK_TRACE
      printf("\n");
#endif
      
      if (blk->ch-- == 0) {
         frame->tmp_data = blk->u8_data - blk->cpl_data.bap;
         blk->u8_data    = blk->cpl_data.bap;
         blk->i          = blk->cpl_data.num_bins;
#ifdef AC3_AUDIO_BLOCK_TRACE
         printf("      mant[CPL]");
         mant_idx = blk->cpl_data.bin_start;
#endif
         goto AUDIO_BLOCK_read_mant_START;
      }
      blk->dither <<= 1;
   } while (blk->u8_data < (u8 *)blk->lfe_ptr);
   
   if ( (blk->u8_data == (u8 *)blk->lfe_ptr)&&(blk->u8_data[0]) ) {
      blk->i          = blk->u8_data[0];
      blk->coeff      = (AC3_COEFF *)(blk->u8_data + offsetof(LFE_STRUCT, coeff));
      blk->u8_data   += offsetof(LFE_STRUCT, bap);
      frame->tmp_data = sizeof(LFE_STRUCT) - offsetof(LFE_STRUCT, bap);
#ifdef AC3_AUDIO_BLOCK_TRACE
      printf("      mant[LFE]");
      mant_idx = 0;
#endif
      goto AUDIO_BLOCK_read_mant_START;
   }

#if (AC3_OUTPUT_CHANNELS == 2)
   /* prevent the following functions from processing 
      unused channels */
   u8_tmp = (frame->mode & AC3_AUDIO_AC_10) ? 3 : 2;
   if (u8_tmp >= frame->num_ch)
      blk->ch      = (frame->num_ch == 3) ? 1 : 0;
   else {
      blk->ch      = 1; /* downmix to 2 channels */
      blk->lfe_ptr = (LFE_STRUCT *)&(blk->ch_data[u8_tmp]);
   }
#endif
   if (blk->cpl_data.ch <= AC3_OUTPUT_CHANNELS) {
#if (AC3_OUTPUT_CHANNELS == 2)
      /* prevent uncouple_channels() function from processing 
         unused channels */
      if ( (u8_tmp == 3)&&(blk->coupled_ch & 0x20) )
         u8_tmp = 2;
      else if (blk->coupled_ch & 0x40)
         u8_tmp = 1;
      else
         u8_tmp = 0;
      blk->ch_data[u8_tmp].cpl_coord[0] |= 0x80;
#endif
      blk->dither = blk->new_bai;    /* restore value */
      uncouple_channels(blk);
   }
   get_coeffs(blk);

   frame->bit_shift      = bit_shift;
   frame->data           = data;
   blk->num             += AC3_OUTPUT_BLK + 1;
   if (blk->num >= (AC3_BLKS_PER_FRAME|AC3_OUTPUT_BLK))
      frame->state = AC3_AUDIO_SKIP_AUX_BYTES;
   else
      frame->state = AUDIO_BLOCK_read_config_STATE;
   frame->payload_len16 -= (u16)(ptr - frame->payload);
   frame->payload        = ptr;
   return;

AUDIO_BLOCK_read_mant_END:
   frame->bit_shift     = bit_shift;
   frame->data          = data;
   frame->payload_len16 = 0;
#undef HIGH_BITS
#undef LOW_BITS
}

#undef DATA_BYTE
#undef BIT_SHIFT_BYTE
#undef DATA_PTR
#undef END_DATA_PTR
#undef PARSER_STATE
#undef STOP_PARSING

