/***************************************************************************
                         bit_alloc.h  -  description
                             -------------------
   Functions for calculating the bit allocation values.
                             -------------------
    begin                : Wed Sep 28 2005
    copyright            : (C) 2005 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*#define BIT_ALLOC_TRACE 1*/
#ifdef BIT_ALLOC_TRACE
# include <stdio.h>
#endif
#include <string.h>
#include <stdlib.h>

#include "AC3_tables.h"
#include "bit_alloc.h"


#define LOGADD(bndpsd, exp, s16v) \
   if (bndpsd > exp) {\
      s16v   = bndpsd - exp; \
      bndpsd = exp; \
   }\
   else \
      s16v   = exp - bndpsd; \
   if (s16v < (LATAB_SIZE << 1)) \
      bndpsd -= (s16)LATAB[s16v >> 1]

#define UPDATE_LEAK(leak, decay, gain, bndpsd, s16v) \
   leak += decay; \
   s16v  = bndpsd + gain; \
   if (leak > s16v) \
      leak = s16v

#define EXCITE_TO_MASK(excite, bndpsd, hth, mask) \
   if (bndpsd > dbknee) \
      excite -= (bndpsd - dbknee) >> 2; \
   mask = (excite <= hth) ? excite : hth

#define LOWCOMP_DEC(dec) \
{\
   lowcomp -= dec; \
   if (lowcomp < 0) \
      lowcomp = 0; \
}
#define LOWCOMP(bndpsd, max, s16v) \
{\
   s16v = *bndpsd - *(bndpsd+1); \
   if (s16v < 0) \
      LOWCOMP_DEC(64) \
   else if (s16v == 256) \
      lowcomp = max; \
}


/* Update values in the bap[] array.  The bap[] values are only
    calculated for the channels that need updating.
      INPUT: cpl     - Pointer to the coupling channel info.  This 
                        function assumes there is at least one
                        CH_STRUCT between cpl and lfe.
             lfe     - Pointer to the low frequency enhancement
                        channel info.
             new_bai - Set when new bit allocation info was read
                        into the channel info structures.
             srate   - Sampling rate (Hz). */
void BIT_ALLOC_update_bap(CPL_STRUCT *cpl, LFE_STRUCT *lfe, 
                          u32 new_bai, u32 srate)
{
   CH_STRUCT *ch = (CH_STRUCT *)(cpl + 1);
   u8        *delt, *start_exp, *end_exp, *bap, 
             *u8_ptr1, *u8_ptr2, *u8_ptr3;
   s16       *s16_ptr1, *s16_ptr2, *s16_ptr3;
   s16        tmp_buf[61], s16_tmp1, s16_tmp2,
              sgain, fgain, dbknee, mfloor, csnroffset,
              fleak, sleak, lowcomp, snroffset;
   u8         end, start, mpe, sdecay, fdecay;
#ifdef BIT_ALLOC_TRACE
   u8         ch_idx = 0xFF;
#endif

#define INIT_VARS(ptr, delt_val, start_val) \
   delt      = delt_val; \
   start_exp = ptr->exp; \
   end_exp   = ptr->exp + ptr->num_bins; \
   bap       = ptr->bap; \
   end       = ptr->num_bins; \
   start     = start_val; \
   fgain     = ((s16)(ptr->snr_gain&0x0E)<<6) + 0x0080; \
   snroffset =  (s16)(ptr->snr_gain&0xF0)>>2; \
   mpe       = ptr->mant_per_exp; \
   ptr->mant_per_exp &= 0x07

   switch (srate) {
   case 48000:
      srate = 0;
      break;
   case 44100:
      srate = HTHTAB_SIZE;
      break;
   case 32000:
   default:
      srate = HTHTAB_SIZE * 2;
   }

   sdecay      = cpl->sdecay;
   fdecay      = cpl->fdecay;
   sgain       = SLOWGAIN[cpl->bai>>5];
   dbknee      = DBPBTAB[(cpl->bai>>3) & 0x03];
   mfloor      = FLOORTAB[cpl->bai     & 0x07];
   csnroffset  = (15 - (s16)cpl->csnroffst) << 6;
   csnroffset += mfloor;
   mfloor    >>= 5;

   if ( (!cpl->num_bins)||
        ((!new_bai)&&(cpl->mant_per_exp <= 0x08)) ) {
#ifdef BIT_ALLOC_TRACE
      ch_idx++;
#endif
      goto INIT_CH_VARS;   /* no need to update coupling channel */
   }
   else {
      INIT_VARS(cpl, cpl->delt, cpl->bin_start);
      fleak = (s16)(cpl->fs_leak & 0xF0) << 4;
      sleak = (s16)(cpl->fs_leak & 0x0F) << 8;
      ch--;
      goto START_UPDATING;
   }
   do {
#ifdef BIT_ALLOC_TRACE
      ch_idx++;
#endif
      if (++ch < (CH_STRUCT *)lfe) {
INIT_CH_VARS:
         if ( (!new_bai)&&(ch->mant_per_exp <= 0x08) )
            continue;   /* no need to update this channel */
         INIT_VARS(ch, ch->delt, 0);
      }
      else {
         if ( (!lfe->num_bins)||
              ((!new_bai)&&(lfe->mant_per_exp <= 0x08)) )
            break;      /* no need to update lfe channel */
         INIT_VARS(lfe, (u8 *)(tmp_buf+60), 0);
         *delt = AC3_DELTBAE_NONE;
         ch    = NULL;
      }

START_UPDATING:
      /* expand packed exponents, if necessary */
#define EXP       u8_ptr1
#define TMP_EXP   u8_ptr2
      if (mpe > 0x20) {
         TMP_EXP = (u8 *)tmp_buf;
         EXP     = start_exp;
         if (!start)
            EXP++;
         if (mpe > 0x40) {
            memcpy(TMP_EXP, EXP, (size_t)(end>>2));
            do {
               *(EXP++) =  *TMP_EXP;
               *(EXP++) =  *TMP_EXP;
               *(EXP++) =  *TMP_EXP;
               *(EXP++) = *(TMP_EXP++);
            } while (EXP < end_exp);
         }
         else {
            memcpy(TMP_EXP, EXP, (size_t)(end>>1));
            do {
               *(EXP++) =  *TMP_EXP;
               *(EXP++) = *(TMP_EXP++);
            } while (EXP < end_exp);
         }
      }
#undef EXP
#undef TMP_EXP

      /* get values for the bndpsd[] array */
#define EXP       u8_ptr1
#define LAST_EXP  u8_ptr2
#define BND       u8_ptr3
#define BNDPSD    s16_ptr1
#define s16v1     s16_tmp1
#define s16v2     s16_tmp2
      {
         EXP      = start_exp;
         LAST_EXP = start_exp - start;
         BND      = (u8 *)BNDTAB + MASKTAB[start];
         BNDPSD   = tmp_buf;
         while (1) {
            LAST_EXP += *BND;
            if (LAST_EXP > end_exp)
               LAST_EXP = end_exp;
            *BNDPSD = (s16)*EXP << 7;
            while (++EXP < LAST_EXP) {
               s16v1 = (s16)*EXP << 7;
               LOGADD(*BNDPSD, s16v1, s16v2);
            }
            if (LAST_EXP == end_exp)
               break;
            BNDPSD++;
            LAST_EXP -= *(BND++);
         }
         if (!ch)
            *(BNDPSD+1) = *BNDPSD;
      }
#undef EXP
#undef LAST_EXP
#undef BND
#undef BNDPSD
#undef s16v1
#undef s16v2
   
      /* get values for the mask[] array */
#define BNDPSD       s16_ptr1
#define LAST_BNDPSD  s16_ptr2
#define HTH          s16_ptr3
#define idx          mpe
#define s16v         s16_tmp1
#ifdef BIT_ALLOC_TRACE
      printf("mask[%3hu]", (u16)ch_idx);
      idx = 0;
#endif
      {
         end         += start - 1;
         BNDPSD       = LAST_BNDPSD = tmp_buf;
         HTH          = (s16 *)HTHTAB + srate;
         LAST_BNDPSD += (end < MASKTAB_SIZE) ? MASKTAB[end] : (BNDTAB_SIZE-1);
         if (start) {
            HTH         += MASKTAB[start];
            LAST_BNDPSD -= MASKTAB[start];
            do {
               UPDATE_LEAK(fleak, (s16)fdecay, fgain, *BNDPSD, s16v);
               UPDATE_LEAK(sleak, (s16)sdecay, sgain, *BNDPSD, s16v);
               s16v = (fleak < sleak) ? fleak : sleak;
#if (BIT_ALLOC_TRACE > 1)
               s16_tmp2 = *BNDPSD;
               printf("  fleak=%4hd sleak=%4hd\n         ", fleak, sleak);
#endif
               EXCITE_TO_MASK(s16v, *BNDPSD, *HTH, *BNDPSD);
#ifdef BIT_ALLOC_TRACE
# if (BIT_ALLOC_TRACE > 1)
               printf("[%3hu]=%4hd  %4hd hth=%4hd\n         ", 
                      (u16)(idx++), *BNDPSD, s16_tmp2, *HTH);
# else
               printf("[%3hu]=%4hd\n         ", 
                      (u16)(idx++), *BNDPSD);
# endif
#endif
               HTH++;
            } while (++BNDPSD <= LAST_BNDPSD);
         }
         else {
            idx     = 0;
            lowcomp = 0;
            do {
               fleak = *BNDPSD + fgain;
               s16v  = *BNDPSD - *(BNDPSD+1);
               if (s16v < 0)
                  LOWCOMP_DEC(64)
               else {
                  if (s16v == 256)
                     lowcomp = 384;
                  if (idx > 1)
                     idx |= 0x40;
               }
               if (idx >= 6) 
                  sleak = *BNDPSD + sgain;
               s16v = fleak + lowcomp;
#if (BIT_ALLOC_TRACE > 1)
               s16_tmp2 = *BNDPSD;
               printf("  lc=%4d\n         ", lowcomp);
#endif
               EXCITE_TO_MASK(s16v, *BNDPSD, *HTH, *BNDPSD);
#ifdef BIT_ALLOC_TRACE
# if (BIT_ALLOC_TRACE > 1)
               printf("[%3hX]=%4hd  %4hd hth=%4hd\n         ", 
                      (u16)idx, *BNDPSD, s16_tmp2, *HTH);
# else
               printf("[%3hX]=%4hd\n         ", 
                      (u16)idx, *BNDPSD);
# endif
#endif
               HTH++;
               BNDPSD++;
            } while (++idx < 7);
            idx &= 0x07;
            while (BNDPSD <= LAST_BNDPSD) {
               UPDATE_LEAK(fleak, (s16)fdecay, fgain, *BNDPSD, s16v);
               UPDATE_LEAK(sleak, (s16)sdecay, sgain, *BNDPSD, s16v);
               if (idx >= 22)
                  s16v = fleak;
               else {
                  if (idx++ < 20)
                     LOWCOMP(BNDPSD, ((idx <= 7)?384:320), s16v)
                  else
                     LOWCOMP_DEC(128)
                  s16v = fleak + lowcomp;
               }
               if (s16v > sleak)
                  s16v = sleak;
#if (BIT_ALLOC_TRACE > 1)
               s16_tmp2 = *BNDPSD;
               printf("  lc=%4d fleak=%4hd sleak=%4hd\n         ", 
                      lowcomp, fleak, sleak);
#endif
               EXCITE_TO_MASK(s16v, *BNDPSD, *HTH, *BNDPSD);
#ifdef BIT_ALLOC_TRACE
# if (BIT_ALLOC_TRACE > 1)
               printf("[%3hu]=%4hd  %4hd hth=%4hd\n         ", 
                      (u16)(idx-1), *BNDPSD, s16_tmp2, *HTH);
# else
               printf("[%3hu]=%4hd\n         ", 
                      (u16)(idx-1), *BNDPSD);
# endif
               if (idx >= 22)
                  idx++;
#endif
               HTH++;
               BNDPSD++;
            }
         }
      }
#ifdef BIT_ALLOC_TRACE
      printf("\n");
#endif
#undef BNDPSD
#undef LAST_BNDPSD
#undef HTH
#undef idx
#undef s16v

      /* alter mask[] array, if delt[] is present */
#define END_DELT     u8_ptr3
#define MASK         s16_ptr1
#define END_MASK_BUF s16_ptr2
#define END_MASK     s16_ptr3
#define s16v         s16_tmp1
      if ( (AC3_DELTBAE(delt) <= AC3_DELTBAE_NEW)&&
           (2 <= AC3_DELTNSEG(delt))&&(AC3_DELTNSEG(delt) <= 16) ) {
#ifdef BIT_ALLOC_TRACE
         s16_tmp2 = 0;
#endif
         END_DELT = delt + AC3_DELTNSEG(delt);
         MASK     = tmp_buf;
         do {
            MASK    += *(++delt);
            if (MASK >= END_MASK_BUF) {
#ifdef BIT_ALLOC_TRACE
               printf("DELTOFFST[%hd] too large (%u+%u)\n", s16_tmp2,
                      (u32)(END_MASK_BUF-tmp_buf), (u32)(MASK-END_MASK_BUF));
#endif
               break;
            }
            END_MASK = MASK + (*(++delt) >> 4);
            if (END_MASK > END_MASK_BUF) {
#ifdef BIT_ALLOC_TRACE
               printf("DELTLEN[%hd] too large (%u+%u)\n", s16_tmp2,
                      (u32)(END_MASK_BUF-tmp_buf), (u32)(END_MASK-END_MASK_BUF));
#endif
               END_MASK = END_MASK_BUF;
            }
            s16v     = (s16)(*delt & 0x0F);
            s16v    -= (s16v < 8) ? 8 : 6;
            s16v   <<= 6;
            while (MASK < END_MASK) {
               *(MASK++) -= s16v;
            }
#ifdef BIT_ALLOC_TRACE
         s16_tmp2++;
#endif
         } while (delt < END_DELT);
      }
#ifdef BIT_ALLOC_TRACE
      else if (AC3_DELTBAE(delt) <= AC3_DELTBAE_NEW)
         printf("DELTNSEG out of bounds (%hu)\n", AC3_DELTNSEG(delt));
#endif
#undef END_DELT
#undef MASK
#undef END_MASK_BUF
#undef END_MASK
#undef s16v

      /* get values for the bap[] array */
#define EXP       u8_ptr1
#define LAST_EXP  u8_ptr2
#define BND       u8_ptr3
#define MASK      s16_ptr1
#define s16v1     s16_tmp1
#define s16v2     s16_tmp2
#ifdef BIT_ALLOC_TRACE
      printf("bap[%3hu]", (u16)ch_idx);
      mpe = start;
#endif
      {
         snroffset = csnroffset - snroffset;
         EXP       = start_exp;
         LAST_EXP  = start_exp - start;
         BND       = (u8 *)BNDTAB + MASKTAB[start];
         MASK      = tmp_buf;
         while (1) {
            LAST_EXP += *BND;
            if (LAST_EXP > end_exp)
               LAST_EXP = end_exp;
            s16v1 = mfloor;
            s16v2 = snroffset - *MASK;
            if (s16v2 >= 0x20)
               s16v1 -= s16v2 >> 5;
            do {
               s16v2 = s16v1 - ((s16)*EXP << 2);
               if (s16v2 <= 0)
                  *bap = 0;
               else if (s16v2 < BAPTAB_SIZE)
                  *bap = BAPTAB[s16v2];
               else
                  *bap = 16;
#ifdef BIT_ALLOC_TRACE
# if (BIT_ALLOC_TRACE > 1)
               printf("[%3hu]=%2hu %hu "
                      "(snroffset=%hd floor=%hd -mask=%hd bapidx=%hd)"
                      "\n        ",
                      (u16)(mpe++),
                      (u16)BAP_BITS(*bap),
                      (u16)BAP_GROUP_SIZE(*bap), 
                      snroffset, mfloor, s16v1, s16v2);
# else
               printf("[%3hu]=%2hu %hu\n        ",
                      (u16)(mpe++),
                      (u16)BAP_BITS(*bap),
                      (u16)BAP_GROUP_SIZE(*bap));
# endif
#endif
               bap++;
            } while (++EXP < LAST_EXP);
            if (LAST_EXP == end_exp)
               break;
            MASK++;
            LAST_EXP -= *(BND++);
         }
      }
#ifdef BIT_ALLOC_TRACE
      printf("\n");
#endif
#undef EXP
#undef LAST_EXP
#undef BND
#undef MASK
#undef s16v1
#undef s16v2
   } while (ch);
#undef INIT_VARS
}

#undef LOGADD
#undef UPDATE_LEAK
#undef EXCITE_TO_MASK
#undef LOWCOMP_DEC
#undef LOWCOMP

