/***************************************************************************
                          ADAPT_FIELD.h  -  description
                             -------------------
   Stuff needed for handling ADAPTATION FIELDs.
                             -------------------
    begin                : Fri Jul 23 2004
    copyright            : (C) 2004 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __ADAPT_FIELD_H__
#define __ADAPT_FIELD_H__

#include "common/typedefs.h"


#define ADAPT_FIELD_DISCONTINUITY_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x80)
#define ADAPT_FIELD_RANDOM_ACCESS_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x40)
#define ADAPT_FIELD_ELEM_STR_PRIORITY_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x20)
#define ADAPT_FIELD_PCR_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x10)
#define ADAPT_FIELD_OPCR_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x08)
#define ADAPT_FIELD_SPLICING_POINT_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x04)
#define ADAPT_FIELD_PRIV_DATA_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x02)
#define ADAPT_FIELD_EXT_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->flags & 0x01)

#define ADAPT_FIELD_LTW_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->ext_flags & 0x80)
#define ADAPT_FIELD_PIECEWISE_RATE_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->ext_flags & 0x40)
#define ADAPT_FIELD_SEAMLESS_SPLICE_FLAG(ptr_to_ADAPT_FIELD) \
   (ptr_to_ADAPT_FIELD->ext_flags & 0x20)

#define ADAPT_FIELD_MAX_SIZE           184   
#define ADAPT_FIELD_MAX_PRIV_DATA_LEN  180

#ifdef __cplusplus
extern "C" {
#endif
            
typedef struct {
   u8  adapt_field_len,         /*  8: Number of bytes following this byte.
         <0-182> adaptation_field_control == '11', whatever is needed to fill the TS_PKT
           <183> adaptation_field_control == '10' */
       flags,
/*
      1   discontinuity_flag           Set to indicate a system time-base, or
                                         continuity_counter discontinuity.
                                         A system time-base discontinuity
                                         occurs when the PID is designated as
                                         a PCR_PID. If the PID is not a
                                         PCR_PID, then the discontinuity is in
                                         the continuity_counter. This bit can
                                         stay set for consecutive packets,
                                         until the discontinuity has passed.
      1   random_access_flag           Set when the current and future packets
                                         with equal PID contain info to aid
                                         random access at this point (you can
                                         start playing video, or audio, from
                                         here).
      1   elem_str_priority_flag       Set when the payload of this packet has
                                         a higher priority than the payloads
                                         of other packets, with equal PID, and
                                         elem_str_priority_indicator is 0.
                                         For video streams, this bit is set
                                         when the payload contains bytes from
                                         an intra-coded slice.
      1   PCR_flag                     Set when this adaptation_field contains
                                         the PCR.
      1   OPCR_flag                    Set when this adaptation_field contains
                                         the OPCR.
      1   splicing_point_flag          Set when this adaptation_field contains
                                         a splicing point.
      1   priv_data_flag               Set when this adaptation_field conatins
                                         private data.
      1   adapt_field_ext_flag         Set when this adaptation_field contains
                                         the adaptation field extension
                                         section.
*/
       priv_data_len,           /*  8: Number of private data bytes. Value
                                         must be small enough to fit in this
                                         packet. */
       adapt_field_ext_len;     /*  8: Number of extension bytes. Value must
                                         be small enough to fit in this 
                                         packet. Value includes reserved
                                         bytes (if any). */
                                         
   s8  splice_countdown,        /*  8: Number of packets with equal PID,
                                         excluding duplicates and packets
                                         with no payload, before a splicing
                                         point occurs. Positive values
                                         indicate the point is in the future,
                                         negative values indicate the point
                                         is in the past. A value of 0 is the
                                         splicing point, which must be an
                                         acces point (random_acces_point=1). */
       ext_flags,
/*
      1   ltw_flag                     Set when this extension contains a
                                         legal time window.
      1   piecewise_rate_flag          Set when this extension contains a
                                         piecewise rate.
      1   seamless_splice_flag         Set when this extension contains the
                                         splice_type, and DTS_next_AU values.
                                         Only set if splicing_point_flag = 1,
                                         and splice_countdown is >= 0.
      5   reserved
*/
       ltw_valid_flag,           /*  1: Set when the ltw_offset value is valid.
                                         When not set, the ltw_offset value is
                                         undefined. */
       splice_type;              /*  4: This value must be the same in all
                                         consecutive packets, with equal PID,
                                         and splice_countdown >= 0.
        <'0000'> Audio streams always use this value
 <'0000'-'0011'> Video streams us the tables on pg31(51)
 <'0100'-'1011'> reserved
 <'1100'-'1111'> user defined */
 
   u16 ltw_offset;              /* 15: Duration of the Legal Time Window, in
                                         units of (300/sys_clk_freq) secs. */
                                         
   u32 piecewise_rate;          /* 22: Defined when ltw_flag and
                                         ltw_valid_flag are set. This value
                                         is used for future packets, with
                                         equal PID, and no valid ltw_offset
                                         value. */
                                         
   s64 PCR,                     /*     Intended time of arrival of the byte
                                         containing the last bit of
                                         prog_clk_ref_base at the input
                                         of the decoder. */
       OPCR,                    /*     This optional clock reference is only
                                         present in packets that contain PCR.
                                         OPCR assists in the reconstruction of
                                         a single program TS from another TS. */
       DTS_next_AU;             /*     This value must be the same in all
                                         consectutive packets, with equal PID,
                                         and splice_countdown >= 0. Indicates
                                         the decoding time of the first
                                         access unit following the splicing
                                         point. */
                                         
   u8  priv_data[ADAPT_FIELD_MAX_PRIV_DATA_LEN];
                                /*     User defined. ATSC streams may use
                                         this space to send private data. */
} ADAPT_FIELD;


/* Reads the first ADAPTATION FIELD, in data_stream, into af.
      INPUT: data_stream - Points to the first byte of the ADAPTATION FIELD.
                             This function assumes there are at most
                             ADAPT_FIELD_MAX_SIZE bytes. This function also
                             assumes the first byte (adapt_field_len) is
                             non-zero.
      OUTPUT: af - Points to the ADAPT_FIELD that will hold data taken from
                     data_stream.
      RETURN: Success - A non-zero value
              Error   - 0 */
int ADAPT_FIELD_read(const u8 *data_stream, ADAPT_FIELD *af);

#ifdef __cplusplus
};
#endif

#endif
