/***************************************************************************
                        detect_mpeg.c  -  description
                             -------------------
    begin                : Sat Feb 5 2005
    copyright            : (C) 2005 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <string.h>

#include "detect_mpeg.h"
#include "TS_PKT.h"
#include "PES_PKT.h"
#include "PMT_SECTION.h"
#ifndef _NO_VIDEO_
# include "vc1_video/wmv.h"
#endif


/* Tries to determine the type of mpeg data in the data stream.
      INPUT: data_stream - Points to the data to look through.
             data_len    - Number of bytes in data_stream.
      RETURN: One of the MPEG_TYPE_* macros. */
int detect_mpeg(const u8 *data_stream, u32 data_len)
{
   const u8        *ptr, *last;
   u32              i, bf;
   union {
#ifndef _NO_AUDIO_
      AC3_SYNC_FRAME   ac3;
      MPEG_AUDIO_FRAME mpeg;
#endif
#ifndef _NO_VIDEO_
      WMV_DATA         wmv;
#endif
      int              unused;
   } frame;

   /* check for TS */
   ptr  =
   last = data_stream;
   i    = 0;
   goto START_TS_SEARCH;
   do {
      ptr++;
      if ((ptr - last) == TS_PKT_SIZE) {
         i++;
         if (i >= 4)
            return MPEG_TYPE_TS;
      }
      last = ptr;
START_TS_SEARCH:
      ptr  = TS_PKT_find(ptr, (data_len - (u32)(ptr - data_stream)));
   } while (ptr);

   /* check for PS or ES video data */
   ptr = data_stream;
   while (1) {
      bf  = 0;
      ptr = detect_start_code(ptr, (data_len - (u32)(ptr-data_stream)), &bf);
      if (ptr) {
         if (*ptr == 0xB3) {
            bf   = 0;
            last = detect_start_code(ptr, (data_len - (u32)(ptr-data_stream)), &bf);
            if (last) {
               if (*last == 0xB5) {
                  if ((data_len - (u32)(last-data_stream)) > 1) {
                     last++;
                     if ((*last & 0xF0) == 0x10)
                        return MPEG_TYPE_ES_V2;
                  }
               }
               else
                  return MPEG_TYPE_ES_V1;
               ptr = last;
            }
         }
         else if (*ptr == 0xBA)
            return MPEG_TYPE_PS;
      }
      else
         break;
   }

#ifndef _NO_AUDIO_   
   /* check for MPEG AUDIO data */
   frame.mpeg.state = MPEG_AUDIO_next_frame_STATE;
   bf = MPEG_AUDIO_next_frame(data_stream, data_len, &(frame.mpeg));
   if ( (frame.mpeg.state == MPEG_AUDIO_next_frame_END_STATE)&&
        (frame.mpeg.layer == MPEG_AUDIO_LAYER2_BYTE)&&
        (frame.mpeg.frame_len <= (data_len-bf)) ) {
      ptr = frame.mpeg.payload + frame.mpeg.frame_len;
      if ( ( ( *(ptr-4) == MPEG_AUDIO_SYNC_BYTE)&&
             ((*(ptr-3)|0x01) == frame.mpeg.layer) )||
           ( ( *(ptr-2) == MPEG_AUDIO_SYNC_BYTE)&&
             ((*(ptr-1)|0x01) == frame.mpeg.layer) ) )
         return MPEG_TYPE_ES_A1;
   }
   
   /* check for AC3 AUDIO data */
   frame.ac3.state = AC3_AUDIO_next_frame_STATE;
   bf = AC3_AUDIO_next_frame(data_stream, data_len, &(frame.ac3));
   if ( (frame.ac3.state == AC3_AUDIO_next_frame_END_STATE)&&
        (frame.ac3.frame_len <= (data_len-bf-2)) ) {
      ptr = frame.ac3.payload + frame.ac3.frame_len;
      if ( (*ptr     == AC3_AUDIO_SYNC_BYTE1)&&
           (*(ptr+1) == AC3_AUDIO_SYNC_BYTE2) )
         return MPEG_TYPE_ES_AC3;
   }
#endif

#ifndef _NO_VIDEO_
   /* check for WMV data */
   frame.wmv.payload     = data_stream;
   frame.wmv.payload_len = data_len;
   frame.wmv.state       = WMV_next_guid_STATE;
   WMV_next_guid(&(frame.wmv));
   if (frame.wmv.state == WMV_GUID_HEADER)
      return MPEG_TYPE_ES_WMV;
#endif
   return MPEG_TYPE_UNKNOWN;
}

/* Find the first start code in the given data stream */
const u8 *detect_start_code(const u8 *data_stream,
                            u32       data_len,
                            u32      *bytes_found)
{
   const
   u8  *ptr = data_stream;
   u32  len = data_len;

   if (!(*bytes_found)) {
      while (1) {
START_SEARCH:
         ptr = memchr(ptr, 0, (size_t)len);
         if (!ptr)
            break;

         ptr++;
         len = data_len - (u32)(ptr - data_stream);
FOUND_FIRST_BYTE:
         if (len >= 3) {
            if ( (!ptr[0])&&(ptr[1] == 0x01) )
               return (ptr + 2);
         }
         else {
            if (len == 2) {
               if ( (!ptr[0])&&(ptr[1] == 0x01) )
                  *bytes_found = 3;
            }
            else if (!len)
               *bytes_found = 1;
            else if (!ptr[0])
               *bytes_found = 2;
            break;
         }
      }
   }
   else if (*bytes_found == 1) {
      *bytes_found = 0;
      goto FOUND_FIRST_BYTE;
   }
   else if (*bytes_found == 2) {
FOUND_FIRST_TWO_BYTES:
      if (*ptr == 0x01) {
         if (len >= 2)
            return (ptr + 1);
         *bytes_found = 3;
      }
      else if (!(*ptr)) {
         if (len >= 2) {
            ptr++;
            len--;
            goto FOUND_FIRST_TWO_BYTES;
         }
      }
      else {
         *bytes_found = 0;
         goto START_SEARCH;
      }
   }
   else
      return ptr;
   return NULL;
}


/* Processes the packet handled by the module.
      INPUT: void * - Pointer to the memory provided for the module.
             void * - Pointer to the PES_PKT that needs handling.
                        Set pes->stream_id to zero to let the PES
                        module know to expect the start of a new
                        PES_PKT. */
void DETECT_MODULE_process_pkt(void *mem, void *pes)
{
   const u8           *ptr;
   DETECT_MODULE_data *data = (DETECT_MODULE_data *)mem;
   PES_PKT            *pkt  = (PES_PKT *)pes;

   while (pkt->payload_len16) {
      switch (data->state) {
      case 0:
         /* first try looking for MPEG video data */
         ptr = detect_start_code(pkt->payload,
                                 (u32)pkt->payload_len16,
                                 &(data->bytes_found));
         if (!ptr) {
#ifndef _NO_AUDIO_
            /* now try looking for MPEG audio data */
            MPEG_AUDIO_next_frame(pkt->payload,
                                  (u32)pkt->payload_len16,
                                  &(data->audio));
            if ( (data->audio.state == MPEG_AUDIO_next_frame_END_STATE)&&
                 (data->audio.frame_len > 4) ) {
               if (data->audio.layer == MPEG_AUDIO_LAYER2_BYTE) {
                  /* there might be some MPEG audio data here */
                  data->audio.frame_len -= 4;
                  if ((u32)data->audio.payload_len16 > data->audio.frame_len) {
                     ptr = data->audio.payload + data->audio.frame_len;
                     data->state = 4;
                  }
                  else {
                     data->audio.frame_len -= (u32)data->audio.payload_len16;
                     data->state = 3;
                  }
               }
               data->audio.state = MPEG_AUDIO_next_frame_STATE;
               data->bytes_found = 0;
            }
            
            if (!data->state) {
               /* now try looking for AC3 audio data */
               AC3_AUDIO_next_frame(pkt->payload, 
                                    (u32)pkt->payload_len16,
                                    &(data->ac3));
               if (data->ac3.state == AC3_AUDIO_next_frame_END_STATE) {
                  /* there might be some AC3 audio data here */
                  if ((u32)data->ac3.payload_len16 > data->ac3.frame_len) {
                     ptr = data->ac3.payload + data->ac3.frame_len;
                     data->state = 7;
                  }
                  else {
                     data->ac3.frame_len -= (u32)data->ac3.payload_len16;
                     data->state = 6;
                  }
                  data->ac3.state   = AC3_AUDIO_next_frame_STATE;
                  data->bytes_found = 0;
               }
            }
            
            if (!ptr)
#endif
               ptr = pkt->payload + pkt->payload_len16;
         }
         else {
            data->bytes_found = 0;
            if (*ptr == 0xB3)
               data->state++;    /* there might be some MPEG video data here */
         }
         break;

      case 1:
         /* continue looking for MPEG video data */
         ptr = detect_start_code(pkt->payload,
                                 (u32)pkt->payload_len16,
                                 &(data->bytes_found));
         if (!ptr)
            ptr = pkt->payload + pkt->payload_len16;
         else {
            data->bytes_found = 0;
            if (*ptr == 0xB5)
               data->state++;    /* this might be an MPEG2 video stream */
            else {
               data->state       = 0;
               data->stream_type = STREAM_TYPE_MPEG1_VIDEO;
               return;           /* found an MPEG1 video stream! */
            }
         }
         break;

      case 2:
         /* continue looking for an MPEG2 video stream */
         ptr = pkt->payload;
         if ((data->bytes_found + (u32)pkt->payload_len16) >= 2) {
            ptr        += 1 - data->bytes_found;
            data->state = data->bytes_found = 0;
            if ((*ptr & 0xF0) == 0x10) {
               data->stream_type = STREAM_TYPE_MPEG2_VIDEO;
               return;           /* found an MPEG2 video stream! */
            }
         }
         else {
            data->bytes_found += (u32)pkt->payload_len16;
            ptr               += pkt->payload_len16;
         }
#ifndef _NO_AUDIO_
         break;

      case 3:
         /* skip over MPEG audio frame data */
         ptr = pkt->payload;
         if ((u32)pkt->payload_len16 > data->audio.frame_len) {
            ptr        += data->audio.frame_len;
            data->state = 4;
         }
         else {
            ptr        += pkt->payload_len16;            
            data->audio.frame_len -= (u32)pkt->payload_len16;
         }
         break;

      case 4:
         /* look for the start of the next MPEG audio frame */
         ptr = pkt->payload;
         if (*ptr == MPEG_AUDIO_SYNC_BYTE)
            data->state++;
         else if (data->bytes_found >= 2) {
            data->state       = 0;
            data->bytes_found = 0;
         }
         else
            data->bytes_found++;
         ptr++;
         break;

      case 5:
         /* make sure this MPEG audio frame is the same as the last one */
         ptr = pkt->payload;
         if ((*ptr|0x01) == data->audio.layer) {
            data->state       = 0;
            data->bytes_found = 0;
            data->stream_type = STREAM_TYPE_MPEG1_AUDIO;
            return;  /* found an MPEG1 audio stream! */
         }
         else if ( (*ptr != MPEG_AUDIO_SYNC_BYTE)||
                   (data->bytes_found >= 2) ) {
            data->state       = 0;
            data->bytes_found = 0;
         }
         else
            data->bytes_found++;
         ptr++;
         break;

      case 6:
         /* skip over AC3 audio frame data */
         ptr = pkt->payload;
         if ((u32)pkt->payload_len16 > data->ac3.frame_len) {
            ptr        += data->ac3.frame_len;
            data->state = 7;
         }
         else {
            ptr        += pkt->payload_len16;            
            data->ac3.frame_len -= (u32)pkt->payload_len16;
         }
         break;
      
      case 7:
         /* look for the start of the next AC3 audio frame */
         ptr = pkt->payload;
         if ( (!data->bytes_found)&&(*ptr == AC3_AUDIO_SYNC_BYTE1) )
            data->bytes_found++;
         else if ( (data->bytes_found)&&(*ptr == AC3_AUDIO_SYNC_BYTE2) ) {
            data->state       = 0;
            data->bytes_found = 0;
            data->stream_type = STREAM_TYPE_ATSC_AC3_AUDIO;
            return;  /* found an AC3 audio stream! */
         }
         else {
            data->state       = 0;
            data->bytes_found = 0;
         }
         ptr++;
#endif
      }

      pkt->payload_len16 -= (u16)(ptr - pkt->payload);
      pkt->payload        = ptr;
   }
}

