/***************************************************************************
                          mpeg_es.c  -  description
                             -------------------
   Functions for handling mpeg elementary streams.
                             -------------------
    begin                : Fri Feb 4 2005
    copyright            : (C) 2005 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "common/Timer.h"
#include "mpeg_es.h"
#include "commands.h"
#include "MPEG2_SYS.h"
#include "PES_PKT.h"
#include "PSI_module.h"
#include "PMT_module.h"
#ifndef _SPEED_TEST_
# include "AVindex.h"
#endif


#if (BUFFER_SIZE > 0x0000FFFF)
# define ES_BUFFER_SIZE    0x0000FFFF
#else
# define ES_BUFFER_SIZE    BUFFER_SIZE
#endif

/* Plays the elementary stream contained in the given file.
      INPUT: fp          - Points to the FILE with the elementary stream.
             stream_type - See PMT_SECTION.h for STREAM_TYPE_* values.
      RETURN: Success - A non-zero value
              Failure - 0 */
int MPEG_ES_Play(FILE_PTR fp, u8 stream_type)
{
   u8                              *mod_mem;
   MPEG2_PSI_PES_MODULE_process_pkt mod_process;

   PES_PKT         pkt;
#ifndef _SPEED_TEST_
   FILE_POS        f_pos, next_index_pt;
#endif
   FILE_BYTE_COUNT bytes_read;
   int             idx;
#ifdef _RECORD_STATS_
   TIME            t1, t2;
   
   sys.stream_bytes = sys.pkt_cnt = 0;
   sys.decode_time  = 0.0;
#endif
   pkt.flags1 = pkt.flags2 = 0;
   
   idx = MPEG_ES_find_module(stream_type);
   if (idx < 0)
      return 0;

   mod_mem     = sys.psi_pes[idx].common.mem;
   mod_process = sys.psi_pes[idx].process_pkt;

   sys.data.a_pos  = sys.data.v_pos  = 0;
   sys.data.a_skip = sys.data.v_skip = 0;
/*   printf("\nA:%8d   V:%8d / %d", sys.data.a_pos, sys.data.v_pos, MPEG2_SYS_CLK_FREQ);
   fflush(stdout);*/

   MPEG2_SYS_modules_init();
   /* send initial offset value to all modules */
   sys.offset = MPEG2_get_offset(0);
   MPEG2_SYS_decoder_pause();
#ifndef _SPEED_TEST_
   FILE_GETPOS(fp, f_pos);
   next_index_pt = AVINDEX_INTERVAL;
   AVindex_init(f_pos, 0, 0, 0, 0);
   if (sys.data.command == CMD_JUMP)
      f_pos = MPEG2_SYS_decoder_jump(fp, NULL, NULL);
#endif
#ifdef _RECORD_STATS_
   TIMER_curtime(&t1);
#endif
   while (sys.data.command != CMD_STOP) {
      FILE_READ(fp, sys.buf, ES_BUFFER_SIZE, bytes_read);
      if (bytes_read < 1)
         break;
#ifndef _SPEED_TEST_
      else if (f_pos >= next_index_pt) {
         next_index_pt += AVINDEX_INTERVAL;
         AVindex_add(f_pos, sys.data.v_pos, sys.data.a_pos, 0, 0);
      }
      f_pos += (FILE_POS)bytes_read;
#endif
#ifdef _RECORD_STATS_
      sys.stream_bytes += (u32)bytes_read;
#endif

      pkt.payload_len16 = (u16)bytes_read;
      pkt.payload       = sys.buf;
      mod_process(mod_mem, &pkt);

/*      printf("\rA:%8d   V:%8d", sys.data.a_pos, sys.data.v_pos);
      fflush(stdout);*/
MPEG_ES_Play_PAUSE_DONE:
      if (bytes_read < ES_BUFFER_SIZE)
         break;
      else if (sys.data.command == CMD_PAUSE) {
#ifdef _RECORD_STATS_
         TIMER_curtime(&t2);
         TIMER_diff(&t1, &t2);
         sys.decode_time += TIMER_getdbl(&t1);
#endif
         MPEG2_SYS_decoder_pause();
#ifdef _RECORD_STATS_
         TIMER_curtime(&t1);
#endif
         goto MPEG_ES_Play_PAUSE_DONE;
      }
#ifndef _SPEED_TEST_
      else if ( (sys.data.command == CMD_JUMP)&&
                (sys.data.a_skip < 10)&&(sys.data.v_skip < 10) ) {
         f_pos = MPEG2_SYS_decoder_jump(fp, NULL, NULL);
         if (idx < VIDEO_DECODERS) {
            sys.data.a_pos = sys.data.a_skip = 0;
         }
         else {
            sys.data.v_pos = sys.data.v_skip = 0;
         }
      }
#endif
   }
#ifdef _RECORD_STATS_
   TIMER_curtime(&t2);
   TIMER_diff(&t1, &t2);
   sys.decode_time += TIMER_getdbl(&t1);
   TIMER_curtime(&t1);
#endif
   MPEG2_SYS_modules_done();
#ifdef _RECORD_STATS_
   TIMER_curtime(&t2);
   TIMER_diff(&t1, &t2);
   sys.decode_time += TIMER_getdbl(&t1);
#endif
/*   printf("\rA:%8d   V:%8d\n", sys.data.a_pos, sys.data.v_pos);*/
   return 1;
}

#undef ES_BUFFER_SIZE

/* Finds the module that can play the type of stream given
      INPUT: stream_type - See PMT_SECTION.h for STREAM_TYPE_* values.
      RETURN: Success - The location of the module in the global
                          struct (sys.psi_pes[]).
              Failure - A negative value. */
int MPEG_ES_find_module(u8 stream_type)
{
   PMT_SECTION pmt;
   ELEM_STREAM es;
   int         i;

   es.stream_type = stream_type;
   es.desc_size   = 0;

   pmt.desc_size = 0;
   pmt.elem_size = 1;
   pmt.elem[0]   = &es;

   for (i = 0; i < MPEG2_SYS_PIDS; i++) {
      if ( (sys.psi_pes[i].stream_init)&&
           (sys.psi_pes[i].stream_init(sys.psi_pes[i].common.mem, &pmt, 0)) )
         return i;
   }
   return -1;
}

