/***************************************************************************
                         dvd_input.h  -  description
                             -------------------
   API to get the stream from the DVD device (using libdvdnav), instead of
    regular files.  This API mimics the one found in common/File.h
                             -------------------
    begin                : Fri Feb 3 2006
    copyright            : (C) 2006 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __DVD_INPUT_H__
#define __DVD_INPUT_H__

#include <dvdnav/dvdnav.h>
#include "common/typedefs.h"
#include "common/File.h"
#include "MPEG2_SYS.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef dvdnav_t *DVD_PTR;
typedef u32       DVD_BLK_COUNT;
typedef u32       DVD_POS;

#define DVD_PTR_OK(ptr)       (ptr)
#define DVD_PTR_BAD(ptr)      (!ptr)
#define DVD_PTR_BAD_VALUE     NULL

#define DVD_BYTES_TO_BLKS(bytes)    ((DVD_POS)bytes >> 11)
#define DVD_BLKS_TO_BYTES(blks)     ((DVD_POS)blks  << 11)
#define DVD_POS_MAKE(chapter, blk)  ((chapter << 24)|(blk & 0x00FFFFFF))
#define DVD_POS_CHAPTER(pos)        (pos >> 24)
#define DVD_POS_BLOCKS(pos)         (pos & 0x00FFFFFF)

#define DVD_OPEN_READ(ptr, device) \
   ptr = dvd_input_open(device)

#define DVD_CLOSE(ptr) \
   dvdnav_close(ptr)

#define DVD_RESET_TITLE_POS(ptr) \
{\
   dvd_title_last = 0xFFFF; \
   dvd_title      = 1; \
}

#define DVD_READ(ptr, buf, buf_bytes, ret_bytes) \
   ret_bytes = dvd_input_read(ptr, buf, buf_bytes)

#define DVD_REWIND(ptr) \
{\
   dvd_chapter   = 1; \
   dvd_blk       = 0; \
   dvd_skip_blks = 0; \
   dvd_time      = 0; \
   dvdnav_title_play(ptr, (int32_t)dvd_title); \
}

#define DVD_GETPOS(ptr, pos) \
   pos = DVD_POS_MAKE(dvd_chapter, dvd_blk)

#define DVD_SETPOS(ptr, pos) \
{\
   dvd_chapter   = DVD_POS_CHAPTER(pos); \
   dvd_blk       = 0; \
   dvd_skip_blks = DVD_POS_BLOCKS(pos); \
   dvd_time      = 0; \
   dvdnav_part_play(ptr, (int32_t)dvd_title, (int32_t)dvd_chapter); \
}

DVD_PTR       dvd_input_open(const char *device);
DVD_BLK_COUNT dvd_input_read(DVD_PTR dvd, u8 *buf, DVD_BLK_COUNT buf_size);
DVD_BLK_COUNT DVD_NUM_CHAPTERS(DVD_PTR dvd);
u16           DVD_AUDIO_LANG(DVD_PTR dvd, u16 PID);
u16           DVD_AUDIO_DEFAULT(DVD_PTR dvd);

#ifdef _DVD_DISPLAY_ERRORS_
extern const char   *dvd_err_str;
#endif
extern const char   *dvd_device_default;
extern char          dvd_lang[];
extern u16           dvd_title_last,
                     dvd_title;
extern DVD_BLK_COUNT dvd_chapter,
                     dvd_blk,
                     dvd_skip_blks;
extern CLK_VAL       dvd_time;

#ifdef __cplusplus
};
#endif

#endif

