/***************************************************************************
                          SEQ_HEADER.h  -  description
                             -------------------
   Stuff needed for handling MPEG2 sequence headers.
                             -------------------
    begin                : Fri Sep 17 2004
    copyright            : (C) 2004 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __SEQ_HEADER_H__
#define __SEQ_HEADER_H__

#include "common/typedefs.h"


#define SEQ_HEADER_PROFILE_ID(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->profile_level & 0x70)
#define PROFILE_HIGH                0x10
#define PROFILE_SPATIALLY_SCALABLE  0x20
#define PROFILE_SNR_SCALABLE        0x30
#define PROFILE_MAIN                0x40
#define PROFILE_SIMPLE              0x50
#define PROFILE_RESERVED(ptr_to_SEQ_HEADER) \
   ( (!(ptr_to_SEQ_HEADER->profile_level & 0x70))|| \
     ( (ptr_to_SEQ_HEADER->profile_level & 0x70) >= 0x60) )

#define SEQ_HEADER_LEVEL_ID(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->profile_level & 0x0F)
#define LEVEL_HIGH       4
#define LEVEL_HIGH_1440  6
#define LEVEL_MAIN       8
#define LEVEL_LOW       10
#define LEVEL_RESERVED(ptr_to_SEQ_HEADER) \
   ( (ptr_to_SEQ_HEADER->profile_level & 0x01)|| \
     ((ptr_to_SEQ_HEADER->profile_level & 0x0F) < 4)|| \
     ((ptr_to_SEQ_HEADER->profile_level & 0x0F) > 10) )

#define SEQ_HEADER_ASPECT_RATIO(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->aspect_frame & 0xF0)
#define ASPECT_RATIO_FORBIDDEN   0x00
#define ASPECT_RATIO_SAR_11      0x10
#define ASPECT_RATIO_DAR_34      0x20
#define ASPECT_RATIO_DAR_916     0x30
#define ASPECT_RATIO_DAR_1221    0x40
#define ASPECT_RATIO_RESERVED(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->aspect_frame >= 0x50)

#define SEQ_HEADER1_ASPECT_RATIO(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->aspect_frame & 0xF0)
#define ASPECT_RATIO_FORBIDDEN   0x00
#define ASPECT_RATIO_SAR_10      0x10
#define ASPECT_RATIO_SAR_06735   0x20
#define ASPECT_RATIO_SAR_07031   0x30
#define ASPECT_RATIO_SAR_07615   0x40
#define ASPECT_RATIO_SAR_08055   0x50
#define ASPECT_RATIO_SAR_08437   0x60
#define ASPECT_RATIO_SAR_08935   0x70
#define ASPECT_RATIO_SAR_09157   0x80
#define ASPECT_RATIO_SAR_09815   0x90
#define ASPECT_RATIO_SAR_10255   0xA0
#define ASPECT_RATIO_SAR_10695   0xB0
#define ASPECT_RATIO_SAR_10950   0xC0
#define ASPECT_RATIO_SAR_11575   0xD0
#define ASPECT_RATIO_SAR_12015   0xE0
#define ASPECT_RATIO_RESERVED1   0xF0

#define SEQ_HEADER_FRAME_RATE_CODE(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->aspect_frame & 0x0F)
#define FRAME_RATE_FORBIDDEN  0
#define FRAME_RATE_23976      1
#define FRAME_RATE_24         2
#define FRAME_RATE_25         3
#define FRAME_RATE_2997       4
#define FRAME_RATE_30         5
#define FRAME_RATE_50         6
#define FRAME_RATE_5994       7
#define FRAME_RATE_60         8
#define FRAME_RATE_RESERVED(ptr_to_SEQ_HEADER) \
   ((ptr_to_SEQ_HEADER->aspect_frame & 0x0F) > 8)

#define SEQ_HEADER_CHROMA_FORMAT(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->flags & 0x03)
#define CHROMA_FORMAT_RESERVED   0
#define CHROMA_FORMAT_420        1
#define CHROMA_FORMAT_422        2
#define CHROMA_FORMAT_444        3
   
#define SEQ_HEADER_FRAME_RATE_EXT_N(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->frame_rate_ext >> 5)
#define SEQ_HEADER_FRAME_RATE_EXT_D(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->frame_rate_ext & 0x1F)
   
#define SEQ_HEADER_INTRA_MATRIX_FLAG(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->flags & 0x80)
#define SEQ_HEADER_NON_INTRA_MATRIX_FLAG(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->flags & 0x40)
#define SEQ_HEADER_PROGRESSIVE_FLAG(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->flags & 0x20)
#define SEQ_HEADER_LOW_DELAY_FLAG(ptr_to_SEQ_HEADER) \
   (ptr_to_SEQ_HEADER->flags & 0x10)

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
   u16 header_size,
       ext_size,
       horiz_size,              /* 12: The 12 least significant bits of the
                                         horizontal frame size. The 2 most
                                         significant bits of this value can
                                         be found in HORIZ_SIZE_EXT.
                           <1-4095> Valid values. */
       vert_size;               /* 12: The 12 least significant bits of the
                                         vertical frame size. The 2 most
                                         significant bits of this value can
                                         be found in VERT_SIZE_EXT. */
   u8  profile_level,
/*
      3   profile_ID                   Identifies the profile of this
                                         sequence. ATSC video streams can be
                                         Main or Simple.
                      <'000'> reserved
                      <'001'> High
                      <'010'> Spatially Scalable
                      <'011'> SNR Scalable
                      <'100'> Main   *
                      <'101'> Simple *
                <'110'-'111'> reserved
      4   level_ID                     Identifies the level of this
                                         sequence. ATSC video streams can be
                                         any non-reserved level.
             <'0000'-'0011'> reserved
                    <'0100'> High
                    <'0101'> reserved
                    <'0110'> High 1440
                    <'0111'> reserved
                    <'1000'> Main
                    <'1001'> reserved
                    <'1010'> Low
             <'1011'-'1111'> reserved
*/
       aspect_frame,
/*
      4   aspect_ratio                 Specifies the sample aspect ratio
                                         (SAR), or the display aspect ratio
                                         (DAR) of the reconstructed frame.
                        <'0000'> forbidden
                        <'0001'> SAR = 1/1    = 1.0
                        <'0010'> DAR = 3/4    = 0.75
                        <'0011'> DAR = 9/16   = 0.5625
                        <'0100'> DAR = 1/2.21 = 0.4525 (approx)
                 <'0101'-'1111'> reserved 
 *
 * If SEQ_DISPLAY_EXT0 is not present then:
 *        SAR = DAR * ((float)horiz_size / (float)vert_size)
 *        (float)
 * If SEQ_DISPLAY_EXT0 is present then:
 *        SAR = DAR * ((float)display_horiz_size / (float)display_vert_size)
 *
      4   frame_rate_code              Specifies the frame of this video
                                         sequence. If the video is
                                         interlaced, then the field rate
                                         is 2*frame_rate.
                           <'0000'> forbidden
                           <'0001'> frame_rate = 24000.0/1001.0 = 23.976 (appox)
                           <'0010'> frame_rate = 24.0
                           <'0011'> frame_rate = 25.0
                           <'0100'> frame_rate = 30000.0/1001.0 = 29.97  (appox)
                           <'0101'> frame_rate = 30.0
                           <'0110'> frame_rate = 50.0
                           <'0111'> frame_rate = 60000.0/1001.0 = 59.94  (appox)
                           <'1000'> frame_rate = 60.0
                    <'1001'-'1111'> reserved
*/
       frame_rate_ext,
/*
      2   FRAME_RATE_EXT_n             Used to change frame_rate to a
          (unsigned byte)                non-standard value.
                           <'00'> ATSC value
      5   FRAME_RATE_EXT_d             Used to change frame_rate to a
          (unsigned byte)                non-standard value.
                           <'00000'> ATSC value
*/
/*
 *        frame_rate *= (float)(FRAME_RATE_EXT_n+1) / (float)(FRAME_RATE_EXT_d+1)
 *        (float)
 */
       flags;
/*
      1   load_intra_q_matrix          Set when the 64 intra quantization
                                         matrix values follow. 
      1   load_non_intra_q_matrix      Set when the 64 non-intra quantization
                                         matrix values follow.
      1   progressive_sequence         Set when the video sequence only
                                         contains progressive frames. When not
                                         set progressive frames, or interlaced
                                         fields are in the sequence.
      1   low_delay                    Set when the video sequence does not
                                         contain any B-frames, and may
                                         contain "big pictures". When not set,
                                         the video sequence does contain
                                         B-frames, and does not contain any
                                         "big pictures".
      2   chroma_format  <'00'> reserved
                         <'01'> 4:2:0 (4Y, 1Cb, 1Cr) Used for ATSC sequences
                         <'10'> 4:2:2 (4Y, 2Cb, 2Cr)
                         <'11'> 4:4:4 (4Y, 4Cb, 4Cr)
*/

   u32 bit_rate,                /* 18: The 18 least significant bits of the
                                         bit rate. The 12 most significant
                                         bits of this value can be found in
                                         BIT_RATE_EXT. This rate is
                                         measured in units of 400bits/sec.
                          <1-262143> Valid values
                          <48500>    ATSC Max value (19.4Mbps channel)
                          <97000>    ATSC Max value (38.8Mbps channel) */
       vbv_buffer_size;         /* 10: The 10 least significant bits of the
                                         VBV buffer size. The 8 most
                                         significant bits of this value can
                                         be found in VBV_BUFFER_SIZE_EXT.
                                         The size is measured in units of
                                         2kB.
                            <488> ATSC Max Value */
/*
 *        vbv_buffer_size *= 2 * 1024  The minimum buffer size, in bytes,
 *                                       needed to decode this video sequence.
 */
   s32 frame_interval;
      
   s8  intra_q_matrix[64],
       non_intra_q_matrix[64];
} SEQ_HEADER;


/* Reads the sequence header in data_stream.
      INPUT: data_stream - Points to the data stream holding the sequence
                             header. This stream does not have to contain
                             the entire header. Repeated calls to this
                             function can be used to read the header in parts.
             data_len    - Number of bytes in the data stream.
      OUTPUT: hdr     - Points to the SEQ_HEADER that will hold the data.
              cur_pos - Used by MPEG2_VIDEO_MODULE.
              code    - Used by MPEG2_VIDEO_MODULE.
      RETURN: Number of bytes read. */
u32 SEQ_HEADER_read(const u8         *data_stream,
                          u32         data_len,
                          SEQ_HEADER *hdr,
                          u32        *cur_pos,
                          u16        *code);

/* Reads the sequence header extension in data_stream.
      INPUT: data_stream - Points to the data stream holding the sequence
                             header extension. This stream does not have to
                             contain the entire extension. Repeated calls to
                             this function can be used to read the extension
                             in parts.
             data_len    - Number of bytes in the data stream.
      OUTPUT: hdr     - Points to the SEQ_HEADER that will hold the data.
              cur_pos - Used by MPEG2_VIDEO_MODULE.
              code    - Used by MPEG2_VIDEO_MODULE.
      RETURN: Number of bytes read. */
u32 SEQ_HEADER_EXT_read(const u8         *data_stream,
                              u32         data_len,
                              SEQ_HEADER *hdr,
                              u32        *cur_pos,
                              u16        *code);

/* Reads the quant matrix extension in data_stream.
      INPUT: data_stream - Points to the data stream holding the quant
                             matrix extension. This stream does not have to
                             contain the entire extension. Repeated calls to
                             this function can be used to read the extension
                             in parts.
             data_len    - Number of bytes in the data stream.
      OUTPUT: hdr     - Points to the SEQ_HEADER that will hold the data.
              cur_pos - Used by MPEG2_VIDEO_MODULE.
              code    - Used by MPEG2_VIDEO_MODULE.
      RETURN: Number of bytes read. */
u32 QUANT_MATRIX_EXT_read(const u8         *data_stream,
                                u32         data_len,
                                SEQ_HEADER *hdr,
                                u32        *cur_pos,
                                u16        *code);
#ifdef __cplusplus
};
#endif

#endif
