/***************************************************************************
                          SLICE.c  -  description
                             -------------------
   Stuff needed for handling MPEG2 slices.  This is where the video
   decoding process actually take place.  This decoder can only handle
   the popular 4:2:0 colour format.
                             -------------------
    begin                : Tue Oct 5 2004
    copyright            : (C) 2004 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*#define _M2V_IDCT_TRACE_*/
/*#define _SHOW_ERRORS_*/
#ifdef _M2V_TRACE_
# ifndef _SHOW_ERRORS_
#  define _SHOW_ERRORS_
# endif
#endif

#ifdef _SHOW_ERRORS_
#include <stdio.h>
#endif
#include <string.h>

#include "SLICE.h"
#include "mpeg_tables.h"
#include "idct.h"
#include "output_blocks.h"
#include "motion_vectors.h"
#include "bitstream.h"


#define RESET_DCT_PRED(s_ptr) \
   s_ptr->dct_pred[0] = s_ptr->dct_pred[1] = s_ptr->dct_pred[2] = 0
#define RESET_PMV(s_ptr, bytes) \
   memset(s_ptr->PMV, 0, bytes)

#define USE_LOCAL_VARS
#ifdef USE_LOCAL_VARS
# define DATA_BYTE         data
# define BIT_SHIFT_BYTE    bit_shift
#else
# define DATA_BYTE         slice->data
# define BIT_SHIFT_BYTE    slice->bit_shift
#endif
#define  DATA_PTR          ptr
#define  END_DATA_PTR      end
#define  PARSER_STATE      *cur_pos
#define  STOP_PARSING      goto SLICE_read_END


#ifdef _M2V_TRACE_
static int frame_cnt = -1,
           mb_cnt    = -1,
           slice_cnt = -1,
           block_cnt = -1;
#endif

#ifdef _M2V_IDCT_TRACE_
static void show_QFS(s16 *QFS);
#endif

/* Processes a slice in data_stream.
      INPUT: data_stream - Points to the data stream holding the slice.
                             This stream does not have to contain the
                             entire slice. Repeated calls to this
                             function can be used to read the slice
                             in parts.
             data_len    - Number of bytes in the data stream.
      OUTPUT: seq_hdr - Points to the SEQ_HEADER of the mpeg2 stream.
              pic_hdr - Points to the PIC_HEADER of the slice.
              cur_pos - Used by MPEG2_VIDEO_MODULE.
              code    - Used by MPEG2_VIDEO_MODULE.
      RETURN: Number of bytes read. */
u32 SLICE_read(const u8    *data_stream,
                     u32    data_len,
                     SLICE *slice,
                     u32   *cur_pos,
                     u16   *code)
{
   const u8 *ptr = data_stream,
            *end = data_stream + data_len;
   u32 tmp;
   s32 dct;
   u8  bit_shift_inc, run, level;
#ifdef USE_LOCAL_VARS
   u8  bit_shift, data;
#endif
#ifdef _DUAL_PRIME_SUPPORT_
   s8  dmvector;
#endif

/*   if (slice->seq_hdr->vert_size > 2800) {
#ifdef _SHOW_ERRORS_
      printf("\nvert_size > 2800\n");
#endif
      goto SLICE_read_ERROR;
   } */
   data_len += (u32)ptr;

#define SLICE_0                0
#define SLICE_1                1
#define SLICE_2                2
#define MACROBLOCK_0           3
#define MACROBLOCK_00          4
#define MACROBLOCK_1           5
#define MACROBLOCK_MODES_0     6
#define MACROBLOCK_MODES_1     7
#define MACROBLOCK_MODES_2     8
#define MACROBLOCK_MODES_3     9
#define MACROBLOCK_MODES_4    10
#define MACROBLOCK_2          11
#define MACROBLOCK_3          12
#define MACROBLOCK_4          13
#define MOTION_VECTOR_1       14
#define MOTION_VECTOR_2       15
#define MOTION_VECTOR_3       16
#define MOTION_VECTOR_4       17
#define MOTION_VECTOR_5       18
#define MOTION_VECTOR_6       19
#define MACROBLOCK_5          20
#define MACROBLOCK_6          21
#define MACROBLOCK_7          22
#define BLOCK_0               23
#define BLOCK_1               24
#define BLOCK_2               25
#define BLOCK_3               26
#define BLOCK_4               27
#define BLOCK_5               28
#define BLOCK_6               29
#define BLOCK_7               30
#define BLOCK_8               31
#define BLOCK_9               32
#define BLOCK_10              33
#define BLOCK_11              34
#define BLOCK_12              35
#define BLOCK_13              36
#define BLOCK_14              37
#define BLOCK_15              38
#define BLOCK_16              39
#define BLOCK_17              40
#define BLOCK_18              41
#define BLOCK_19              42
#define BLOCK_20              43
#define MACROBLOCK_8          44
#define MACROBLOCK_9          45
#define MACROBLOCK_10         46

START_AGAIN:
   if (*cur_pos) {
#ifdef USE_LOCAL_VARS
      bit_shift = slice->bit_shift;
      data      = slice->data;
#endif
      switch (*cur_pos) {
      case SLICE_1: 
         goto SLICE_skip_extra_byte;
      case SLICE_2:
         goto SLICE_2_START;
      case MACROBLOCK_0:
         goto MACROBLOCK_read_addr_inc;
      case MACROBLOCK_00:
         goto MACROBLOCK_read_next_addr_inc;
      case MACROBLOCK_1:
         goto MACROBLOCK_1_START;
      case MACROBLOCK_MODES_0:
         goto MACROBLOCK_MODES_0_START;
      case MACROBLOCK_MODES_1:
         goto MACROBLOCK_MODES_P_FRAME;
      case MACROBLOCK_MODES_2:
         goto MACROBLOCK_MODES_B_FRAME;
      case MACROBLOCK_MODES_3: 
         goto MACROBLOCK_MODES_3_START;
      case MACROBLOCK_MODES_4: 
         goto MACROBLOCK_MODES_4_START;
      case MACROBLOCK_2:
         goto MACROBLOCK_2_START;
      case MACROBLOCK_3: 
         goto MACROBLOCK_3_START;
      case MACROBLOCK_4:
         goto MACROBLOCK_read_MOTION_BCK;
      case MOTION_VECTOR_1: 
         goto MOTION_VECTOR_read_sel_2;
      case MOTION_VECTOR_2: 
         goto MOTION_VECTOR_read;
      case MOTION_VECTOR_3: 
         goto MOTION_VECTOR_3_START;
      case MOTION_VECTOR_4: 
         goto MOTION_VECTOR_4_START;
      case MOTION_VECTOR_5: 
         goto MOTION_VECTOR_5_START;
      case MOTION_VECTOR_6: 
         goto MOTION_VECTOR_6_START;
      case MACROBLOCK_5: 
         goto MACROBLOCK_read_marker_bit;
      case MACROBLOCK_6: 
         goto MACROBLOCK_6_START;
      case MACROBLOCK_7: 
         goto MACROBLOCK_7_START;
      case BLOCK_0: 
         goto BLOCKS_read_start;
      case BLOCK_1: 
         goto BLOCK_read;
      case BLOCK_2: 
         goto BLOCK_2_START;
      case BLOCK_3: 
         goto BLOCK_3_START;
      case BLOCK_4: 
         goto FIRST_INTRA_CHROMA_DCT_read;
      case BLOCK_5: 
         goto BLOCK_5_START;
      case BLOCK_6: 
         goto BLOCK_6_START;
      case BLOCK_7: 
         goto BLOCK_7_START;
      case BLOCK_8: 
         goto NEXT_DCT_COEF_read;
      case BLOCK_9: 
         goto FIRST_DCT_COEF_read;
      case BLOCK_10: 
         goto TABLE0_DCT_COEF_read;
      case BLOCK_11: 
         goto BLOCK_11_START;
      case BLOCK_12: 
         goto BLOCK_12_START;
      case BLOCK_13: 
         goto TABLE1_DCT_COEF_read;
      case BLOCK_14: 
         goto BLOCK_14_START;
      case BLOCK_15: 
         goto BLOCK_15_START;
      case BLOCK_16: 
         goto BLOCK_16_START;
      case BLOCK_17: 
         goto BLOCK_17_START;
      case BLOCK_18: 
         goto BLOCK_18_START;
      case BLOCK_19: 
         goto BLOCK_19_START;
      case BLOCK_20: 
         goto BLOCK_read_done;
      case MACROBLOCK_8: 
         goto MACROBLOCK_next;
      case MACROBLOCK_9: 
         goto MACROBLOCK_9_START;
      case MACROBLOCK_10: 
         goto MACROBLOCK_10_START;
      default:
         goto SLICE_read_ERROR;
      }
   }
   
   /* read SLICE */
/*   case SLICE_0: */
SLICE_start_new:
#ifdef _M2V_TRACE_
      if (frame_cnt < 0) {
         frame_cnt = slice_cnt = 0;
      }
      else if ((*code - 1) < slice->mb_row) {
         if (slice->fields_done == PIC_HEADER_STRUCTURE(slice->pic_hdr))
            frame_cnt++;
         slice_cnt = 0;
      }
      else
         slice_cnt++;
#endif
      slice->mb_row = *code - 1;
/*
      if (slice->mb_row > slice->cur_row) {
         * fill in missing rows *
         tmp  = (u32)(slice->mb_row - slice->cur_row - 1);
         tmp *= (u32)slice->mb_wid;
         tmp += (u32)(slice->mb_wid - slice->cur_col);
         skip_macroblocks(slice, tmp);
         printf("skipping %u macroblocks, because of missing rows\n", tmp);
      }
*/      
      slice->q_scale_code = *ptr >> 3;
      RESET_PMV(slice, (sizeof(s16)*12)); /* reset DCT and MV predictors */

      BIT_SHIFT_BYTE = 5;
      DATA_BYTE      = *ptr << 5;
      ptr++;
      if (data_len == (u32)ptr) {
         *cur_pos = SLICE_1;
         goto SLICE_read_END;
      }
/*   case SLICE_1: */
SLICE_skip_extra_byte:
      GET_LSB;
      slice->r = DATA_BYTE & 0x80;  /* extra data flag */
      ALIGN_DATA_BYTE(SLICE_2, 1);
/*   case SLICE_2: */
SLICE_2_START:
      if (slice->r) {
         ALIGN8_DATA_BYTE(SLICE_1);
         goto SLICE_skip_extra_byte;
      }

   /* read MACROBLOCK */
#ifdef _M2V_TRACE_
      mb_cnt = (int)(slice->mb_row * slice->mb_wid) - 1;
#endif
      slice->mb_col = 0xFFFF;
/*   case MACROBLOCK_0: */
MACROBLOCK_read_addr_inc:
      slice->signed_level = 0;
/*   case MACROBLOCK_00: */
MACROBLOCK_read_next_addr_inc:
      GET_LSB;

      /* macroblock_addr_inc VLC: BYTE 1 */
      if (DATA_BYTE >= 0x06) {
         if (DATA_BYTE >= 0x80) {         /*-1-*/
            tmp = 1;
            bit_shift_inc = 1;
         }
         else if (DATA_BYTE >= 0x40) {    /* 01 */
            if (DATA_BYTE >= 0x60)        /*-011-*/
               tmp = 2;
            else                          /*-010-*/
               tmp = 3;
            bit_shift_inc = 3;
         }
         else if (DATA_BYTE >= 0x20) {    /* 001 */
            if (DATA_BYTE >= 0x30)        /*-0011-*/
               tmp = 4;
            else                          /*-0010-*/
               tmp = 5;
            bit_shift_inc = 4;
         }
         else if (DATA_BYTE >= 0x10) {    /* 0001 */
            if (DATA_BYTE >= 0x18)        /*-0001 1-*/
               tmp = 6;
            else                          /*-0001 0-*/
               tmp = 7;
            bit_shift_inc = 5;
         }
         else if (DATA_BYTE >= 0x0C) {    /* 0000 11 */
            if (DATA_BYTE >= 0x0E)        /*-0000 111-*/
               tmp = 8;
            else                          /*-0000 110-*/
               tmp = 9;
            bit_shift_inc = 7;
         }
         else {                           /* 0000 1011 -> 0000 0110 */
            tmp = 21 - (u32)DATA_BYTE;
            bit_shift_inc = 8;
         }

         /* finished reading VLC code */
         goto MACROBLOCK_read_addr_inc_done;
      }

      /* VLC code is longer than 8 bits, first 5 bits are 0 */
      ALIGN_DATA_BYTE(MACROBLOCK_1, 5);
/*   case MACROBLOCK_1: */
MACROBLOCK_1_START:
      GET_LSB;

      /* macroblock_addr_inc VLC: BYTE 2 */
      if (DATA_BYTE >= 0x90) {
         if (DATA_BYTE >= 0xA0)              /* 0000 0101 XX */
            tmp = 39 - (u32)(DATA_BYTE >> 3);
         else if (DATA_BYTE >= 0x98)         /*-0000 0100 11-*/
            tmp = 20;
         else                                /*-0000 0100 10-*/
            tmp = 21;
         bit_shift_inc = 5;
      }
      else if (DATA_BYTE >= 0x60) {          /* 0000 0100 0XX */
                                             /* 0000 0011 XXX */
         tmp = 57 - (u32)(DATA_BYTE >> 2);
         bit_shift_inc = 6;
      }
      else if ( (0x20 <= DATA_BYTE)&&(DATA_BYTE <= 0x23) ) {
                                             /*-0000 0001 000 */
         /* encountered VLC escape code */
         slice->signed_level += 33;

         /* decode the next code */
         ALIGN_DATA_BYTE(MACROBLOCK_00, 6);
         goto MACROBLOCK_read_next_addr_inc;
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid mb_addr_inc VLC2 (0x%02hX)\n",
                (u16)DATA_BYTE);
#endif
         goto SLICE_read_ERROR;
      }

MACROBLOCK_read_addr_inc_done:
      tmp += (u32)slice->signed_level;
      if (tmp > 1) {
         if (PIC_HEADER_CODING_TYPE(slice->pic_hdr) == CODING_TYPE_P_FRAME)
            RESET_PMV(slice, (sizeof(s16)*12)); /* reset DCT & MV predictors */
         else
            RESET_DCT_PRED(slice);  /* reset DCT predictors */

         if (slice->mb_col != 0xFFFF)
            skip_macroblocks(slice, (tmp - 1));
         slice->mb_col += (u16)tmp;
      }
      else
         slice->mb_col++;
#ifdef _M2V_TRACE_
      mb_cnt += (int)tmp;
      printf("FRAME %d, SLICE %d, MACROBLOCK %d\n",
             frame_cnt, slice_cnt, mb_cnt);
      printf("  mb_col = %hu, mb_row = %hu\n"
             "  cur_col= %hu, cur_row= %hu\n"
             "  q_scale_code = 0x%02hX, VLC mb_addr_inc = %u\n",
             slice->mb_col,  slice->mb_row,
             slice->cur_col, slice->cur_row,
             (u16)slice->q_scale_code, tmp);
#endif
      ALIGN_DATA_BYTE(MACROBLOCK_MODES_0, bit_shift_inc);

   /* read MACROBLOCK_MODES */
/*   case MACROBLOCK_MODES_0: */
MACROBLOCK_MODES_0_START:
      GET_LSB;
      /* read mb_type VLC */
#ifdef _M2V_TRACE_
      printf("  VLC mb_type(");
#endif
      switch (PIC_HEADER_CODING_TYPE(slice->pic_hdr)) {
      case CODING_TYPE_B_FRAME:
#ifdef _M2V_TRACE_
         printf("B");
#endif
         goto MACROBLOCK_MODES_B_FRAME;
      case CODING_TYPE_P_FRAME:
#ifdef _M2V_TRACE_
         printf("P");
#endif
         goto MACROBLOCK_MODES_P_FRAME;
      case CODING_TYPE_I_FRAME:
#ifdef _M2V_TRACE_
         printf("I");
#endif
         break;
      default:
#ifdef _SHOW_ERRORS_
         printf("\nInvalid picture_coding_type\n");
#endif
         goto SLICE_read_ERROR;
      }

      run = MB_INTRA_BIT;
      if (DATA_BYTE >= 0x80)        /*-1-*/
         bit_shift_inc = 1;
      else if (DATA_BYTE >= 0x40) { /*-01-*/
         run |= MB_QUANT_BIT;
         bit_shift_inc = 2;
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid MACROBLOCK_MODES I frame VLC (0x%02hX)\n",
                (u16)DATA_BYTE);
#endif
         goto SLICE_read_ERROR;

      }
      goto MACROBLOCK_MODES_mb_type_done;

/*   case MACROBLOCK_MODES_1: */
MACROBLOCK_MODES_P_FRAME:
      if (DATA_BYTE >= 0x80) {      /*-1-*/
         run = MB_MOTION_FWD_BIT | MB_PATTERN_BIT;
         bit_shift_inc = 1;
      }
      else if (DATA_BYTE >= 0x40) { /*-01-*/
         run = MB_PATTERN_BIT;
         bit_shift_inc = 2;
      }
      else if (DATA_BYTE >= 0x20) { /*-001-*/
         run = MB_MOTION_FWD_BIT;
         bit_shift_inc = 3;
      }
      else if (DATA_BYTE >= 0x08) {
         if (DATA_BYTE >= 0x18)     /*-0001 1-*/
            run = MB_INTRA_BIT;
         else if (DATA_BYTE >= 0x10)/*-0001 0-*/
            run = MB_MOTION_FWD_BIT | MB_PATTERN_BIT | MB_QUANT_BIT;
         else                       /*-0000 1-*/
            run = MB_PATTERN_BIT | MB_QUANT_BIT;
         bit_shift_inc = 5;
      }
      else if (DATA_BYTE >= 0x04) { /*-0000 01-*/
         run = MB_INTRA_BIT | MB_QUANT_BIT;
         bit_shift_inc = 6;
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid MACROBLOCK_MODES P frame VLC (0x%02hX)\n",
                (u16)DATA_BYTE);
#endif
         goto SLICE_read_ERROR;

      }

      if (!(run & (MB_INTRA_BIT|MB_MOTION_FWD_BIT))) {
         slice->vector[0][0][0] = 0;
         slice->vector[0][0][1] = 0;
         RESET_PMV(slice, (sizeof(s16)*8));  /* reset MV predictors */
      }
      goto MACROBLOCK_MODES_mb_type_done;

/*   case MACROBLOCK_MODES_2: */
MACROBLOCK_MODES_B_FRAME:
      if (DATA_BYTE >= 0x80) {      /* 1 */
         run = MB_MOTION_FWD_BIT | MB_MOTION_BCK_BIT;
         if (DATA_BYTE >= 0xC0)     /*-11-*/
            run |= MB_PATTERN_BIT;
         bit_shift_inc = 2;
      }
      else if (DATA_BYTE >= 0x40) { /* 01 */
         run = MB_MOTION_BCK_BIT;
         if (DATA_BYTE >= 0x60)     /*-011-*/
            run |= MB_PATTERN_BIT;
         bit_shift_inc = 3;
      }
      else if (DATA_BYTE >= 0x20) { /* 001 */
         run = MB_MOTION_FWD_BIT;
         if (DATA_BYTE >= 0x30)     /*-0011-*/
            run |= MB_PATTERN_BIT;
         bit_shift_inc = 4;
      }
      else if (DATA_BYTE >= 0x10) { /* 0001 */
         if (DATA_BYTE >= 0x18)     /*-0001 1-*/
            run = MB_INTRA_BIT;
         else {                     /*-0001 0-*/
            run = MB_MOTION_FWD_BIT | MB_MOTION_BCK_BIT |
                  MB_QUANT_BIT | MB_PATTERN_BIT;
         }
         bit_shift_inc = 5;
      }
      else if (DATA_BYTE >= 0x08) { /* 0000 1 */
         run = MB_QUANT_BIT | MB_PATTERN_BIT;
         if (DATA_BYTE >= 0x0C)     /*-0000 11-*/
            run |= MB_MOTION_FWD_BIT;
         else                       /*-0000 10-*/
            run |= MB_MOTION_BCK_BIT;
         bit_shift_inc = 6;
      }
      else if (DATA_BYTE >= 0x04) { /*-0000 01-*/
         run = MB_QUANT_BIT | MB_INTRA_BIT;
         bit_shift_inc = 6;
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid MACROBLOCK_MODES B frame VLC (0x%02hX)\n",
                (u16)DATA_BYTE);
#endif
         goto SLICE_read_ERROR;

      }
MACROBLOCK_MODES_mb_type_done:
      slice->mb_flags = run;
#ifdef _M2V_TRACE_
      printf(") =%s%s%s%s%s\n",
             (MB_MOTION_FWD(slice)) ? " FWD" : "",
             (MB_MOTION_BCK(slice)) ? " BCK" : "",
             (MB_PATTERN(slice))    ? " CBP" : "",
             (MB_INTRA(slice))    ? " INTRA" : "",
             (MB_QUANT(slice))    ? " QUANT" : "");
#endif
      if (!(run & MB_INTRA_BIT)) {
         RESET_DCT_PRED(slice);  /* reset DCT predictors */
         memset(slice->mv_blk, 0, (sizeof(u8 *)*4));
      }
      else if (!PIC_HEADER_CONCEALMENT_VECTORS(slice->pic_hdr))
         RESET_PMV(slice, (sizeof(s16)*8));  /* reset MV predictors */
      ALIGN_DATA_BYTE(MACROBLOCK_MODES_3, bit_shift_inc);

/*   case MACROBLOCK_MODES_3: */
MACROBLOCK_MODES_3_START:
      if ( (MB_MOTION_FWD(slice))||(MB_MOTION_BCK(slice)) ) {
         GET_LSB;
         bit_shift_inc = 0;
         if (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME) {
            if (!PIC_HEADER_FRAME_PRED_DCT(slice->pic_hdr)) {
#ifdef _M2V_TRACE_
               printf("  frame_motion_type = 0x%1hX\n",
                      (u16)(DATA_BYTE >> 6));
#endif
               /* frame_motion_type */
               if (DATA_BYTE >= 0xC0) {      /*-11-*/
#ifdef _DUAL_PRIME_SUPPORT_
                  slice->pred_type = PRED_TYPE_DUAL_PRIME;
                  slice->mv_count  = 1;
#else
# ifdef _SHOW_ERRORS_
                  printf("\nMissing Dual Prime support\n");
# endif
                  goto SLICE_read_ERROR;
#endif
               }
               else if (DATA_BYTE >= 0x80) { /*-10-*/
                  slice->pred_type = PRED_TYPE_FRAME_BASED;
                  slice->mv_count  = 1;
               }
               else if (DATA_BYTE >= 0x40) { /*-01-*/
                  slice->pred_type = PRED_TYPE_FIELD_BASED;
                  slice->mv_count  = 2;
               }
               else {
#ifdef _SHOW_ERRORS_
                  printf("\nInvalid frame_motion_type (0x%02hX)\n",
                         (u16)(DATA_BYTE & 0xC0));
#endif
                  goto SLICE_read_ERROR;
               }
               bit_shift_inc += 2;
            }
            else {
               slice->pred_type = PRED_TYPE_FRAME_BASED;
               slice->mv_count  = 1;
            }
         }
         else {
#ifdef _M2V_TRACE_
            printf("  field_motion_type = 0x%1hX\n",
                   (u16)(DATA_BYTE >> 6));
#endif
            /* field_motion_type */
            if (DATA_BYTE >= 0xC0) {      /*-11-*/
#ifdef _DUAL_PRIME_SUPPORT_
               slice->pred_type = PRED_TYPE_DUAL_PRIME;
               slice->mv_count  = 1;
#else
# ifdef _SHOW_ERRORS_
               printf("\nMissing Dual Prime support\n");
# endif
               goto SLICE_read_ERROR;
#endif
            }
            else if (DATA_BYTE >= 0x80) { /*-10-*/
               slice->pred_type = PRED_TYPE_MC_16x8;
               slice->mv_count  = 2;
            }
            else if (DATA_BYTE >= 0x40) { /*-01-*/
               slice->pred_type = PRED_TYPE_FIELD_BASED;
               slice->mv_count  = 1;
            }
            else {
#ifdef _SHOW_ERRORS_
               printf("\nInvalid field_motion_type (0x%02hX)\n",
                      (u16)(DATA_BYTE & 0xC0));
#endif
               goto SLICE_read_ERROR;
            }
            bit_shift_inc += 2;
         }

         if (bit_shift_inc) {
            ALIGN_DATA_BYTE(MACROBLOCK_MODES_4, bit_shift_inc);
         }
      }
      else {
         /* I-picture with concealment vectors, or
            P-picture with no motion vectors in bitstream */
         slice->mv_count  = 1;
         slice->mv_blk[0] = slice->ref_blkF;
         run              = PIC_HEADER_STRUCTURE(slice->pic_hdr);
         if (run == STRUCTURE_FULL_FRAME)
            slice->pred_type = PRED_TYPE_FRAME_BASED;
         else {
            slice->pred_type = PRED_TYPE_FIELD_BASED;
            if (run == STRUCTURE_BOT_FIELD)
               slice->mv_blk[0] += slice->frame_wid;
         }
      }

/*   case MACROBLOCK_MODES_4: */
MACROBLOCK_MODES_4_START:
      if ( (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME)&&
           (!PIC_HEADER_FRAME_PRED_DCT(slice->pic_hdr))&&
           ( (MB_INTRA(slice))||(MB_PATTERN(slice)) ) ) {
         GET_LSB;
#ifdef _M2V_TRACE_
         printf("  frame_dct_type = %s\n",
                (DATA_BYTE & 0x80) ? "field" : "frame");
#endif
         /* dct_type */
         if (DATA_BYTE & 0x80)
            slice->mb_flags |= MB_DCT_TYPE_BIT;
         ALIGN_DATA_BYTE(MACROBLOCK_2, 1);
      }
   /* finished reading MACROBLOCK_MODES */

   /* read MACROBLOCK continue */
/*   case MACROBLOCK_2: */
MACROBLOCK_2_START:
      if (MB_QUANT(slice)) {
         GET_LSB;
         slice->q_scale_code = DATA_BYTE >> 3;
#ifdef _M2V_TRACE_
         printf("  q_scale_code = 0x%02hX\n", (u16)slice->q_scale_code);
#endif
         ALIGN_DATA_BYTE(MACROBLOCK_3, 5);
      }

/*   case MACROBLOCK_3: */
MACROBLOCK_3_START:
      if ( (MB_MOTION_FWD(slice))||
           ( (MB_INTRA(slice))&&
             (PIC_HEADER_CONCEALMENT_VECTORS(slice->pic_hdr)) ) ) {
         /* motion vector 0 (forward prediction) */
         slice->s = 0;
         goto MOTION_VECTOR_read_sel_1;
      }

/*   case MACROBLOCK_4: */
MACROBLOCK_read_MOTION_BCK:
      if (!MB_MOTION_BCK(slice))
         goto MACROBLOCK_read_marker_bit;

      /* motion vector 1 (backward prediction) */
      slice->s = 1;

/*   case MOTION_VECTOR_0: */
   /* read MOTION VECTOR (set slice->s to 0, 1) */
MOTION_VECTOR_read_sel_1:
      slice->r = 0;
#ifdef _DUAL_PRIME_SUPPORT_
      if ( (slice->mv_count == 1)&&(slice->pred_type != PRED_TYPE_FIELD_BASED) ) {
#else
      if (slice->pred_type == PRED_TYPE_FRAME_BASED) {
#endif
         /* find the reference macroblocks for the motion vectors */
         if (slice->s)
            slice->mv_blk[1] = slice->ref_blkB;
         else
            slice->mv_blk[0] = slice->ref_blkF;

         /* skip vertical field select bit */
         slice->t = 0;
         goto MOTION_VECTOR_read;
      }

      /* slice->r, and slice->s set */
/*   case MOTION_VECTOR_1: */
MOTION_VECTOR_read_sel_2:
      slice->t = 0;
      GET_LSB;
      /* find the reference macroblocks for the motion vectors */
      run   = (slice->r << 1) | slice->s;
      level = PIC_HEADER_STRUCTURE(slice->pic_hdr);
      if ( (level == slice->fields_done)||
           (PIC_HEADER_CODING_TYPE(slice->pic_hdr) == CODING_TYPE_B_FRAME) ) {
         /* 1st P-field, P-frame or B-picture predictions */
         slice->mv_blk[run] = (slice->s) ? slice->ref_blkB : slice->ref_blkF;
         if (DATA_BYTE >= 0x80)  /* bottom field */
            slice->mv_blk[run] += slice->frame_wid;
      }
      else if (level == STRUCTURE_TOP_FIELD) {
         /* 2nd P-field (top) predictions */
         if (DATA_BYTE >= 0x80)  /* opposite field */
            slice->mv_blk[run] = slice->ref_blkB + slice->frame_wid;
         else                    /* same field */
            slice->mv_blk[run] = slice->ref_blkF;
      }
      else {
         /* 2nd P-field (bottom) predictions */
         if (DATA_BYTE >= 0x80)  /* same field */
            slice->mv_blk[run] = slice->ref_blkF + slice->frame_wid;
         else                    /* opposite field */
            slice->mv_blk[run] = slice->ref_blkB;
      }
#ifdef _M2V_TRACE_
      printf("motion_vert_field_sel[r=%hu][s=%hu]%sset\n",
             (u16)slice->r, (u16)slice->s,
             (DATA_BYTE & 0x80) ? " " : " not ");
#endif
      ALIGN_DATA_BYTE(MOTION_VECTOR_2, 1);

      /* slice->r, slice->s, and slice->t set */
/*   case MOTION_VECTOR_2: */
MOTION_VECTOR_read:
      GET_LSB;

      /* motion_code VLC: BYTE 1 */
      if (DATA_BYTE >= 0x06) {
         if (DATA_BYTE >= 0x80) {      /*-1-*/
            run = 0;
            bit_shift_inc = 1;
         }
         else if (DATA_BYTE >= 0x40) { /*-01-*/
            run = 1;
            bit_shift_inc = 2;
         }
         else if (DATA_BYTE >= 0x20) { /*-001-*/
            run = 2;
            bit_shift_inc = 3;
         }
         else if (DATA_BYTE >= 0x10) { /*-0001-*/
            run = 3;
            bit_shift_inc = 4;
         }
         else if (DATA_BYTE >= 0x0C) { /*-0000 11-*/
            run = 4;
            bit_shift_inc = 6;
         }
         else {                        /* 0000 101 -> 0000 011 */
            run = 10 - (DATA_BYTE >> 1);
            bit_shift_inc = 7;
         }

         /* finished reading motion_code VLC */
         goto MOTION_VECTOR_read_motion_code_done;
      }

      /* motion_code is longer than 8 bits, first 5 bits are 0 */
      ALIGN_DATA_BYTE(MOTION_VECTOR_3, 5);
/*   case MOTION_VECTOR_3: */
MOTION_VECTOR_3_START:
      GET_LSB;

      /* motion_code VLC: BYTE 2 */
      if (DATA_BYTE >= 0x90) {
         if (DATA_BYTE >= 0xB0)        /*-0000 0101 1-*/
            run =  8;
         else if (DATA_BYTE >= 0xA0)   /*-0000 0101 0-*/
            run =  9;
         else                          /*-0000 0100 1-*/
            run = 10;
         bit_shift_inc = 4;
      }
      else if (DATA_BYTE >= 0x80) {
         if (DATA_BYTE >= 0x88)        /*-0000 0100 01-*/
            run = 11;
         else                          /*-0000 0100 00-*/
            run = 12;
         bit_shift_inc = 5;
      }
      else if (DATA_BYTE >= 0x60) {    /* 0000 0011 XX */
         run = 28 - (DATA_BYTE >> 3);
         bit_shift_inc = 5;
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid motion_code VLC2 (0x%02hX)\n",
                (u16)DATA_BYTE);
#endif
         goto SLICE_read_ERROR;

      }
MOTION_VECTOR_read_motion_code_done:
      /* finished reading motion_code VLC */
      slice->signed_level = (s16)run;
      ALIGN_DATA_BYTE(MOTION_VECTOR_4, bit_shift_inc);
/*   case MOTION_VECTOR_4: */
MOTION_VECTOR_4_START:
#ifdef _M2V_TRACE_
      if (!slice->t) {
         printf("last PMV[%hu][%hu]=(%hd, %hd)\n",
                (u16)slice->r, (u16)slice->s,
                slice->PMV[slice->r][slice->s][0],
                slice->PMV[slice->r][slice->s][1]);
      }
      printf("motion_code[r=%hu][s=%hu][t=%hu] =",
             (u16)slice->r, (u16)slice->s, (u16)slice->t);
#endif
      if (slice->signed_level) {
         GET_LSB;
         /* get sign bit */
         slice->i = DATA_BYTE & 0x80;
#ifdef _M2V_TRACE_
         printf("%s%hd\n", (slice->i) ? " -" : " ", slice->signed_level);
#endif
         ALIGN_DATA_BYTE(MOTION_VECTOR_5, 1);
      }
      else {
#ifdef _M2V_TRACE_
         printf(" 0\n");
#endif
         if ( (slice->t)&&(MV_FORMAT_FIELD(slice))&&
              (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME) )
            run = 1;
         else
            run = 0;
         slice->signed_level = slice->PMV[slice->r][slice->s][slice->t] >> run;
         slice->vector[slice->r][slice->s][slice->t] = slice->signed_level;
         slice->PMV[slice->r][slice->s][slice->t]    = slice->signed_level << run;
         goto MOTION_VECTOR_read_dmvector;
      }

/*   case MOTION_VECTOR_5: */
MOTION_VECTOR_5_START:
      run = slice->pic_hdr->f_code[slice->s][slice->t];
      if ( (1 < run)&&(run < 10) ) {
         slice->cc = run - 1;
         slice->signed_level--;
         slice->signed_level <<= slice->cc;
         slice->signed_level++;
         
         GET_LSB;
         /* read motion_residual VLC */
         run                  = 9 - run;
         slice->signed_level += (s16)(DATA_BYTE >> run);
         if (slice->i)
            slice->signed_level = -slice->signed_level;
#ifdef _M2V_TRACE_
         printf("motion_residual[r=%hu][s=%hu][t=%hu] = %hu\n",
                (u16)slice->r, (u16)slice->s, (u16)slice->t,
                (u16)(DATA_BYTE >> run));
#endif
         bit_shift_inc = 8 - run;
         ALIGN_DATA_BYTE(MOTION_VECTOR_6, bit_shift_inc);
      }
      else {
         slice->cc = 0;
         if (slice->i)
            slice->signed_level = -slice->signed_level;
      }

/*   case MOTION_VECTOR_6: */
MOTION_VECTOR_6_START:
      /* calculate motion vector */
      if ( (slice->t)&&(MV_FORMAT_FIELD(slice))&&
           (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME) )
         run = 1;
      else
         run = 0;
      slice->signed_level += slice->PMV[slice->r][slice->s][slice->t] >> run;

      if (slice->signed_level < (-16 << slice->cc))
         slice->signed_level += 32 << slice->cc;
      else if (slice->signed_level > ((16 << slice->cc) - 1))
         slice->signed_level -= 32 << slice->cc;

      slice->PMV[slice->r][slice->s][slice->t]    = slice->signed_level << run;
      slice->vector[slice->r][slice->s][slice->t] = slice->signed_level;

MOTION_VECTOR_read_dmvector:
#ifdef _M2V_TRACE_
      if (slice->t) {
         printf("vector[%hu][%hu]=(%hd, %hd)  ",
                (u16)slice->r, (u16)slice->s,
                slice->vector[slice->r][slice->s][0],
                slice->vector[slice->r][slice->s][1]);
         printf("PMV[%hu][%hu]=(%hd, %hd)\n",
                (u16)slice->r, (u16)slice->s,
                slice->PMV[slice->r][slice->s][0],
                slice->PMV[slice->r][slice->s][1]);
      }
#endif
#ifdef _DUAL_PRIME_SUPPORT_
      if (slice->pred_type == PRED_TYPE_DUAL_PRIME) {
         GET_LSB;
         /* read dmvector */
         if (DATA_BYTE >= 0x80) {
            if (DATA_BYTE >= 0xC0)
               dmvector = -1;
            else
               dmvector =  1;
            bit_shift_inc = 2;
         }
         else {
            dmvector = 0;
            bit_shift_inc = 1;
         }
# ifdef _M2V_TRACE_
         printf("dmvector[t=%hu] = %hd\n",
                (u16)slice->t, (s16)dmvector);
# endif
         if (!slice->t) {
            /* copy top field, same parity vector */
            slice->PMV[1][0][0] = slice->PMV[0][0][0];
            
            /* start calculating opposite parity vectors */
            slice->vector[0][1][0] = slice->vector[0][0][0];         /* r=2 */
            if (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME) {
               slice->vector[1][0][0]   = slice->vector[0][0][0];    /* r=1 */
               slice->vector[1][1][0]   = slice->vector[0][0][0] * 3;
               slice->vector[1][1][0] >>= 1;
               slice->vector[1][1][0]  += (s16)dmvector;             /* r=3 */
            }
            slice->vector[0][1][0] >>= 1;
            slice->vector[0][1][0]  += (s16)dmvector;                /* r=2 */

            slice->t = 1;
            tmp = MOTION_VECTOR_2;
         }
         else {
            /* copy top field, same parity vector */
            slice->PMV[1][0][1] = slice->PMV[0][0][1];
            
            slice->mv_blk[1] = slice->ref_blkF; /* mv_blk[0] is set earlier */

            /* finish calculating opposite parity vectors */            
            slice->vector[0][1][1] = slice->vector[0][0][1];         /* r=2 */
            if (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME) {
               slice->mv_blk[2] = slice->ref_blkF;
               slice->mv_blk[3] = slice->ref_blkF;
               
               slice->vector[1][0][1]   = slice->vector[0][0][1];    /* r=1 */
               slice->vector[1][1][1]   = slice->vector[0][0][1] * 3;
               slice->vector[1][1][1] >>= 1;
               slice->vector[1][1][1]  += (s16)dmvector + 1;         /* r=3 */

               dmvector--;                  
               slice->mv_blk[1] += slice->frame_wid;
               slice->mv_blk[2] += slice->frame_wid;
            }
            else if (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_TOP_FIELD) {
               dmvector--;
               slice->mv_blk[1] += slice->frame_wid;
            }
            else {
               dmvector++;
               slice->mv_blk[0] += slice->frame_wid;
            }
            slice->vector[0][1][1] >>= 1;
            slice->vector[0][1][1]  += (s16)dmvector;                /* r=2 */
# ifdef _M2V_TRACE_
            printf("vector[0][0]=(%hd, %hd)  ",
                   slice->vector[0][0][0],
                   slice->vector[0][0][1]);
            printf("vector[2][0]=(%hd, %hd)\n",
                   slice->vector[0][1][0],
                   slice->vector[0][1][1]);
            if (PIC_HEADER_STRUCTURE(slice->pic_hdr) == STRUCTURE_FULL_FRAME) {
               printf("vector[1][0]=(%hd, %hd)  ",
                      slice->vector[1][0][0],
                      slice->vector[1][0][1]);
               printf("vector[3][0]=(%hd, %hd)\n",
                      slice->vector[1][1][0],
                      slice->vector[1][1][1]);
            }
# endif
            tmp = MACROBLOCK_5;
         }

         ALIGN_DATA_BYTE(tmp, bit_shift_inc);
         if (tmp == MOTION_VECTOR_2)
            goto MOTION_VECTOR_read;      /* read vertical component of motion vector */
      }
      else
#endif
      if (!slice->t) {
         slice->t = 1;
         goto MOTION_VECTOR_read;         /* read vertical component of motion vector */
      }
      else if (++(slice->r) < slice->mv_count)
         goto MOTION_VECTOR_read_sel_2;   /* read the next motion vector */
      else if (!slice->s)
         goto MACROBLOCK_read_MOTION_BCK; /* read the backward prediction vectors */

   /* MACROBLOCK read marker bit */
/*   case MACROBLOCK_5: */
MACROBLOCK_read_marker_bit:
      if (!MB_INTRA(slice)) {
         /* update PMVs if necessary */
         run   = PIC_HEADER_STRUCTURE(slice->pic_hdr);
         level = slice->pred_type;
         if ( ((run == STRUCTURE_FULL_FRAME)&&(level == PRED_TYPE_FRAME_BASED))||
              ((run != STRUCTURE_FULL_FRAME)&&(level == PRED_TYPE_FIELD_BASED)) ) {
            if (MB_MOTION_FWD(slice)) {
               slice->PMV[1][0][0] = slice->PMV[0][0][0];
               slice->PMV[1][0][1] = slice->PMV[0][0][1];
            }
            if (MB_MOTION_BCK(slice)) {
               slice->PMV[1][1][0] = slice->PMV[0][1][0];
               slice->PMV[1][1][1] = slice->PMV[0][1][1];
            }
         }
      }   
      else if (PIC_HEADER_CONCEALMENT_VECTORS(slice->pic_hdr)) {
         /* skip marker bit */
         GET_LSB;
         ALIGN_DATA_BYTE(MACROBLOCK_6, 1);
      }

/*   case MACROBLOCK_6: */
MACROBLOCK_6_START:
      /* slice->r = decoded cbp value */
      if (!MB_PATTERN(slice)) {
         slice->r = 0;
         goto BLOCKS_read_start;
      }

/*CODED_BLOCK_PATTERN_read:*/
      GET_LSB;
      /* coded block pattern VLC: BYTE 1 */
      if (DATA_BYTE >= 0x04) {
         if (DATA_BYTE >= 0xE0) {         /*-111-*/
            run = 60;
            bit_shift_inc = 3;
         }
         else if (DATA_BYTE >= 0xA0) {
            if (DATA_BYTE >= 0xD0)        /*-1101-*/
               run =  4;
            else if (DATA_BYTE >= 0xC0)   /*-1100-*/
               run =  8;
            else if (DATA_BYTE >= 0xB0)   /*-1011-*/
               run = 16;
            else                          /*-1010-*/
               run = 32;
            bit_shift_inc = 4;
         }
         else {
            if (DATA_BYTE >= 0x40) {      /* 1001 1 -> 0100 0 */
               tmp           = (u32)((DATA_BYTE >> 3) - 0x08);
               bit_shift_inc = 5;
            }
            else if (DATA_BYTE >= 0x30) { /* 0011 XX */
               tmp           = (u32)(DATA_BYTE >> 2);
               bit_shift_inc = 6;
            }
            else if (DATA_BYTE >= 0x20) { /* 0010 XXX */
               tmp           = (u32)(DATA_BYTE >> 1);
               bit_shift_inc = 7;
            }
            else {                        /* 000X XXXX */
               tmp           = (u32)(DATA_BYTE + (24 - 0x04));
               bit_shift_inc = 8;
            }
            run = CBP_TABLE[tmp];
         }

         /* finished reading coded block pattern VLC */
         slice->r = run;
         goto CODED_BLOCK_PATTERN_read_done;
      }

      /* coded block pattern is longer than 8 bits, first 6 bits are 0 */
      ALIGN_DATA_BYTE(MACROBLOCK_7, 6);
/*   case MACROBLOCK_7: */
MACROBLOCK_7_START:
      GET_LSB;
      /* slice->r = decoded cbp value */
      /* coded block pattern VLC: BYTE 2 */
      if (DATA_BYTE >= 0x20) {
         slice->r = CBP_TABLE[(DATA_BYTE >> 5) + (52 - 0x01)];
         bit_shift_inc = 3;
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid coded block pattern VLC2 (0x%02hX)\n",
                (u16)DATA_BYTE);
#endif
         goto SLICE_read_ERROR;
      }
CODED_BLOCK_PATTERN_read_done:
#ifdef _M2V_TRACE_
      printf("  VLC coded_block_pattern = %hu\n", (u16)slice->r);
#endif
      ALIGN_DATA_BYTE(BLOCK_0, bit_shift_inc);

/*   case BLOCK_0: */
BLOCKS_read_start:
      /* slice->s = num of BLOCKS / MACROBLOCK */
      /* slice->i = index of current BLOCK */
/*
      switch (SEQ_HEADER_CHROMA_FORMAT(slice->seq_hdr)) {
      case CHROMA_FORMAT_444:
         slice->s = 12;
         break;
      case CHROMA_FORMAT_422:
         slice->s =  8;
         break;
      case CHROMA_FORMAT_420:
      default: */
         slice->s =  6;
/*      }*/
      slice->i = 0;
      
      /* slice->r, slice->s, and slice->i set */
      /* cbp,      num_BLKS      BLK_idx      */
/*   case BLOCK_1: */
BLOCK_read:
      MV_CLEAR_BLOCK(slice->QFS);
      MV_DONE;
      slice->mb_flags &= 0xFE;
BLOCK_read_no_clear:
#ifdef _M2V_TRACE_
      block_cnt++;
      if ( (MB_INTRA(slice))&&(PIC_HEADER_INTRA_VLC_FORMAT(slice->pic_hdr)) )
         printf("DCT VLC Table 1\n");
      else
         printf("DCT VLC Table 0\n");
#endif
      if (slice->i < 4)
         slice->cc = 0;
      else if (slice->i & 0x01)
         slice->cc = 2;
      else
         slice->cc = 1;

      if ( (!(slice->r & (1 << (5-slice->i))))&&(!MB_INTRA(slice)) ) {
         /* skipping this BLOCK, go to the next */
#ifdef _M2V_TRACE_
         printf("SKIPPED BLOCK %hu\n", (u16)slice->i);
#endif
         output_block(slice);
         slice->i++;
         if (slice->i >= slice->s)
            goto MACROBLOCK_next;
         goto BLOCK_read_no_clear;
      }

      /* slice->t = index to slice->QFS[] array */
      slice->t = 0;
      
      if (!MB_INTRA(slice))
         goto FIRST_DCT_COEF_read;
      else if (slice->cc)
         goto FIRST_INTRA_CHROMA_DCT_read;

/*   case BLOCK_2: */
BLOCK_2_START:
      GET_LSB;

      /* intra_luma_dct_size VLC: BYTE 1 */
      if (DATA_BYTE < 0xFF) {
         if (DATA_BYTE < 0x80) {          /* 0 */
            if (DATA_BYTE < 0x40)         /*-00-*/
               run = 1;
            else                          /*-01-*/
               run = 2;
            bit_shift_inc = 2;
         }
         else if (DATA_BYTE < 0xE0) {
            if (DATA_BYTE < 0xA0)         /*-100-*/
               run = 0;
            else if (DATA_BYTE < 0xC0)    /*-101-*/
               run = 3;
            else                          /*-110-*/
               run = 4;
            bit_shift_inc = 3;
         }
         else if (DATA_BYTE < 0xF0) {     /*-1110-*/
            run = 5;
            bit_shift_inc = 4;
         }
         else if (DATA_BYTE < 0xF8) {     /*-1111 0-*/
            run = 6;
            bit_shift_inc = 5;
         }
         else if (DATA_BYTE < 0xFC) {     /*-1111 10-*/
            run = 7;
            bit_shift_inc = 6;
         }
         else if (DATA_BYTE < 0xFE) {     /*-1111 110-*/
            run = 8;
            bit_shift_inc = 7;
         }
         else {                           /*-1111 1110-*/
            run = 9;
            bit_shift_inc = 8;
         }

         /* finished reading intra_luma_dct_size VLC */
         goto INTRA_DCT_SIZE_done;
      }

      /* intra_luma_dct_size VLC is longer than 8 bits, first 8 bits are set */
      ALIGN8_DATA_BYTE(BLOCK_3);
/*   case BLOCK_3: */
BLOCK_3_START:
      GET_LSB;

      /* intra_luma_dct_size VLC: BYTE 2 */
      if (DATA_BYTE >= 0x80)  /*-1111 1111 1-*/
         run = 11;
      else                    /*-1111 1111 0-*/
         run = 10;
      bit_shift_inc = 1;

      /* finished reading intra_luma_dct_size VLC */
      goto INTRA_DCT_SIZE_done;

/*   case BLOCK_4: */
FIRST_INTRA_CHROMA_DCT_read:
      GET_LSB;

      /* intra_chroma_dct_size VLC: BYTE 1 */
      if (DATA_BYTE < 0xFF) {
         if (DATA_BYTE < 0xC0) {          /*-00-*/
                                          /*-01-*/
                                          /*-10-*/
            run = DATA_BYTE >> 6;
            bit_shift_inc = 2;
         }
         else if (DATA_BYTE < 0xE0) {     /*-110-*/
            run = 3;
            bit_shift_inc = 3;
         }
         else if (DATA_BYTE < 0xF0) {     /*-1110-*/
            run = 4;
            bit_shift_inc = 4;
         }
         else if (DATA_BYTE < 0xF8) {     /*-1111 0-*/
            run = 5;
            bit_shift_inc = 5;
         }
         else if (DATA_BYTE < 0xFC) {     /*-1111 10-*/
            run = 6;
            bit_shift_inc = 6;
         }
         else if (DATA_BYTE < 0xFE) {     /*-1111 110-*/
            run = 7;
            bit_shift_inc = 7;
         }
         else {                           /*-1111 1110-*/
            run = 8;
            bit_shift_inc = 8;
         }

         /* finished reading intra_chroma_dct_size VLC */
         goto INTRA_DCT_SIZE_done;
      }

      /* intra_chroma_dct_size VLC is longer than 8 bits, first 8 bits are set */
      ALIGN8_DATA_BYTE(BLOCK_5);
/*   case BLOCK_5: */
BLOCK_5_START:
      GET_LSB;

      /* intra_chroma_dct_size VLC: BYTE 2 */
      if (DATA_BYTE < 0x80) {    /*-1111 1111 0-*/
         run = 9;
         bit_shift_inc = 1;
      }
      else {                     /* 1111 1111 1 */
         if (DATA_BYTE < 0xC0)   /*-1111 1111 10-*/
            run = 10;
         else                    /*-1111 1111 11-*/
            run = 11;
         bit_shift_inc = 2;
      }
INTRA_DCT_SIZE_done:
      slice->dct_size = run;
      ALIGN_DATA_BYTE(BLOCK_6, bit_shift_inc);

      /* slice->r, slice->s, slice->t, slice->i, and slice->cc set */
      /* cbp,      num_BLKS, QFS_idx,  BLK_idx                     */
/*   case BLOCK_6: */
BLOCK_6_START:
#ifdef _M2V_TRACE_
      printf("  dct_dc_%s: dct_size = %hu, dct_diff = ",
             (slice->cc) ? "chroma" : "luma",
             (u16)slice->dct_size);
#endif
      if (!slice->dct_size) {
         bit_shift_inc = 0;
         slice->signed_level = slice->dct_pred[slice->cc];
#ifdef _M2V_TRACE_
         printf("0\nDC>  0 < 0, %hd\n", slice->signed_level);
#endif
         /* inverse scan location */
         run = 0;
         /* intra inverse quantization */
         slice->signed_level <<= 11 - slice->pic_hdr->intra_dc_precision;
         goto DCT_COEF_saturation;
      }

      GET_LSB;
      /* read dct_diff VLC: BYTE 1 */
      if (slice->dct_size <= 8) {
         bit_shift_inc       = slice->dct_size;
         slice->signed_level = (s16)(DATA_BYTE >> (8 - slice->dct_size));
         goto PRED_DC_DCT_COEF;
      }

      slice->signed_level = (s16)DATA_BYTE << (slice->dct_size - 8);
      ALIGN8_DATA_BYTE(BLOCK_7);
/*   case BLOCK_7: */
BLOCK_7_START:
      GET_LSB;
      /* read dct_diff VLC: BYTE 2 */
      bit_shift_inc        = slice->dct_size - 8;
      slice->signed_level |= (s16)(DATA_BYTE >> (8 - bit_shift_inc));

PRED_DC_DCT_COEF:
      if (slice->signed_level < (s16)(1 << (slice->dct_size - 1)))
         slice->signed_level -= (s16)(1 << slice->dct_size) - 1;
#ifdef _M2V_TRACE_
      printf("%hd\n", slice->signed_level);
#endif
      slice->signed_level       += slice->dct_pred[slice->cc];
      slice->dct_pred[slice->cc] = slice->signed_level;
#ifdef _M2V_TRACE_
      printf("DC>  0 < 0, %hd\n", slice->signed_level);
#endif
      /* inverse scan location */
      run = 0;
      /* intra inverse quantization */
      slice->signed_level <<= 11 - slice->pic_hdr->intra_dc_precision;
      goto DCT_COEF_saturation;

/*   case BLOCK_8: */
NEXT_DCT_COEF_read:
      if ( (MB_INTRA(slice))&&(PIC_HEADER_INTRA_VLC_FORMAT(slice->pic_hdr)) )
         goto TABLE1_DCT_COEF_read;
      else
         goto TABLE0_DCT_COEF_read;

/*   case BLOCK_9: */
FIRST_DCT_COEF_read:
      GET_LSB;

      if (DATA_BYTE >= 0x80) {   /*-1-*/
         run   = 0;
         level = 1;
         bit_shift_inc = 1;
         goto DCT_COEF_read_done;
      }
      goto TABLE0_DCT_COEF_read_cont;

/*   case BLOCK_10: */
TABLE0_DCT_COEF_read:
      GET_LSB;

      /* DCT_COEF VLC: BYTE 1 */
      if (DATA_BYTE >= 0x80) {      /* 1 */
         bit_shift_inc = 2;
         if (DATA_BYTE < 0xC0)      /*-10-*/
            goto BLOCK_read_done;
                                    /*-11-*/
         run   = 0;
         level = 1;

         /* finished reading DCT_COEF VLC */
         goto DCT_COEF_read_done;
      }

TABLE0_DCT_COEF_read_cont:
      if (DATA_BYTE >= 0x04) {
         if (DATA_BYTE >= 0x60) {         /*-011-*/
            bit_shift_inc = 3;
            run   = 1;
            level = 1;
         }
         else if (DATA_BYTE >= 0x40) {
            bit_shift_inc = 4;
            if (DATA_BYTE >= 0x50) {      /*-0101-*/
               run   = 2;
               level = 1;
            }
            else {                        /*-0100-*/
               run   = 0;
               level = 2;
            }
         }
         else if (DATA_BYTE >= 0x28) {
            bit_shift_inc = 5;
            if (DATA_BYTE >= 0x30) {
               level = 1;
               if (DATA_BYTE >= 0x38)     /*-0011 1-*/
                  run = 3;
               else                       /*-0011 0-*/
                  run = 4;
            }
            else {                        /*-0010 1-*/
               run   = 0;
               level = 3;
            }
         }
         else if (DATA_BYTE >= 0x20) {    /* 0010 0XXX */
            bit_shift_inc = 8;
            tmp   = (u32)DATA_BYTE - (32-8);
            run   = DCT_TABLE0[tmp][0];
            level = DCT_TABLE0[tmp][1];
         }
         else {
            if (DATA_BYTE >= 0x10) {      /* 0001 XX */
               bit_shift_inc = 6;
               tmp           = (u32)(DATA_BYTE >> 2) - 4;
            }
            else if (DATA_BYTE >= 0x08) { /* 0000 1XX */
               bit_shift_inc = 7;
               tmp           = (u32)(DATA_BYTE >> 1);    /* + (4-4);*/
            }
            else {                        /*-0000 01-*/
               bit_shift_inc = 6;
               goto DCT_COEF_read_escape;
            }
            run   = DCT_TABLE0[tmp][0];
            level = DCT_TABLE0[tmp][1];
         }
         /* finished reading DCT_COEF VLC */
         goto DCT_COEF_read_done;
      }
      /* DCT_COEF VLC is longer than 8 bits, first 6 bits are zero */
      ALIGN_DATA_BYTE(BLOCK_11, 6);
/*   case BLOCK_11: */
BLOCK_11_START:
      GET_LSB;

      /* DCT_COEF VLC: BYTE 2 */
      if (DATA_BYTE >= 0x10) {
         if (DATA_BYTE >= 0x80) {      /* 0000 001X XX */
            bit_shift_inc = 4;
            tmp           = (u32)(DATA_BYTE >> 4) + (16-8);
         }
         else if (DATA_BYTE >= 0x40) { /* 0000 0001 XXXX */
            bit_shift_inc = 6;
            tmp           = (u32)(DATA_BYTE >> 2) + (24-16);
         }
         else if (DATA_BYTE >= 0x20) { /* 0000 0000 1XXX X */
            bit_shift_inc = 7;
            tmp           = (u32)(DATA_BYTE >> 1) + (40-16);
         }
         else {                        /* 0000 0000 01XX XX */
            bit_shift_inc = 8;
            tmp           = (u32)DATA_BYTE + (56-16);
         }
         run   = DCT_TABLE0[tmp][0];
         level = DCT_TABLE0[tmp][1];

         /* finished reading DCT_COEF VLC */
         goto DCT_COEF_read_done;
      }
      /* DCT_COEF VLC is longer than 14 bits, first 10 bits are zero */
      ALIGN_DATA_BYTE(BLOCK_12, 4);
/*   case BLOCK_12: */
BLOCK_12_START:
      GET_LSB;

      /* DCT_COEF VLC: BYTE 3 */
      if (DATA_BYTE >= 0x80) {      /* 0000 0000 001X XXX */
         bit_shift_inc = 5;
         tmp           = (u32)(DATA_BYTE >> 3) + (72-16);
      }
      else if (DATA_BYTE >= 0x40) { /* 0000 0000 0001 XXXX */
         bit_shift_inc = 6;
         tmp           = (u32)(DATA_BYTE >> 2) + (88-16);
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid DCT_COEF Table0 VLC\n");
#endif
         goto SLICE_read_ERROR;
      }
      run   = DCT_TABLE0[tmp][0];
      level = DCT_TABLE0[tmp][1];

      /* finished reading DCT_COEF VLC */
      goto DCT_COEF_read_done;

/*   case BLOCK_13: */
TABLE1_DCT_COEF_read:
      GET_LSB;

      /* DCT_COEF VLC: BYTE 1 */
      if (DATA_BYTE >= 0x04) {
         if (DATA_BYTE >= 0x80) {            /* 1 */
            if (DATA_BYTE < 0xC0) {          /*-10-*/
               bit_shift_inc = 2;
               run   =  0;
               level =  1;
            }
            else if (DATA_BYTE < 0xE0) {     /*-110-*/
               bit_shift_inc = 3;
               run   =  0;
               level =  2;
            }
            else if (DATA_BYTE < 0xF0) {     /* 1110 */
               bit_shift_inc = 5;
               run   = 0;
               if (DATA_BYTE >= 0xE8)        /*-1110 1-*/
                  level = 5;
               else                          /*-1110 0-*/
                  level = 4;
            }
            else if (DATA_BYTE <= 0xF9) {    /* 1111 XXX */
               bit_shift_inc = 7;
               tmp   = (u32)(DATA_BYTE >> 1) - 120;
               run   = DCT_TABLE1[tmp][0];
               level = DCT_TABLE1[tmp][1];
            }
            else {                           /* 1111 1XXX */
               bit_shift_inc = 8;
               tmp   = (u32)DATA_BYTE - (250-5);
               run   = DCT_TABLE1[tmp][0];
               level = DCT_TABLE1[tmp][1];
            }
         }
         else if (DATA_BYTE >= 0x60) {       /* 011 */
            bit_shift_inc = 4;
            if (DATA_BYTE >= 0x70) {         /*-0111-*/
               run   = 0;
               level = 3;
            }
            else                             /*-0110-*/
               goto BLOCK_read_done;
         }
         else if (DATA_BYTE >= 0x40) {       /*-010-*/
            bit_shift_inc = 3;
            run   = 1;
            level = 1;
         }
         else if (DATA_BYTE >= 0x20) {       /* 001 */
            if (DATA_BYTE >= 0x28) {
               bit_shift_inc = 5;
               if (DATA_BYTE >= 0x38) {      /*-0011 1-*/
                  run   = 3;
                  level = 1;
               }
               else if (DATA_BYTE >= 0x30) { /*-0011 0-*/
                  run   = 1;
                  level = 2;
               }
               else {                        /*-0010 1-*/
                  run   = 2;
                  level = 1;
               }
            }
            else {                           /* 0010 0XXX */
               bit_shift_inc = 8;
               tmp   = (u32)DATA_BYTE - (32-11);
               run   = DCT_TABLE1[tmp][0];
               level = DCT_TABLE1[tmp][1];
            }
         }
         else if (DATA_BYTE >= 0x10) {       /* 0001 XX */
            bit_shift_inc = 6;
            tmp   = (u32)(DATA_BYTE >> 2) + (19-4);
            run   = DCT_TABLE1[tmp][0];
            level = DCT_TABLE1[tmp][1];
         }
         else if (DATA_BYTE >= 0x08) {       /* 0000 1XX */
            bit_shift_inc = 7;
            tmp   = (u32)(DATA_BYTE >> 1) + (23-4);
            run   = DCT_TABLE1[tmp][0];
            level = DCT_TABLE1[tmp][1];
         }
         else {                              /*-0000 01-*/
            bit_shift_inc = 6;
            goto DCT_COEF_read_escape;
         }
         /* finished reading DCT_COEF VLC */
         goto DCT_COEF_read_done;
      }
      /* DCT_COEF VLC is longer than 8 bits, first 6 bits are zero */
      ALIGN_DATA_BYTE(BLOCK_14, 6);
/*   case BLOCK_14: */
BLOCK_14_START:
      GET_LSB;

      /* DCT_COEF VLC: BYTE 2 */
      if (DATA_BYTE >= 0x10) {
         if (DATA_BYTE >= 0xE0) {         /*-0000 0011 1-*/
            bit_shift_inc = 3;
            run   = 15;
            level =  1;
         }
         else if (DATA_BYTE >= 0xC0) {    /* 0000 0011 0 */
            bit_shift_inc = 4;
            if (DATA_BYTE >= 0xD0) {      /*-0000 0011 01-*/
               run   = 16;
               level =  1;
            }
            else {                        /*-0000 0011 00-*/
               run   =  2;
               level =  4;
            }
         }
         else if (DATA_BYTE >= 0x80) {    /* 0000 0010 */
            bit_shift_inc = 3;
            if (DATA_BYTE >= 0xA0) {      /*-0000 0010 1-*/
               run   = 14;
               level =  1;
            }
            else {                        /*-0000 0010 0-*/
               run   =  5;
               level =  2;
            }
         }
         else if (DATA_BYTE >= 0x40) {
            bit_shift_inc = 6;
            if (DATA_BYTE >= 0x7C) {      /*-0000 0001 1111-*/
               run   = 17;
               level =  1;
            }
            else if (DATA_BYTE >= 0x78) { /*-0000 0001 1110-*/
               run   =  6;
               level =  2;
            }
            else if (DATA_BYTE >= 0x74) { /* 0000 0001 1101 */
#ifdef _SHOW_ERRORS_
               printf("\nInvalid DCT_COEF Table1 VLC (0000 0001 1101)\n");
#endif
               goto SLICE_read_ERROR;
            }
            else if (DATA_BYTE >= 0x70) { /*-0000 0001 1100-*/
               run   =  3;
               level =  3;
            }
            else if (DATA_BYTE >= 0x6C) { /* 0000 0001 1011 */
#ifdef _SHOW_ERRORS_
               printf("\nInvalid DCT_COEF Table1 VLC (0000 0001 1011)\n");
#endif
               goto SLICE_read_ERROR;
            }
            else if (DATA_BYTE >= 0x68) { /*-0000 0001 1010-*/
               run   = 18;
               level =  1;
            }
            else if (DATA_BYTE >= 0x64) { /*-0000 0001 1001-*/
               run   = 19;
               level =  1;
            }
            else if (DATA_BYTE >= 0x60) { /*-0000 0001 1000-*/
#ifdef _SHOW_ERRORS_
               printf("\nInvalid DCT_COEF Table1 VLC (0000 0001 1000)\n");
#endif
               goto SLICE_read_ERROR;
            }
            else if (DATA_BYTE >= 0x58) {
               level = 1;
               if (DATA_BYTE >= 0x5C)     /*-0000 0001 0111-*/
                  run = 20;
               else                       /*-0000 0001 0110-*/
                  run = 21;
            }
            else if (DATA_BYTE >= 0x54) { /*-0000 0001 0101-*/
               run   = 7;
               level = 2;
            }
            else if (DATA_BYTE >= 0x4C) { /* 0000 0001 0100 */
                                          /* 0000 0001 0011 */
#ifdef _SHOW_ERRORS_
               printf("\nInvalid DCT_COEF Table1 VLC (0000 0001 0100) or "
                      "(0000 0001 0011)\n");
#endif
               goto SLICE_read_ERROR;
            }
            else if (DATA_BYTE >= 0x48) { /*-0000 0001 0010-*/
               run   = 4;
               level = 3;
            }
            else if (DATA_BYTE >= 0x44) { /*-0000 0001 0001-*/
               run   = 8;
               level = 2;
            }
            else {                        /* 0000 0001 0000 */
#ifdef _SHOW_ERRORS_
               printf("\nInvalid DCT_COEF Table1 VLC (0000 0001 0000)\n");
#endif
               goto SLICE_read_ERROR;
            }
         }
         else if (DATA_BYTE >= 0x20) {    /* 0000 0000 1 */
            bit_shift_inc = 7;
            if (DATA_BYTE >= 0x38) {      /* 0000 0000 111 */
               run   = 25 - ((DATA_BYTE & 0x06) >> 1);
               level = 1;
            }
            else if (DATA_BYTE >= 0x36) { /*-0000 0000 1101 1-*/
               run   = 26;
               level =  1;
            }
            else if (DATA_BYTE >= 0x2E) { /* 0000 0000 1101 0 */
                                          /* 0000 0000 1100 */
                                          /* 0000 0000 1011 1 */
#ifdef _SHOW_ERRORS_
               printf("\nInvalid DCT_COEF Table1 VLC (0000 0000 1101 0) or "
                      "(0000 0000 1100) or (0000 0000 1011 1)\n");
#endif
               goto SLICE_read_ERROR;
            }
            else if (DATA_BYTE >= 0x2C) { /*-0000 0000 1011 0-*/
               run   = 1;
               level = 6;
            }
            else if (DATA_BYTE >= 0x2A) { /*-0000 0000 1010 1-*/
               run   = 1;
               level = 7;
            }
            else if (DATA_BYTE >= 0x28) { /*-0000 0000 1010 0-*/
               run   = 2;
               level = 5;
            }
            else {                        /* 0000 0000 100X X */
               tmp   = (u32)(DATA_BYTE >> 1) + (27-16);
               run   = DCT_TABLE1[tmp][0];
               level = DCT_TABLE1[tmp][1];
            }
         }
         else {                           /* 0000 0000 01 */
            bit_shift_inc = 8;
            level = 31 - (DATA_BYTE & 0x0F);
            run   = 0;
         }
         /* finished reading DCT_COEF VLC */
         goto DCT_COEF_read_done;
      }
      /* DCT_COEF VLC is longer than 14 bits, first 10 bits are zero */
      ALIGN_DATA_BYTE(BLOCK_15, 4);
/*   case BLOCK_15: */
BLOCK_15_START:
      GET_LSB;

      /* DCT_COEF VLC: BYTE 3 */
      if (DATA_BYTE >= 0x80) {      /* 0000 0000 001X XXX */
         bit_shift_inc = 5;
         tmp           = (u32)(DATA_BYTE >> 3) + (72-16);
      }
      else if (DATA_BYTE >= 0x40) { /* 0000 0000 0001 XXXX */
         bit_shift_inc = 6;
         tmp           = (u32)(DATA_BYTE >> 2) + (88-16);
      }
      else {
#ifdef _SHOW_ERRORS_
         printf("\nInvalid DCT_COEF Table1 VLC\n");
#endif
         goto SLICE_read_ERROR;
      }
      run   = DCT_TABLE0[tmp][0];
      level = DCT_TABLE0[tmp][1];

      /* finished reading DCT_COEF VLC */

      /* slice->t */
      /* QFS_idx  */
DCT_COEF_read_done:
#ifdef _M2V_TRACE_
      printf("%2hu> %2hu <", (u16)slice->t, (u16)run);
#endif
      if ((slice->t + run) >= 64) {
         if (slice->t >= 64) {
#ifdef _SHOW_ERRORS_
            printf("\nDCT_COEF ERROR> QFS_idx(%hu) value too big\n",
                   (u16)slice->t);
#endif
            goto SLICE_read_ERROR;
         }
         slice->t  = 63;
      }
      else
         slice->t += run;

      slice->signed_level = (s16)level;
      ALIGN_DATA_BYTE(BLOCK_16, bit_shift_inc);
/*   case BLOCK_16: */
BLOCK_16_START:
      GET_LSB;

      if (DATA_BYTE & 0x80)
         slice->signed_level = -slice->signed_level;
#ifdef _M2V_TRACE_
      printf("%2hu, %hd\n", (u16)slice->t, slice->signed_level);
#endif
      bit_shift_inc = 1;
DCT_COEF_inv_quant:
      /* inverse scan location */
      if (PIC_HEADER_ALT_SCAN(slice->pic_hdr))
         run = 1;
      else
         run = 0;
      run = SCAN_TABLE[run][slice->t];

      /* inverse quantization */
      dct = (s32)slice->signed_level << 1;
      if (!MB_INTRA(slice)) {
         if (dct > 0)
            dct++;
         else if (dct < 0)
            dct--;
         dct *= (s32)slice->seq_hdr->non_intra_q_matrix[run];
      }
      else
         dct *= (s32)slice->seq_hdr->intra_q_matrix[run];
      if (!PIC_HEADER_Q_SCALE_TYPE(slice->pic_hdr))
         dct *= (s32)slice->q_scale_code << 1;
      else
         dct *= (s32)QUANT_SCALE1[slice->q_scale_code];
      slice->signed_level = (s16)(dct / 32);

      /* run,                 and slice->t */
      /* inverse_scan_location    QFS_idx  */
DCT_COEF_saturation:
      /* saturation */
      if (slice->signed_level > 2047)
         slice->signed_level =  2047;
      else if (slice->signed_level < -2048)
         slice->signed_level = -2048;

      /* mismatch control sum */
      if (slice->signed_level & 0x0001)
         slice->mb_flags ^= MB_MISMATCH_BIT;

      slice->QFS[run] = slice->signed_level;
      slice->t++;
      ALIGN_DATA_BYTE(BLOCK_8, bit_shift_inc);
      goto NEXT_DCT_COEF_read;

      /* slice->t */
      /* QFS_idx  */
DCT_COEF_read_escape:
      ALIGN_DATA_BYTE(BLOCK_17, bit_shift_inc);
/*   case BLOCK_17: */
BLOCK_17_START:
      GET_LSB;

      /* read 6 bit run code */
      run = DATA_BYTE >> 2;
#ifdef _M2V_TRACE_
      printf("%2hu> %2hu <", (u16)slice->t, (u16)run);
#endif
      if ((slice->t + run) >= 64) {
         if (slice->t >= 64) {
#ifdef _SHOW_ERRORS_
            printf("\nDCT_COEF ESC ERROR> QFS_idx(%hu) value too big\n",
                   (u16)slice->t);
#endif
            goto SLICE_read_ERROR;
         }
         slice->t  = 63;
      }
      else
         slice->t += run;

      /* read bits 11, and 10 of 12 bit level code */
      if (DATA_BYTE & 0x02)
         slice->signed_level = (s16)0xF000;
      else
         slice->signed_level = 0;
      slice->signed_level |= (s16)(DATA_BYTE & 0x03) << 10;
      ALIGN8_DATA_BYTE(BLOCK_18);
/*   case BLOCK_18: */
BLOCK_18_START:
      GET_LSB;

      /* read bits 9 to 2 of 12 bit level code */
      slice->signed_level |= (s16)DATA_BYTE << 2;
      ALIGN8_DATA_BYTE(BLOCK_19);
/*   case BLOCK_19: */
BLOCK_19_START:
      GET_LSB;

      /* read bits 1, and 0 of 12 bit level code */
      slice->signed_level |= (s16)(DATA_BYTE >> 6);
#ifdef _M2V_TRACE_
      printf("%2hu, %hd  ESC\n", (u16)slice->t, slice->signed_level);
#endif
      bit_shift_inc = 2;
      goto DCT_COEF_inv_quant;

      /* slice->s, and slice->i set */
      /* num_BLKS,     BLK_idx      */
/*   case BLOCK_20: */
BLOCK_read_done:
#ifdef _M2V_TRACE_
      printf("EOB>%2d\n", block_cnt);
#endif
      /* mismatch control */
      if (!(slice->mb_flags & MB_MISMATCH_BIT)) {
         if (slice->QFS[63] & 0x0001)
            slice->QFS[63]--;
         else
            slice->QFS[63]++;
      }
      
#ifdef _M2V_IDCT_TRACE_
/*if ( (frame_cnt == 6)&&(slice->cur_col == 6)&&(slice->cur_row == 0) )
*/{
      printf("DCT coefficients for MB %d, block %hu\n", mb_cnt, (u16)slice->i);
      show_QFS(slice->QFS);
}
#endif
      idct(slice->QFS);
#ifdef _M2V_IDCT_TRACE_
/*if ( (frame_cnt == 6)&&(slice->cur_col == 6)&&(slice->cur_row == 0) )
*/{
      printf("Recovered samples\n");
      show_QFS(slice->QFS);
}
#endif
      output_block(slice);
      
      slice->i++;
      if (slice->i < slice->s)
         tmp = BLOCK_1;       /* BLOCK_read */
      else if ( (!slice->cur_col)&&(slice->cur_row == slice->mb_hgt) ) {
#ifdef _M2V_TRACE_
         printf("END PICTURE\n");
#endif
         *cur_pos = 0;
         *code    = 0x80FF;   /* finished with this picture */
         goto SLICE_read_END;
      }
      else
         tmp = MACROBLOCK_8;  /* MACROBLOCK_next */
      
      ALIGN_DATA_BYTE(tmp, bit_shift_inc);
      if (slice->i < slice->s)
         goto BLOCK_read;

/*   case MACROBLOCK_8: */
MACROBLOCK_next:
      GET_LSB;
#ifdef _M2V_TRACE_
         block_cnt = -1;
         printf("\n");
#endif
      
      /* no more MACROBLOCKs exist if there are 23 zero bits here */
      if (DATA_BYTE)
         goto MACROBLOCK_read_addr_inc;
      
      /* 8 zero bits, since end of last MACROBLOCK */
      slice->t      = DATA_BYTE;
      slice->i      = BIT_SHIFT_BYTE;
      slice->buf[0] = *ptr;
      ALIGN8_DATA_BYTE(MACROBLOCK_9);
/*   case MACROBLOCK_9: */
MACROBLOCK_9_START:
      GET_LSB;

      if (DATA_BYTE) {
         tmp = 1;
MACROBLOCK_next_start:
         slice->data      = slice->t;
         slice->bit_shift = slice->i;
         *cur_pos         = MACROBLOCK_0; /* MACROBLOCK_read_addr_inc */
         SLICE_read(slice->buf, tmp, slice, cur_pos, code);
         if (*code == 0x0FFF)
            goto SLICE_read_END;
         goto START_AGAIN;
      }

      /* 16 zero bits, since end of last MACROBLOCK */
      slice->buf[1] = *ptr;
      ALIGN8_DATA_BYTE(MACROBLOCK_10);
/*   case MACROBLOCK_10: */
MACROBLOCK_10_START:
      GET_LSB;

      if (DATA_BYTE & 0xFE) {
         tmp = 2;
         goto MACROBLOCK_next_start;
      }

      /* 23 zero bits, since end of last MACROBLOCK (SLICE has ended) */
      if (!slice->buf[0])
         *cur_pos  = 1; /* found 1st byte of next start_code ? */
      else
         *cur_pos  = 0;
      if (!slice->buf[1])
         *cur_pos += 1; /* found 1st, or 2nd byte of next start_code ? */

      if ( (*cur_pos == 2)&&(*ptr == 1)&&((u32)(ptr+2) < data_len)&&
           (0x01 < *(ptr+1))&&(*(ptr+1) <= 0xAF) ) {
/*         printf("Found next SLICE!\n"
                "cur_pos=%u, data=0x%02hX, "
                "ptr[0]=0x%02hX, ptr[1]=0x%02hX, ptr[2]=0x%02hX\n\n",
                *cur_pos, (u16)data, (u16)ptr[0], (u16)ptr[1], (u16)ptr[2]); */
         *code = (u16)(*(ptr+1));
         ptr  += 2;
         goto SLICE_start_new;
      }
/*      else {
         printf("Did not find next SLICE\n"
                "cur_pos=%u, data=0x%02hX, "
                "ptr[0]=0x%02hX, ptr[1]=0x%02hX, ptr[2]=0x%02hX\n\n",
                *cur_pos, (u16)data, (u16)ptr[0], (u16)ptr[1], (u16)ptr[2]);
      } */

      *code |= 0x8000;
/*
      printf("  0x%02hX 0x%02hX 0x%02hX  (%u)\n\n", (u16)slice->buf[0],
                                                    (u16)slice->buf[1],
                                                    (u16)(*ptr),
                                                    *cur_pos);
*/

SLICE_read_END:
#ifdef USE_LOCAL_VARS
   slice->bit_shift = bit_shift;
   slice->data      = data;
#endif
   return (u32)(ptr - data_stream);

SLICE_read_ERROR:
#ifdef _M2V_TRACE_
   printf("SLICE_read_ERROR:\n");
   printf("   frame=%d, slice=%d, mblock=%d, block=%d\n", frame_cnt,
                                                          slice_cnt,
                                                          mb_cnt,
                                                          block_cnt);
   if (block_cnt >= 0) {
      /* slice->r, slice->s, slice->t, slice->i, and slice->cc set */
      /* cbp,      num_BLKS, QFS_idx,  BLK_idx                     */
      printf("   BLK_idx=%hu, num_BLKS=%hu, cc=%hu, QFS_idx=%hu, TABLE=",
             (u16)slice->i,
             (u16)slice->s,
             (u16)slice->cc,
             (u16)slice->t);

      if ( (MB_INTRA(slice))&&(PIC_HEADER_INTRA_VLC_FORMAT(slice->pic_hdr)) )
         printf("1\n\n");
      else
         printf("0\n\n");
   }
   else
      printf("\n");
/*   frame_cnt = -100; */
#endif
   *cur_pos = 0;
   *code    = 0x0FFF;
   ptr++;
   return (u32)(ptr - data_stream);

#undef SLICE_0
#undef SLICE_1
#undef SLICE_2
#undef MACROBLOCK_0
#undef MACROBLOCK_00
#undef MACROBLOCK_1
#undef MACROBLOCK_MODES_0
#undef MACROBLOCK_MODES_1
#undef MACROBLOCK_MODES_2
#undef MACROBLOCK_MODES_3
#undef MACROBLOCK_MODES_4
#undef MACROBLOCK_2
#undef MACROBLOCK_3
#undef MACROBLOCK_4
#undef MOTION_VECTOR_1
#undef MOTION_VECTOR_2
#undef MOTION_VECTOR_3
#undef MOTION_VECTOR_4
#undef MOTION_VECTOR_5
#undef MOTION_VECTOR_6
#undef MACROBLOCK_5
#undef MACROBLOCK_6
#undef MACROBLOCK_7
#undef BLOCK_0
#undef BLOCK_1
#undef BLOCK_2
#undef BLOCK_3
#undef BLOCK_4
#undef BLOCK_5
#undef BLOCK_6
#undef BLOCK_7
#undef BLOCK_8
#undef BLOCK_9
#undef BLOCK_10
#undef BLOCK_11
#undef BLOCK_12
#undef BLOCK_13
#undef BLOCK_14
#undef BLOCK_15
#undef BLOCK_16
#undef BLOCK_17
#undef BLOCK_18
#undef BLOCK_19
#undef BLOCK_20
#undef MACROBLOCK_8
#undef MACROBLOCK_9
#undef MACROBLOCK_10
}     

#ifdef _M2V_IDCT_TRACE_
static void show_QFS(s16 *QFS)
{
   int tmp;

      for (tmp = 0; tmp < 8; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 16; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 24; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 32; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 40; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 48; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 56; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
      for (; tmp < 64; tmp++) {
         printf("%6hd", QFS[tmp]);
      }
      printf("\n");
}
#endif

#undef RESET_DCT_PRED
#undef RESET_PMV
#undef DATA_BYTE
#undef BIT_SHIFT_BYTE
#undef DATA_PTR
#undef END_DATA_PTR
#undef PARSER_STATE
#undef STOP_PARSING

