/***************************************************************************
                   mmx_gcc_motion_vector.h  -  description
                             -------------------
   MMX versions of the motion vector functions (GCC inline assembly).
                             -------------------
    begin                : Tue Dec 28 2004
    copyright            : (C) 2004 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Copy a frame. */
void MV_COPY_FRAME(void *to, void *from, u32 size)
{  __asm__ __volatile__(
      "MVCF_LOOP:\n\t"
      "movq "     "(%1), " "%%mm0\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "$8, "   "%1\n\t"
      "addl "     "$8, "   "%0\n\t"
      "subl "     "$8, "   "%2\n\t"
      "jg "       "MVCF_LOOP\n\t"
      "emms"
   : : "q" (to), "q" (from), "r" ((int)size));
}

/* Clear a block. */
void MV_CLEAR_BLOCK(s16 *blk)
{  __asm__ __volatile__(
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVCB_LOOP:\n\t"
      "movq "     "%%mm7, ""(%0)\n\t"
      "movq "     "%%mm7, ""8(%0)\n\t"
      "addl "     "$16, "  "%0\n\t"
      "dec "               "%1\n\t"
      "jnz "      "MVCB_LOOP\n\t"
      "subl "     "$128, " "%0"
   : : "q" (blk), "r" ((int)8));
}
      

/* The MV_PRED_?? functions are used to get the predictors from
     the reference block(s). */
void MV_PRED___(s16 *inp, u8 *ref, u32 inc)
{  __asm__ __volatile__(
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVP_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%2, "   "%1\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "$16, "  "%0\n\t"
      "dec "               "%3\n\t"
      "jnz "      "MVP_LOOP\n\t"
      "subl "     "$128, " "%0"
   : : "q" (inp), "q" (ref), "r" ((int)inc), "r" ((int)8));
}

void MV_PRED_X_(s16 *inp, u8 *ref, u32 inc)
{  __asm__ __volatile__(
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVP_X_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "movq "     "1(%1), ""%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "psraw "    "$1, "   "%%mm2\n\t"
      "psraw "    "$1, "   "%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%2, "   "%1\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "$16, "  "%0\n\t"
      "dec "               "%3\n\t"
      "jnz "      "MVP_X_LOOP\n\t"
      "subl "     "$128, " "%0"
   : : "q" (inp), "q" (ref), "r" ((int)inc), "r" ((int)8));
}

void MV_PRED__Y(s16 *inp, u8 *ref, u32 inc, u32 inc_y)
{  __asm__ __volatile__(
      "addl "     "%1, "   "%2\n\t"
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVP_Y_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "movq "     "(%2), " "%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "psraw "    "$1, "   "%%mm2\n\t"
      "psraw "    "$1, "   "%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "addl "     "%3, "   "%1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%3, "   "%2\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "$16, "  "%0\n\t"
      "dec "               "%4\n\t"
      "jnz "      "MVP_Y_LOOP\n\t"
      "subl "     "$128, " "%0"
   : : "q" (inp), "q" (ref), "q" ((int)inc_y), 
       "r" ((int)inc), "r" ((int)8));
}

void MV_PRED_XY(s16 *inp, u8 *ref, u32 inc, u32 inc_y)
{  __asm__ __volatile__(
      "addl "     "%1, "   "%2\n\t"
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVP_XY_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "movq "     "1(%1), ""%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "movq "     "(%2), " "%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "movq "     "1(%2), ""%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "psraw "    "$2, "   "%%mm2\n\t"
      "psraw "    "$2, "   "%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "addl "     "%3, "   "%1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%3, "   "%2\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "$16, "  "%0\n\t"
      "dec "               "%4\n\t"
      "jnz "      "MVP_XY_LOOP\n\t"
      "subl "     "$128, " "%0"
   : : "q" (inp), "q" (ref), "q" ((int)inc_y), 
       "r" ((int)inc), "r" ((int)8));
}

   
/* The MV_FIELD_PRED_?? functions are similar to the MV_PRED_?? functions,
     except these functions are used when different reference blocks are
     needed for each field. */
void MV_FIELD_PRED___(s16 *inp0, u32 inp_inc, 
                      u8  *ref0, u32 inc)
{  __asm__ __volatile__(
      "shll "     "$1, "   "%3\n\t"
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVFP_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%2, "   "%1\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "%3, "   "%0\n\t"
      "dec "               "%4\n\t"
      "jnz "      "MVFP_LOOP"
   : : "q" (inp0), "q" (ref0), "r" ((int)inc),
       "r" ((int)inp_inc), "r" ((int)4));
}

void MV_FIELD_PRED_X_(s16 *inp0, u32 inp_inc, 
                      u8  *ref0, u32 inc)
{  __asm__ __volatile__(
      "shll "     "$1, "   "%3\n\t"
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVFP_X_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "movq "     "1(%1), ""%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "psraw "    "$1, "   "%%mm2\n\t"
      "psraw "    "$1, "   "%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%2, "   "%1\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "%3, "   "%0\n\t"
      "dec "               "%4\n\t"
      "jnz "      "MVFP_X_LOOP"
   : : "q" (inp0), "q" (ref0), "r" ((int)inc),
       "r" ((int)inp_inc), "r" ((int)4));
}

void MV_FIELD_PRED__Y(s16 *inp0, u32 inp_inc, 
                      u8  *ref0, u32 inc, u32 inc_y)
{  __asm__ __volatile__(
      "shll "     "$1, "   "%4\n\t"
      "addl "     "%1, "   "%2\n\t"
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVFP_Y_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "movq "     "(%2), " "%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "psraw "    "$1, "   "%%mm2\n\t"
      "psraw "    "$1, "   "%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "addl "     "%3, "   "%1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%3, "   "%2\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "%4, "   "%0\n\t"
      "dec "               "%5\n\t"
      "jnz "      "MVFP_Y_LOOP"
   : : "q" (inp0), "q" (ref0), "q" ((int)inc_y),
       "r" ((int)inc), "r" ((int)inp_inc), "r" ((int)4));
}

void MV_FIELD_PRED_XY(s16 *inp0, u32 inp_inc, 
                      u8  *ref0, u32 inc, u32 inc_y)
{  __asm__ __volatile__(
      "shll "     "$1, "   "%4\n\t"
      "addl "     "%1, "   "%2\n\t"
      "pxor "     "%%mm7, ""%%mm7\n"
      "MVFP_XY_LOOP:\n\t"
      "movq "     "(%1), " "%%mm2\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "%%mm2, ""%%mm3\n\t"
      "punpcklbw ""%%mm7, ""%%mm2\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "punpckhbw ""%%mm7, ""%%mm3\n\t"
      "movq "     "1(%1), ""%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "movq "     "(%2), " "%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "movq "     "1(%2), ""%%mm4\n\t"
      "movq "     "%%mm4, ""%%mm5\n\t"
      "punpcklbw ""%%mm7, ""%%mm4\n\t"
      "punpckhbw ""%%mm7, ""%%mm5\n\t"
      "paddsw "   "%%mm4, ""%%mm2\n\t"
      "paddsw "   "%%mm5, ""%%mm3\n\t"
      "psraw "    "$2, "   "%%mm2\n\t"
      "psraw "    "$2, "   "%%mm3\n\t"
      "paddsw "   "%%mm2, ""%%mm0\n\t"
      "paddsw "   "%%mm3, ""%%mm1\n\t"
      "addl "     "%3, "   "%1\n\t"
      "movq "     "%%mm0, ""(%0)\n\t"
      "addl "     "%3, "   "%2\n\t"
      "movq "     "%%mm1, ""8(%0)\n\t"
      "addl "     "%4, "   "%0\n\t"
      "dec "               "%5\n\t"
      "jnz "      "MVFP_XY_LOOP"
   : : "q" (inp0), "q" (ref0), "q" ((int)inc_y),
       "r" ((int)inc), "r" ((int)inp_inc), "r" ((int)4));
}


/* Saturates the final pixel values, in the block, and stores them in
     the frame buffer. */
void MV_SATURATION(s16 *blk, u8 *outp, int inc)
{  __asm__ __volatile__(
      "movd "     "%3, "   "%%mm6\n\t"
      "pxor "     "%%mm7, ""%%mm7\n\t"
      "movl "     "$8, "   "%3\n\t"
      "punpcklbw ""%%mm7, ""%%mm6\n"
      "MV_SATURATION_LOOP:\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "paddsw "   "%%mm6, ""%%mm0\n\t"
      "paddsw "   "%%mm6, ""%%mm1\n\t"
      "movq "     "%%mm0, ""%%mm2\n\t"
      "movq "     "%%mm1, ""%%mm3\n\t"
      "pcmpgtw "  "%%mm7, ""%%mm2\n\t"
      "pcmpgtw "  "%%mm7, ""%%mm3\n\t"
      "pand "     "%%mm2, ""%%mm0\n\t"
      "pand "     "%%mm3, ""%%mm1\n\t"
      "packuswb " "%%mm1, ""%%mm0\n\t"
      "addl "     "$16, "  "%0\n\t"
      "movq "     "%%mm0, ""(%1)\n\t"
      "addl "     "%2, "   "%1\n\t"
      "dec "               "%3\n\t"
      "jnz "      "MV_SATURATION_LOOP\n\t"
      "emms"
   : : "q" (blk), "q" (outp), "r" ((int)inc), "r" (0x80808080));
}


/* Adds the predictor block to the DCT block, to obtain the final pixel
     values.  The final values are stored in the frame buffer. */
void MV_ADD_PRED(s16 *blk, s16 *p_blk, u8 *outp, 
                 int  inc, u32  sh)
{  __asm__ __volatile__(
      "movd "     "%4, "   "%%mm4\n\t"
      "pxor "     "%%mm7, ""%%mm7\n\t"
      "movl "     "$8, "   "%4\n"
      "MV_ADD_PRED_LOOP:\n\t"
      "movq "     "(%1), " "%%mm0\n\t"
      "movq "     "8(%1), ""%%mm1\n\t"
      "psraw "    "%%mm4, ""%%mm0\n\t"
      "psraw "    "%%mm4, ""%%mm1\n\t"
      "paddsw "   "(%0), " "%%mm0\n\t"
      "paddsw "   "8(%0), ""%%mm1\n\t"
      "movq "     "%%mm0, ""%%mm2\n\t"
      "movq "     "%%mm1, ""%%mm3\n\t"
      "pcmpgtw "  "%%mm7, ""%%mm2\n\t"
      "pcmpgtw "  "%%mm7, ""%%mm3\n\t"
      "pand "     "%%mm2, ""%%mm0\n\t"
      "pand "     "%%mm3, ""%%mm1\n\t"
      "addl "     "$16, "  "%0\n\t"
      "packuswb " "%%mm1, ""%%mm0\n\t"
      "addl "     "$16, "  "%1\n\t"
      "movq "     "%%mm0, ""(%2)\n\t"
      "addl "     "%3, "   "%2\n\t"
      "dec "               "%4\n\t"
      "jnz "      "MV_ADD_PRED_LOOP\n\t"
      "emms"
   : : "q" (blk), "q" (p_blk), "q" (outp), "r" ((int)inc),
       "r" ((int)sh));
}


/* Copies the predictor block into the frame buffer. */
void MV_COPY_PRED(s16 *p_blk, u8 *outp, int inc, u32 sh)
{  __asm__ __volatile__(
      "movd "     "%3, "   "%%mm4\n\t"
      "pxor "     "%%mm7, ""%%mm7\n\t"
      "movl "     "$8, "   "%3\n"
      "MV_COPY_PRED_LOOP:\n\t"
      "movq "     "(%0), " "%%mm0\n\t"
      "movq "     "8(%0), ""%%mm1\n\t"
      "psraw "    "%%mm4, ""%%mm0\n\t"
      "psraw "    "%%mm4, ""%%mm1\n\t"
      "movq "     "%%mm0, ""%%mm2\n\t"
      "movq "     "%%mm1, ""%%mm3\n\t"
      "pcmpgtw "  "%%mm7, ""%%mm2\n\t"
      "pcmpgtw "  "%%mm7, ""%%mm3\n\t"
      "pand "     "%%mm2, ""%%mm0\n\t"
      "pand "     "%%mm3, ""%%mm1\n\t"
      "packuswb " "%%mm1, ""%%mm0\n\t"
      "addl "     "$16, "  "%0\n\t"
      "movq "     "%%mm0, ""(%1)\n\t"
      "addl "     "%2, "   "%1\n\t"
      "dec "               "%3\n\t"
      "jnz "      "MV_COPY_PRED_LOOP\n\t"
      "emms"
   : : "q" (p_blk), "q" (outp), "r" ((int)inc), "r" ((int)sh));
}

