/***************************************************************************
                           dct32.c  -  description
                             -------------------
    begin                : Tue Apr 26 2005
    copyright            : (C) 2005 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "dct32.h"

#if (FP_SHIFT <= 12)
# define CS1_SHIFT 0
# define CS2_SHIFT 1
#else
# define CS1_SHIFT 0
# define CS2_SHIFT 0
#endif

/* 1.0 / (2.0 * cos( ((2*k+1)*pi) / (2*N) )) */
static const AV1 cos_const[] =
{                                   /* STAGE 1: N=32 */
   (s32)FP_CONST( 0.500602998) >> CS2_SHIFT, /* k= 0 */
   (s32)FP_CONST( 0.50547096)  >> CS2_SHIFT, /* k= 1 */
   (s32)FP_CONST( 0.51544731)  >> CS2_SHIFT, /* k= 2 */
   (s32)FP_CONST( 0.531042591) >> CS2_SHIFT, /* k= 3 */
   (s32)FP_CONST( 0.553103896) >> CS2_SHIFT, /* k= 4 */
   (s32)FP_CONST( 0.582934968) >> CS2_SHIFT, /* k= 5 */
   (s32)FP_CONST( 0.622504123) >> CS2_SHIFT, /* k= 6 */
   (s32)FP_CONST( 0.674808341) >> CS2_SHIFT, /* k= 7 */
   (s32)FP_CONST( 0.744536271) >> CS2_SHIFT, /* k= 8 */
   (s32)FP_CONST( 0.839349645) >> CS2_SHIFT, /* k= 9 */
   (s32)FP_CONST( 0.972568238) >> CS2_SHIFT, /* k=10 */
   (s32)FP_CONST( 1.169439933) >> CS2_SHIFT, /* k=11 */
   (s32)FP_CONST( 1.484164616) >> CS2_SHIFT, /* k=12 */
   (s32)FP_CONST( 2.05778101)  >> CS2_SHIFT, /* k=13 */
   (s32)FP_CONST( 3.407608418) >> CS2_SHIFT, /* k=14 */
   (s32)FP_CONST(10.190008124) >> CS2_SHIFT, /* k=15 */

                                    /* STAGE 2: N=16 */
   (s32)FP_CONST( 0.502419286) >> CS1_SHIFT, /* k= 0 */
   (s32)FP_CONST( 0.522498615) >> CS1_SHIFT, /* k= 1 */
   (s32)FP_CONST( 0.566944035) >> CS1_SHIFT, /* k= 2 */
   (s32)FP_CONST( 0.646821783) >> CS1_SHIFT, /* k= 3 */
   (s32)FP_CONST( 0.788154623) >> CS1_SHIFT, /* k= 4 */
   (s32)FP_CONST( 1.060677686) >> CS1_SHIFT, /* k= 5 */
   (s32)FP_CONST( 1.722447098) >> CS1_SHIFT, /* k= 6 */
   (s32)FP_CONST( 5.101148619) >> CS1_SHIFT, /* k= 7 */

                                    /* STAGE 3: N= 8 */
   (s32)FP_CONST( 0.509795579) >> CS1_SHIFT, /* k= 0 */
   (s32)FP_CONST( 0.601344887) >> CS1_SHIFT, /* k= 1 */
   (s32)FP_CONST( 0.899976223) >> CS1_SHIFT, /* k= 2 */
   (s32)FP_CONST( 2.562915448) >> CS1_SHIFT, /* k= 3 */

#if defined(__MMX__) && (FP_SHIFT == 12)
   (1 << 12), (1 << 12),            /* STAGE 4: N= 4 */
   (s32)FP_CONST(-1.306562965) >> CS1_SHIFT, /* k= 1 */
   (s32)FP_CONST(-0.541196100) >> CS1_SHIFT, /* k= 0 */

   (1 << 12),
   (s32)FP_CONST(-0.707106781) >> CS1_SHIFT,
   (1 << 12),
#else
                                    /* STAGE 4: N= 4 */
   (s32)FP_CONST( 0.541196100) >> CS1_SHIFT, /* k= 0 */
   (s32)FP_CONST( 1.306562965) >> CS1_SHIFT, /* k= 1 */
#endif
                                    /* STAGE 5: N= 2 */
   (s32)FP_CONST( 0.707106781) >> CS1_SHIFT  /* k= 0 */
};


#if defined(__MMX__) && (FP_SHIFT == 12)

# include "mmx/mmx_dct32.c"

#else

/* using UNROLL_LOOPS should give an increase in performance */
#define UNROLL_LOOPS

#define SWAP(a, b) \
   tmp1    = data[a]; \
   data[a] = data[b]; \
   data[b] = tmp1


/* Does a 32pt DCT operation.
      INPUT: data - An array holding the 32 sub-band samples.
      OUTPUT: data - The 32 samples are replaced by 32 dct
                       coefficients. */
void dct32(AV1 *data)
{
#ifndef UNROLL_LOOPS
   AV2        tmp2;
   AV1        tmp1,
             *top     = data,
             *bot     = data + 31;
   const AV1 *cos_val = cos_const;

# define BF_P(i, s) \
   tmp2    = (AV2)(top[i] - bot[i]); \
   top[i] += bot[i]; \
   tmp2   *= (AV2)(*cos_val); \
   bot[i]  = (AV1)(tmp2 >> (FP_SHIFT-s))

# define BF_N(i, s) \
   tmp2    = (AV2)(bot[i] - top[i]); \
   top[i] += bot[i]; \
   tmp2   *= (AV2)(*cos_val); \
   bot[i]  = (AV1)(tmp2 >> (FP_SHIFT-s))
   
# define SUM(i) \
   top[i]   += bot[i]; \
   top[i-2] += top[i]; \
   top[i]   += bot[i-2]; \
   bot[i-2] += bot[i]
   
   /* STAGE 1: top=data[ 0], bot=data[31], cos_val=cos_const[0] */
   for (; cos_val < (cos_const+16); cos_val++) {
      BF_P(0,CS2_SHIFT);
      top++;
      bot--;
   }
   top -= 16;

   /* STAGE 2: top=data[ 0], bot=data[15], cos_val=cos_const[16] */
   for (; cos_val < (cos_const+16+8); cos_val++) {
      BF_P( 0,CS1_SHIFT); BF_N(16,CS1_SHIFT);
      top++;
      bot--;
   }
   top -= 8;

   /* STAGE 3: top=data[ 0], bot=data[ 7], cos_val=cos_const[16+8] */
   for (; cos_val < (cos_const+16+8+4); cos_val++) {
      BF_P( 0,CS1_SHIFT); BF_N( 8,CS1_SHIFT);
      BF_P(16,CS1_SHIFT); BF_N(24,CS1_SHIFT);
      top++;
      bot--;
   }
   top -= 4;

   /* STAGE 4: top=data[ 0], bot=data[ 3], cos_val=cos_const[16+8+4] */
   for (; cos_val < (cos_const+16+8+4+2); cos_val++) {
      BF_P( 0,CS1_SHIFT); BF_N( 4,CS1_SHIFT);
      BF_P( 8,CS1_SHIFT); BF_N(12,CS1_SHIFT);
      BF_P(16,CS1_SHIFT); BF_N(20,CS1_SHIFT);
      BF_P(24,CS1_SHIFT); BF_N(28,CS1_SHIFT);
      top++;
      bot--;
   }
   top -= 2;

   /* STAGE 5: top=data[ 0], bot=data[ 1], cos_val=cos_const[16+8+4+2] */
   BF_P( 0,CS1_SHIFT); BF_N( 2,CS1_SHIFT); top[ 2] += bot[ 2];
   BF_P( 4,CS1_SHIFT); BF_N( 6,CS1_SHIFT); SUM( 6);
   BF_P( 8,CS1_SHIFT); BF_N(10,CS1_SHIFT); top[10] += bot[10];
   BF_P(12,CS1_SHIFT); BF_N(14,CS1_SHIFT); SUM(14);
   BF_P(16,CS1_SHIFT); BF_N(18,CS1_SHIFT); top[18] += bot[18];
   BF_P(20,CS1_SHIFT); BF_N(22,CS1_SHIFT); SUM(22);
   BF_P(24,CS1_SHIFT); BF_N(26,CS1_SHIFT); top[26] += bot[26];
   BF_P(28,CS1_SHIFT); BF_N(30,CS1_SHIFT); SUM(30);

#else
   AV2 tmp2;
   AV1 tmp1;

# define BF_P(i, j, k, s) \
   tmp2      = (AV2)(data[i] - data[j-i]); \
   data[i]  += data[j-i]; \
   tmp2     *= (AV2)cos_const[k+i]; \
   data[j-i] = (AV1)(tmp2 >> (FP_SHIFT-s))

# define BF_N(i, j, k, s) \
   tmp2      = (AV2)(data[j-i] - data[i]); \
   data[i]  += data[j-i]; \
   tmp2     *= (AV2)cos_const[k+i]; \
   data[j-i] = (AV1)(tmp2 >> (FP_SHIFT-s))

# define SUM(i) \
   data[i]   += data[i+1]; \
   data[i-2] += data[i]; \
   data[i]   += data[i-1]; \
   data[i-1] += data[i+1]
   
   /* STAGE 1 */
   BF_P( 0, 31, 0,CS2_SHIFT);
   BF_P( 1, 31, 0,CS2_SHIFT);
   BF_P( 2, 31, 0,CS2_SHIFT);
   BF_P( 3, 31, 0,CS2_SHIFT);
   BF_P( 4, 31, 0,CS2_SHIFT);
   BF_P( 5, 31, 0,CS2_SHIFT);
   BF_P( 6, 31, 0,CS2_SHIFT);
   BF_P( 7, 31, 0,CS2_SHIFT);
   BF_P( 8, 31, 0,CS2_SHIFT);
   BF_P( 9, 31, 0,CS2_SHIFT);
   BF_P(10, 31, 0,CS2_SHIFT);
   BF_P(11, 31, 0,CS2_SHIFT);
   BF_P(12, 31, 0,CS2_SHIFT);
   BF_P(13, 31, 0,CS2_SHIFT);
   BF_P(14, 31, 0,CS2_SHIFT);
   BF_P(15, 31, 0,CS2_SHIFT);

   /* STAGE 2 */
   BF_P(0, 15, 16,CS1_SHIFT); BF_N(16, 31+16, 0,CS1_SHIFT);
   BF_P(1, 15, 16,CS1_SHIFT); BF_N(17, 31+16, 0,CS1_SHIFT);
   BF_P(2, 15, 16,CS1_SHIFT); BF_N(18, 31+16, 0,CS1_SHIFT);
   BF_P(3, 15, 16,CS1_SHIFT); BF_N(19, 31+16, 0,CS1_SHIFT);
   BF_P(4, 15, 16,CS1_SHIFT); BF_N(20, 31+16, 0,CS1_SHIFT);
   BF_P(5, 15, 16,CS1_SHIFT); BF_N(21, 31+16, 0,CS1_SHIFT);
   BF_P(6, 15, 16,CS1_SHIFT); BF_N(22, 31+16, 0,CS1_SHIFT);
   BF_P(7, 15, 16,CS1_SHIFT); BF_N(23, 31+16, 0,CS1_SHIFT);

   /* STAGE 3 */
   BF_P( 0,  7,   24,CS1_SHIFT); BF_N( 8, 15+8, 16,CS1_SHIFT); 
   BF_P(16, 23+16, 8,CS1_SHIFT); BF_N(24, 31+24, 0,CS1_SHIFT);
   BF_P( 1,  7,   24,CS1_SHIFT); BF_N( 9, 15+8, 16,CS1_SHIFT); 
   BF_P(17, 23+16, 8,CS1_SHIFT); BF_N(25, 31+24, 0,CS1_SHIFT);
   BF_P( 2,  7,   24,CS1_SHIFT); BF_N(10, 15+8, 16,CS1_SHIFT); 
   BF_P(18, 23+16, 8,CS1_SHIFT); BF_N(26, 31+24, 0,CS1_SHIFT);
   BF_P( 3,  7,   24,CS1_SHIFT); BF_N(11, 15+8, 16,CS1_SHIFT); 
   BF_P(19, 23+16, 8,CS1_SHIFT); BF_N(27, 31+24, 0,CS1_SHIFT);

   /* STAGE 4 */
   BF_P( 0,  3,    28,CS1_SHIFT); BF_N( 4,  7+4,  24,CS1_SHIFT);
   BF_P( 8, 11+8,  20,CS1_SHIFT); BF_N(12, 15+12, 16,CS1_SHIFT);
   BF_P(16, 19+16, 12,CS1_SHIFT); BF_N(20, 23+20,  8,CS1_SHIFT);
   BF_P(24, 27+24,  4,CS1_SHIFT); BF_N(28, 31+28,  0,CS1_SHIFT);
   BF_P( 1,  3,    28,CS1_SHIFT); BF_N( 5,  7+4,  24,CS1_SHIFT);
   BF_P( 9, 11+8,  20,CS1_SHIFT); BF_N(13, 15+12, 16,CS1_SHIFT);
   BF_P(17, 19+16, 12,CS1_SHIFT); BF_N(21, 23+20,  8,CS1_SHIFT);
   BF_P(25, 27+24,  4,CS1_SHIFT); BF_N(29, 31+28,  0,CS1_SHIFT);

   /* STAGE 5 */
   BF_P( 0,  1,    30,CS1_SHIFT); BF_N( 2,  3+2,  28,CS1_SHIFT);
                                  data[ 2] += data[ 3];
   BF_P( 4,  5+4,  26,CS1_SHIFT); BF_N( 6,  7+6,  24,CS1_SHIFT);
                                   SUM( 6);
   BF_P( 8,  9+8,  22,CS1_SHIFT); BF_N(10, 11+10, 20,CS1_SHIFT);
                                  data[10] += data[11];
   BF_P(12, 13+12, 18,CS1_SHIFT); BF_N(14, 15+14, 16,CS1_SHIFT);
                                   SUM(14);
   BF_P(16, 17+16, 14,CS1_SHIFT); BF_N(18, 19+18, 12,CS1_SHIFT);
                                  data[18] += data[19];
   BF_P(20, 21+20, 10,CS1_SHIFT); BF_N(22, 23+22,  8,CS1_SHIFT);
                                   SUM(22);
   BF_P(24, 25+24,  6,CS1_SHIFT); BF_N(26, 27+26,  4,CS1_SHIFT);
                                  data[26] += data[27];
   BF_P(28, 29+28,  2,CS1_SHIFT); BF_N(30, 31+30,  0,CS1_SHIFT);
                                   SUM(30);

#endif

   /* STAGE 6 */
   data[ 8] += data[12];
   data[12] += data[10];
   data[10] += data[14];
   data[14] += data[ 9];
   data[ 9] += data[13];
   data[13] += data[11];
   data[11] += data[15];
   data[24] += data[28];
   data[28] += data[26];
   data[26] += data[30];
   data[30] += data[25];
   data[25] += data[29];
   data[29] += data[27];
   data[27] += data[31];

   /* put coefficients in the proper order */
   SWAP( 1, 16);
   SWAP( 2,  8);
   SWAP( 3, 24);
   SWAP( 5, 20);
   SWAP( 6, 12);
   SWAP( 7, 28);
   SWAP( 9, 18);
   SWAP(11, 26);
   SWAP(13, 22);
   SWAP(15, 30);
   SWAP(19, 25);
   SWAP(23, 29);

   /* finish the odd coefficients */
   data[ 1] += data[ 3];
   data[ 3] += data[ 5];
   data[ 5] += data[ 7];
   data[ 7] += data[ 9];
   data[ 9] += data[11];
   data[11] += data[13];
   data[13] += data[15];
   data[15] += data[17];
   data[17] += data[19];
   data[19] += data[21];
   data[21] += data[23];
   data[23] += data[25];
   data[25] += data[27];
   data[27] += data[29];
   data[29] += data[31];
}

#undef BF_P
#undef BF_N
#undef SUM
#undef SWAP
#undef CS1_SHIFT
#undef CS2_SHIFT
#undef UNROLL_LOOPS

#endif

