/***************************************************************************
                       I_MACROBLOCK.c  -  description
                             -------------------
   Stuff needed for handling I picture macroblocks.
                             -------------------
    begin                : Mon Jun 26 2006
    copyright            : (C) 2006 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*#define VC1_VIDEO_TRACE 2*/
#include <string.h>
#include <stdlib.h>

#include "I_MACROBLOCK.h"
#include "vlc_tables.h"
#include "tables.h"
#include "inv_trans.h"
#include "bitstream.h"
#include "mpeg2_video/motion_vectors.h"
#include "VC1_motion_vectors.h"

#if defined(VC1_VIDEO_TRACE)||defined(OUTPUT_BLOCK_TRACE)
# include <stdio.h>
#endif


#define DATA_PTR        ptr
#define END_DATA_PTR    end_pos
#define PARSER_STATE    seq->state


/* Read the I picture macroblock data.
      INPUT: seq - Pointer to a SEQ_LAYER structure.
             pic - Pointer to a PIC_LAYER structure. */
void VC1_VIDEO_read_I_MACROBLOCK(SEQ_LAYER *seq, PIC_LAYER *pic)
{
   const u8 *ptr       = seq->payload,
            *end_pos   = seq->payload + seq->payload_len,
            *ACdelta;
   s16      *b, *c, *a, *d;
   u8        bit_shift = seq->bit_shift,
             data      = seq->data,
             bit_shift_inc,
             tmp;

#undef  DATA_BYTE
#define DATA_BYTE       data
#undef  BIT_SHIFT_BYTE
#define BIT_SHIFT_BYTE  bit_shift
#undef  STOP_PARSING
#define STOP_PARSING    goto VC1_VIDEO_read_I_MACROBLOCK_STOP

#define NEXT            (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 1)
#define CBPCY           (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 2)
#define AC_PRED         (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 3)
#define START_BLOCK     (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 4)
#define DC_DIFF1        (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 5)
#define DC_DIFF2        (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 6)
#define DC_DIFF_ESC     (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 7)
#define DC_PRED         (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 8)
#define AC_INDEX        (VC1_VIDEO_read_I_MACROBLOCK_STATE+ 9)
#define AC_ESC_MODE     (VC1_VIDEO_read_I_MACROBLOCK_STATE+10)
#define AC_ESC_MODE3_1  (VC1_VIDEO_read_I_MACROBLOCK_STATE+11)
#define AC_ESC_MODE3_2  (VC1_VIDEO_read_I_MACROBLOCK_STATE+12)
#define AC_ESC_MODE3_3  (VC1_VIDEO_read_I_MACROBLOCK_STATE+13)
#define AC_ESC_MODE3_4  (VC1_VIDEO_read_I_MACROBLOCK_STATE+14)
#define AC_ESC_MODE3_5  (VC1_VIDEO_read_I_MACROBLOCK_STATE+15)
#define AC_ESC_MODE3_6  (VC1_VIDEO_read_I_MACROBLOCK_STATE+16)
#define AC_INDEX2       (VC1_VIDEO_read_I_MACROBLOCK_STATE+17)
#define AC_SIGN         (VC1_VIDEO_read_I_MACROBLOCK_STATE+18)
#define AC_NEXT         (VC1_VIDEO_read_I_MACROBLOCK_STATE+19)
   if (PARSER_STATE > VC1_VIDEO_read_I_MACROBLOCK_STATE) {
      switch (PARSER_STATE) {
      case NEXT:
         goto VC1_VIDEO_read_I_MACROBLOCK_NEXT;
      case CBPCY:
         goto VC1_VIDEO_read_I_MACROBLOCK_CBPCY;
      case AC_PRED:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_PRED;
      case START_BLOCK:
         goto VC1_VIDEO_read_I_MACROBLOCK_START_BLOCK;
      case DC_DIFF1:
         goto VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF1;
      case DC_DIFF2:
         goto VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF2;
      case DC_DIFF_ESC:
         goto VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF_ESC;
      case DC_PRED:
         goto VC1_VIDEO_read_I_MACROBLOCK_DC_PRED;
      case AC_INDEX:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_INDEX;
      case AC_ESC_MODE:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE;
      case AC_ESC_MODE3_1:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_1;
      case AC_ESC_MODE3_2:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_2;
      case AC_ESC_MODE3_3:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_3;
      case AC_ESC_MODE3_4:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_4;
      case AC_ESC_MODE3_5:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_5;
      case AC_ESC_MODE3_6:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_6;
      case AC_INDEX2:
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_INDEX2;
      case AC_SIGN:
         tmp = *(pic->tab+2);
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_SIGN;
      case AC_NEXT:
         tmp = *(pic->tab+2);
         goto VC1_VIDEO_read_I_MACROBLOCK_AC_NEXT;
      }
   }

   /* initialize everything before we start decoding the first MB */
   pic->x     = pic-> y     = 0;
   pic->CBP_l = pic->CBP_lt = pic->CBP_t = 0;
   pic->rnd   = 1;

   pic->Mquant = pic->Pquant;
   if (pic->Pquant <= 5) {
      pic->DCstep     = pic->Pquant & 0xFE;
      pic->DCpred_def = 1024 / (s16)pic->DCstep;
   }
   else if (pic->Pquant <= 9) {
      pic->DCstep     =    8;
      pic->DCpred_def =  128;
   }
   else {
      pic->DCstep     = (pic->Pquant >> 2) + 6;
      if ( (pic->Pquant >= 19)&&(VC1_SEQ_OVERLAPp(seq)) )
         pic->DCpred_def = 0;
      else
         pic->DCpred_def = (1024 + (s16)(pic->DCstep >> 1)) / (s16)pic->DCstep;
   }

   b = pic->DCpred_Y;
   c = pic->DCpred_C;
   a = c + (seq->wid >> 3) + 2;
   do {
      *(b++) = pic->DCpred_def;
      *(b++) = pic->DCpred_def;
      *(c++) = pic->DCpred_def;
   } while (c < a);

   memset(pic->ACpred_Y, 0, (sizeof(s16)*((7*(VC1_MAX_FRAME_WID/8)) + 14)));
   memset(pic->ACpred_C, 0, (sizeof(s16)*((7*(VC1_MAX_FRAME_WID/8)) + 14)));

VC1_VIDEO_read_I_MACROBLOCK_NEXT:
   pic->tab = VC1_I_CBP_TAB;
   pic->b   = 0;
#if defined(VC1_VIDEO_TRACE)||defined(OUTPUT_BLOCK_TRACE)
   VC1_TRACE_BEG_BLK
   printf("  <I_MACROBLOCK %hu,%hu>\n", (pic->x >> 4), (pic->y >> 4));
   VC1_TRACE_END_BLK
#endif

VC1_VIDEO_read_I_MACROBLOCK_CBPCY:
   VC1_PARSE_VLC(pic->tab, CBPCY)
   bit_shift_inc = *(pic->tab+1);
   pic->CBP      = *(pic->tab+2);

#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("    CBP = 0x%02hX -> ", (u16)(pic->CBP >> 2));
   VC1_TRACE_END_BLK
#endif
   VC1_VIDEO_get_CBP(pic->CBP, pic->CBP_l, pic->CBP_lt, pic->CBP_t, tmp);
   pic->CBP_l = pic->CBP;
#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("0x%02hX\n", (u16)(pic->CBP >> 2));
   VC1_TRACE_END_BLK
#endif
   ALIGN_DATA_BYTE(AC_PRED, bit_shift_inc);

VC1_VIDEO_read_I_MACROBLOCK_AC_PRED:
   GET_LSB;
   if (DATA_BYTE >= 0x80)
      pic->mv_cbp_tab |= 0x01;
   else
      pic->mv_cbp_tab &= 0xFC;
#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("    ACpred = %c\n", (VC1_AC_PREDp(pic) ? 'y' : 'n'));
   VC1_TRACE_END_BLK
#endif
   pic->ACidx      = VC1_PIC_TRANS_AC2p(pic);
   pic->ACrunlevel = VC1_RUN_LEVEL_TAB[pic->ACidx];
   ALIGN_DATA_BYTE(START_BLOCK, 1);

   do {
VC1_VIDEO_read_I_MACROBLOCK_START_BLOCK:
      memset(pic->coeff, 0, (sizeof(s16)*64));
#if defined(VC1_VIDEO_TRACE)||defined(OUTPUT_BLOCK_TRACE)
      VC1_TRACE_BEG_BLK
      printf("    <BLOCK %hu>\n", pic->b);
      VC1_TRACE_END_BLK
#endif
      pic->tab = VC1_DC_DIFF_TAB[VC1_PIC_TRANS_DCp(pic) | (pic->b & 4)];

VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF1:
      VC1_PARSE_VLC(pic->tab, DC_DIFF1)
      bit_shift_inc = *(pic->tab+1);
      pic->coeff[0] = (s16)(*(pic->tab+2));
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("      DCdiff = %hd -> ", pic->coeff[0]);
      VC1_TRACE_END_BLK
#endif

      if (pic->coeff[0]) {
         ALIGN_DATA_BYTE(DC_DIFF2, bit_shift_inc);
VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF2:
         GET_LSB;
         if (pic->coeff[0] == 255) {   /* ESCAPE */
            pic->coeff[0] = (s16)DATA_BYTE;
            ALIGN8_DATA_BYTE(DC_DIFF_ESC)
VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF_ESC:
            GET_LSB;
            if (pic->Pquant > 5)
               goto VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF_SIGN;
            bit_shift_inc   = (pic->Pquant <= 3) ? 2 : 1;
            pic->coeff[0] <<= bit_shift_inc;
         }
         else if (pic->Pquant > 5)
            goto VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF_SIGN;
         else {
            bit_shift_inc   = (pic->Pquant <= 3) ? 2 : 1;
            pic->coeff[0] <<= bit_shift_inc;
            pic->coeff[0]  -= (s16)(1 | bit_shift_inc);
         }
         pic->coeff[0]  += (s16)(DATA_BYTE >> (8-bit_shift_inc));
         DATA_BYTE     <<= bit_shift_inc;
         BIT_SHIFT_BYTE += bit_shift_inc;

VC1_VIDEO_read_I_MACROBLOCK_DC_DIFF_SIGN:
         bit_shift_inc = 1;
         if (DATA_BYTE >= 0x80)
            pic->coeff[0] = -(pic->coeff[0]);
      }
      ALIGN_DATA_BYTE(DC_PRED, bit_shift_inc);

VC1_VIDEO_read_I_MACROBLOCK_DC_PRED:
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("0x%04hX\n", (u16)pic->coeff[0]);
      VC1_TRACE_END_BLK
#endif
      /* DC predictor */
      tmp = pic->mv_cbp_tab & 0xFD;
      if (pic->b < 4) {
         b = pic->DCpred_Y + (pic->x >> 2);
         if (pic->b >= 2) {
            c  = b + ((pic->b - 2) << 1);
            b  = c + 1; d  = c + 2; a  = c + 3;
         }
         else if (pic->b) {
            b += 2;
            c  = b + 1; a  = b + 2; d  = b + 3;
         }
         else {
            b += 3;
            c  = b - 2; a  = b - 1; d  = b;
            b[4] = b[1];
         }
#if (VC1_VIDEO_TRACE == 1)
         VC1_TRACE_BEG_BLK
         printf("B=%04hX A=%04hX\nC=%04hX d=", *b, *a, *c);
         VC1_TRACE_END_BLK
#endif
         if ( (*c != *a)&&(abs(*b - *a) > abs(*b - *c)) ) {
            c = a;
            tmp |= 0x02;   /* pred_dir = top */
         }
      }
      else {
         b = pic->DCpred_C + (pic->b - 4);
         c = b + (pic->x >> 3);
         d = c + 2;
#if (VC1_VIDEO_TRACE == 1)
         VC1_TRACE_BEG_BLK
         printf("B=%04hX A=%04hX\nC=%04hX d=", *b, *d, *c);
         VC1_TRACE_END_BLK
#endif
         if ( (*c != *d)&&(abs(*b - *d) > abs(*b - *c)) ) {
            c = d;
            tmp |= 0x02;   /* pred_dir = top */
         }
         *b = *d;
      }
      pic->coeff[0]  += *c;
      *d              = pic->coeff[0];
#if (VC1_VIDEO_TRACE == 1)
      VC1_TRACE_BEG_BLK
      printf("%04hX\n", *d);
      VC1_TRACE_END_BLK
#endif
      pic->mv_cbp_tab = tmp;
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      if (tmp & 0x01) {
         printf("      pred_dir = %s\n", ((tmp & 0x02) ? "TOP" : "LEFT"));
      }
      VC1_TRACE_END_BLK
#endif
#if (VC1_VIDEO_TRACE > 2)
      VC1_TRACE_BEG_BLK
      printf("      DCpred = 0x%04hX\n"
             "      coeff[0] = 0x%04hX\n",
             (u16)(*c), (u16)pic->coeff[0]);
      VC1_TRACE_END_BLK
#endif

      if (pic->CBP >= 0x80) {
         pic->zz = VC1_ZIGZAG_TAB[tmp & 0x03];
         do {
            pic->tab = VC1_AC_INDEX_TAB[pic->ACidx];
            pic->c   = 0;

VC1_VIDEO_read_I_MACROBLOCK_AC_INDEX:
            VC1_PARSE_VLC(pic->tab, AC_INDEX)
            bit_shift_inc = *(pic->tab+1);
            tmp           = *(pic->tab+2);
#if (VC1_VIDEO_TRACE > 2)
            VC1_TRACE_BEG_BLK
            printf("      AC(index,run,level,sign) = ");
            VC1_TRACE_END_BLK
#endif
            if (tmp == 255) {    /* ESCAPE */
               ALIGN_DATA_BYTE(AC_ESC_MODE, bit_shift_inc);
VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE:
               GET_LSB;
               if (DATA_BYTE < 0x40) {       /* Mode3 */
                  if (DATA_BYTE >= 0x20)
                     pic->trans_tab |= VC1_AC_MODE3_LAST;
#if (VC1_VIDEO_TRACE > 3)
                  VC1_TRACE_BEG_BLK
                  printf("\nMODE3: last=%c\n",
                         (VC1_AC_MODE3_LASTp(pic) ? 'y' : 'n'));
                  VC1_TRACE_END_BLK
#endif
                  ALIGN_DATA_BYTE(AC_ESC_MODE3_1, 3);

VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_1:
                  if (!VC1_AC_MODE3p(pic)) {
                     pic->trans_tab |= VC1_AC_MODE3;

                     pic->tab = VC1_AC_MODE3_LEVEL_SIZE_TAB[VC1_AC_MODE3_TABp(pic)];
VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_2:
                     VC1_PARSE_VLC(pic->tab, AC_ESC_MODE3_2)
                     bit_shift_inc   = *(pic->tab+1);
                     pic->level_size = *(pic->tab+2);
                     DATA_BYTE     <<= bit_shift_inc;
                     BIT_SHIFT_BYTE += bit_shift_inc;
                     pic->run_size   = 5 - (DATA_BYTE >> 6);
#if (VC1_VIDEO_TRACE > 3)
                     VC1_TRACE_BEG_BLK
                     printf("       size(level,run)=%hu,%hu\n",
                            (u16)pic->level_size, (u16)(8 - pic->run_size));
                     VC1_TRACE_END_BLK
#endif
                     ALIGN_DATA_BYTE(AC_ESC_MODE3_3, 2);
                  }

VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_3:
                  GET_LSB;
                  bit_shift_inc = 8 - pic->run_size;
                  pic->zz      += DATA_BYTE >> pic->run_size;
#if (VC1_VIDEO_TRACE > 2)
                  VC1_TRACE_BEG_BLK
                  printf("255,%2hu,", (u16)(DATA_BYTE >> pic->run_size));
                  VC1_TRACE_END_BLK
#endif
                  DATA_BYTE     <<= bit_shift_inc;
                  BIT_SHIFT_BYTE += bit_shift_inc;

                  if (DATA_BYTE < 0x80)
                     pic->c = 0x4000;
                  ALIGN_DATA_BYTE(AC_ESC_MODE3_4, 1);

VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_4:
                  if (pic->level_size > 8) {
                     GET_LSB;
                     pic->c |= (s16)DATA_BYTE << (pic->level_size - 8);
                     ALIGN8_DATA_BYTE(AC_ESC_MODE3_5)
VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_5:
                     bit_shift_inc = pic->level_size - 8;
                  }
                  else
                     bit_shift_inc = pic->level_size;
                  GET_LSB;
                  pic->c |= (s16)DATA_BYTE >> (8 - bit_shift_inc);
#if (VC1_VIDEO_TRACE > 2)
                  VC1_TRACE_BEG_BLK
                  printf("%2hd,%hd\n", (pic->c & 0x0FFF), ((pic->c >> 14)^1));
                  VC1_TRACE_END_BLK
#endif
                  if (pic->c >= 0x4000)
                     pic->c -= 0x4000;
                  else
                     pic->c  = -(pic->c);
                  ALIGN_DATA_BYTE(AC_ESC_MODE3_6, bit_shift_inc);
VC1_VIDEO_read_I_MACROBLOCK_AC_ESC_MODE3_6:
                  if (VC1_AC_MODE3_LASTp(pic)) {
                     pic->trans_tab &= ~VC1_AC_MODE3_LAST;
                     tmp = 255;
                  }
                  else
                     tmp =   0;
                  goto VC1_VIDEO_read_I_MACROBLOCK_AC_NEXT;
               }
               else if (DATA_BYTE < 0x80) {  /* Mode2 */
                  bit_shift_inc   = 2;
                  pic->trans_tab |= VC1_AC_MODE2;
               }
               else                          /* Mode1 */
                  bit_shift_inc   = 1;

               pic->tab = VC1_AC_INDEX_TAB[pic->ACidx];
               ALIGN_DATA_BYTE(AC_INDEX2, bit_shift_inc);
VC1_VIDEO_read_I_MACROBLOCK_AC_INDEX2:
               VC1_PARSE_VLC(pic->tab, AC_INDEX2)
               tmp     = *(pic->tab+2);
               ACdelta = pic->ACrunlevel;
#if (VC1_VIDEO_TRACE > 2)
               VC1_TRACE_BEG_BLK
               printf("%3hu,", (u16)tmp);
               VC1_TRACE_END_BLK
#endif
               if (VC1_AC_MODE2p(pic)) {
                  bit_shift_inc = ACdelta[((u32)tmp << 1)+7];
                  ACdelta      += 3;
               }
               else {
                  bit_shift_inc = ACdelta[((u32)tmp << 1)+6];
                  ACdelta      += 1;
               }
               if (tmp >= *(pic->ACrunlevel))
                  ACdelta++;
               ACdelta      += *ACdelta;
               ACdelta      += bit_shift_inc;
               bit_shift_inc = *(pic->tab+1);
               ACdelta      += *(pic->ACrunlevel) + *(pic->ACrunlevel);
               if (VC1_AC_MODE2p(pic)) {
                  pic->trans_tab &= ~VC1_AC_MODE2;
                  pic->zz        += *(ACdelta);
#if (VC1_VIDEO_TRACE > 2)
                  VC1_TRACE_BEG_BLK
                  printf("%hu+%hu,%2hu,",
                         (u16)pic->ACrunlevel[((u32)tmp << 1)+6],
                         (u16)ACdelta[0],
                         (u16)pic->ACrunlevel[((u32)tmp << 1)+7]);
                  VC1_TRACE_END_BLK
#endif
               }
               else {
                  pic->c = (s16)(*(ACdelta));
#if (VC1_VIDEO_TRACE > 2)
                  VC1_TRACE_BEG_BLK
                  printf("%2hu,%hu+%hu,",
                         (u16)pic->ACrunlevel[((u32)tmp << 1)+6],
                         (u16)pic->ACrunlevel[((u32)tmp << 1)+7],
                         (u16)ACdelta[0]);
                  VC1_TRACE_END_BLK
#endif
               }
            }
#if (VC1_VIDEO_TRACE > 2)
            VC1_TRACE_BEG_BLK
               printf("%3hu,%2hu,%2hu,",
                      (u16)tmp,
                      (u16)pic->ACrunlevel[((u32)tmp << 1)+6],
                      (u16)pic->ACrunlevel[((u32)tmp << 1)+7]);
            VC1_TRACE_END_BLK
#endif
            pic->zz += (u16)pic->ACrunlevel[((u32)tmp << 1)+6];
            pic->c  += (s16)pic->ACrunlevel[((u32)tmp << 1)+7];
            ALIGN_DATA_BYTE(AC_SIGN, bit_shift_inc);

VC1_VIDEO_read_I_MACROBLOCK_AC_SIGN:
            GET_LSB;
            if (DATA_BYTE >= 0x80)
               pic->c = -(pic->c);
#if (VC1_VIDEO_TRACE > 2)
            VC1_TRACE_BEG_BLK
            printf("%hu\n", (u16)(DATA_BYTE >> 7));
            VC1_TRACE_END_BLK
#endif
            ALIGN_DATA_BYTE(AC_NEXT, 1);

VC1_VIDEO_read_I_MACROBLOCK_AC_NEXT:
            pic->coeff[*(pic->zz++)] = pic->c;
         } while (tmp < *(pic->ACrunlevel));
      }
      pic->CBP <<= 1;

      VC1_VIDEO_end_I_BLOCK(seq, pic);

#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      for (tmp = 0; tmp < 64; tmp++) {
         if ((tmp & 0x07) == 0x00)
            printf("      ");
         printf("%04hX ", pic->coeff[tmp]);
         if ((tmp & 0x07) == 0x07)
            printf("\n");
      }
      printf("    <END_BLOCK>\n");
      VC1_TRACE_END_BLK
#endif
      if (pic->b == 3) {
         pic->ACidx      = 4 | VC1_PIC_TRANS_AC1p(pic);
         pic->ACrunlevel = VC1_RUN_LEVEL_TAB[pic->ACidx];
      }
   } while (++(pic->b) < 6);

   pic->CBPbuf[pic->x >> 4] = pic->CBP_l;
   pic->x                  += 16;
   if (pic->x >= seq->wid) {
      pic->x  =  0;  /* finished a row */
      pic->y += 16;
      if (pic->y >= seq->hgt) {
         pic->y = 0; /* finished the frame */
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("<END_PIC>\n");
         VC1_TRACE_END_BLK
#endif
         goto VC1_VIDEO_read_I_MACROBLOCK_END_PIC;
      }

      pic->DCpred_Y[3] = pic->DCpred_def;
      pic->DCpred_C[0] = pic->DCpred_C[1] = pic->DCpred_def;
      memset((pic->ACpred_Y + (7*(VC1_MAX_FRAME_WID/8))), 0, (sizeof(s16)*14));
      memset((pic->ACpred_C + (7*(VC1_MAX_FRAME_WID/8))), 0, (sizeof(s16)*14));
      pic->CBP_l  = pic->CBP_t = 0;
   }
   if (pic->y) {
      pic->CBP_lt = pic->CBP_t;
      pic->CBP_t  = pic->CBPbuf[pic->x >> 4];
   }
   goto VC1_VIDEO_read_I_MACROBLOCK_NEXT;

VC1_VIDEO_read_I_MACROBLOCK_END_PIC:
   PARSER_STATE = VC1_VIDEO_frame_completed_STATE;

VC1_VIDEO_read_I_MACROBLOCK_STOP:
   seq->bit_shift   = BIT_SHIFT_BYTE;
   seq->data        = DATA_BYTE;
   seq->payload_len = 0;

#undef NEXT
#undef CBPCY
#undef AC_PRED
#undef START_BLOCK
#undef DC_DIFF1
#undef DC_DIFF2
#undef DC_DIFF_ESC
#undef DC_PRED
#undef AC_INDEX
#undef AC_ESC_MODE
#undef AC_ESC_MODE3_1
#undef AC_ESC_MODE3_2
#undef AC_ESC_MODE3_3
#undef AC_ESC_MODE3_4
#undef AC_ESC_MODE3_5
#undef AC_ESC_MODE3_6
#undef AC_INDEX2
#undef AC_SIGN
#undef AC_NEXT
}

/* Process an intra block.
      INPUT: seq - Pointer to a SEQ_LAYER structure.
             pic - Pointer to a PIC_LAYER structure. */
void VC1_VIDEO_end_I_BLOCK(SEQ_LAYER *seq, PIC_LAYER *pic)
{
   s16 *pred_top, *pred_lft,
       *coeff = pic->coeff + 1;
   int  i     = 14;

   /* AC Prediction */
   if (pic->b < 4) {
      pred_top = pred_lft = pic->ACpred_Y;
      if (pic->b & 1)
         pred_top += 7;
      if (pic->b & 2)
         pred_lft += 7;
   }
   else {
      pred_lft = pic->ACpred_C;
      if (pic->b & 1)
         pred_lft += 7;
      pred_top = pred_lft;
   }
   pred_top += 7*(pic->x >> 3);
   pred_lft += 7*(VC1_MAX_FRAME_WID/8);

   if (VC1_AC_PREDp(pic) == VC1_AC_PRED_LEFT) {
      do {
         *(pred_top++) = *(coeff++);
      } while (--i > 7);
      do {
         *coeff       += *pred_lft;
         *(pred_lft++) = *coeff;
         coeff        += 8;
      } while (--i);
   }
   else if (VC1_AC_PREDp(pic) == VC1_AC_PRED_TOP) {
      do {
         *coeff       += *pred_top;
         *(pred_top++) = *(coeff++);
      } while (--i > 7);
      do {
         *(pred_lft++) = *coeff;
         coeff        += 8;
      } while (--i);
   }
   else {
      do {
         *(pred_top++) = *(coeff++);
      } while (--i > 7);
      do {
         *(pred_lft++) = *coeff;
         coeff        += 8;
      } while (--i);
   }

   /* Inverse Quantization */
   coeff       = pic->coeff;
   i           = 63;
   *(coeff++) *= (s16)pic->DCstep;
   if (VC1_PIC_UNIFORM_QUANTp(pic)) {
      do {
         if (*coeff)
            *coeff *= (s16)pic->Mquant;
         coeff++;
      } while (--i);
   }
   else {
      s16 q = (s16)(pic->Mquant >> 1);
      do {
         if (*coeff) {
            *coeff *= (s16)pic->Mquant;
            if (*coeff > 0)
               *coeff += q;
            else
               *coeff -= q;
         }
         coeff++;
      } while (--i);
   }

   /* Inverse Transformation */
   VC1_8x8_inv_trans(pic->coeff);

   /* put the final block into the frame */
   i = (int)pic->frame_wid;
   if (pic->b >= 4)
      i >>= 1;

#ifdef OUTPUT_BLOCK_TRACE
   VC1_TRACE_BEG_BLK
   printf("BLK: pred_blk = %u\n", (u32)pic->pred_blk);
   if (pic->pred_blk < pic->pred_frame)
      printf("SAT_ERROR: pred_blk < pred_frame  wid=%d\n", i);
   else if ((pic->pred_blk+(i*7)+8) > (pic->pred_frame+pic->frame_size))
      printf("SAT_ERROR: pred_blk > pred_frame+%u  wid=%d\n", pic->frame_size, i);
   VC1_TRACE_END_BLK
#else
# ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("BLK: pred_blk = %8u\n", (u32)(pic->pred_blk - pic->pred_frame));
   VC1_TRACE_END_BLK
# endif

   if ( (VC1_SEQ_OVERLAPp(seq))&&(pic->Pquant >= 18) ) {
      /* Overlap Smoothing */
# ifndef USE_PRECISE_OVERLAP
      if ( (pic->x)||(pic->b == 1)||(pic->b == 3) ) {
         VC1_overlap_Y(pic->coeff, pic->pred_blk, i);
         if ( (pic->y)||(pic->b & 2) ) {
            VC1_overlap_X_last(pic->pred_blk, i);
            if ( (pic->b & 0x05)&&((pic->x+16) >= seq->wid) )
               VC1_overlap_X(pic->coeff, pic->pred_blk, i);
         }
      }
# else
      pic->overlap = 0;
      if ( (pic->x)||(pic->b == 1)||(pic->b == 3) ) {
         pic->overlap |= 0x04;
         if ( (pic->y)||(pic->b & 2) ) {
            pic->overlap |= 0x02;
            if ( (pic->b & 0x05)&&((pic->x+16) >= seq->wid) )
               pic->overlap |= 0x01;
         }
      }

      pred_top = pred_lft = pic->Ocoeff;
      if (pic->b <= 3) {
         pred_lft += ((pic->b & 1) << 4) + (pic->x << 1);
         pred_top += ((pic->b & 2) << 4) + (VC1_MAX_FRAME_WID*2);
         pic->c    = 64 - ((pic->b & 2) << 3);
      }
      else if (pic->b == 4) {
         pred_lft += (VC1_MAX_FRAME_WID*2) + 96 + pic->x;
         pred_top += (VC1_MAX_FRAME_WID*3) + 96;
         pic->c    = 32;
      }
      else {
         pred_lft += (VC1_MAX_FRAME_WID*3) +144 + pic->x;
         pred_top += (VC1_MAX_FRAME_WID*4) +144;
         pic->c    = 32;
      }
      VC1_overlap_Y(pic->coeff, (pred_top + pic->c),
                    ((pic->b) ? pred_lft : NULL), pred_top,
                    ((pic->overlap & 0x04) ? pic->pred_blk : NULL), i);
      VC1_overlap_X_last(pic->coeff, pred_top,
                         ((pic->overlap & 0x02) ? pic->pred_blk : NULL), i);
      VC1_overlap_X(pic->coeff, pred_lft, pred_top,
                    ((pic->overlap & 0x01) ? pic->pred_blk : NULL), i);
# endif
      MV_SATURATION(pic->coeff, pic->pred_blk, i);
   }
   else
      MV_COPY_PRED(pic->coeff, pic->pred_blk, i, 0);
# ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   VC1_MV_COPY_OUTPUT(pic->coeff, pic->pred_blk, i);
   VC1_TRACE_END_BLK
# endif
#endif

   /* move frame pointers into correct position */
   switch (pic->b) {
   case 0:
      i   =  8;
      break;
   case 1:
      i <<=  3;
      i  -=  8;
      break;
   case 2:
      i   =  8;
      break;
   case 3:
      i  *= ((3 * (int)pic->y) >> 2) + 8;
      i  += (16 + (int)pic->x) >> 1;
      i   = (int)((pic->luma_size * 5) >> 2) - i;
      break;
   case 4:
      i   = (int)pic->luma_size / -4;
      break;
   case 5:
      if ((pic->x+16) == seq->wid) {
         i *= ((3 * (int)pic->y) >> 1) + 30;
         i += (int)pic->frame_wid - (int)seq->wid;
      }
      else
         i *=  (3 * (int)pic->y) >> 1;

      i  += (32 + (int)pic->x) >> 1;
      i  -= (int)pic->luma_size;
   }

   pic->pred_blk += i;
#ifdef OUTPUT_BLOCK_TRACE
   VC1_TRACE_BEG_BLK
   printf("              = %u\n", (u32)pic->pred_blk);
   VC1_TRACE_END_BLK
#elif defined(VC1_VIDEO_TRACE)
   VC1_TRACE_BEG_BLK
   printf("              = %8u\n", (u32)(pic->pred_blk - pic->pred_frame));
   VC1_TRACE_END_BLK
#endif

}

