/***************************************************************************
                         PIC_LAYER.c  -  description
                             -------------------
   Stuff needed for handling the VC1 picture layer.
                             -------------------
    begin                : Tue May 23 2006
    copyright            : (C) 2006 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*#define _BITSTREAM_TRACE_
#define BIT_TRACE_BEG_BLK  VC1_TRACE_BEG_BLK
#define BIT_TRACE_END_BLK  VC1_TRACE_END_BLK*/

/*#define VC1_VIDEO_TRACE*/
#ifdef VC1_VIDEO_TRACE
# include <stdio.h>
#endif

#include "PIC_LAYER.h"
#include "vlc_tables.h"
#include "bitstream.h"

#ifdef VC1_VIDEO_TRACE
u32 VC1_frame_cnt = 0;
# define VC1_ACTIVATE_TRACE \
   if ( (111 <= VC1_frame_cnt)&&(VC1_frame_cnt < 0x80000000) ) \
      VC1_frame_cnt += 0x80000001; \
   else \
      VC1_frame_cnt++;
#endif

#define DATA_PTR        ptr
#define END_DATA_PTR    end_pos
#define PARSER_STATE    seq->state

#undef  DATA_BYTE
#define DATA_BYTE       data
#undef  BIT_SHIFT_BYTE
#define BIT_SHIFT_BYTE  bit_shift


/* Read the first part of the PIC_LAYER data.
      INPUT: seq - Pointer to a SEQ_LAYER structure.
             pic - Pointer to a PIC_LAYER structure. */
void VC1_VIDEO_read_PIC_LAYER1(SEQ_LAYER *seq, PIC_LAYER *pic)
{
   const u8 *ptr       = seq->payload,
            *end_pos   = seq->payload + seq->payload_len;
   u8        bit_shift = seq->bit_shift,
             data      = seq->data,
             bit_shift_inc;

#undef  STOP_PARSING
#define STOP_PARSING    goto VC1_VIDEO_read_PIC_LAYER1_STOP

#define PICTYPE         (VC1_VIDEO_read_PIC_LAYER1_STATE+1)
#define PIC_LAYER1_END  (VC1_VIDEO_read_PIC_LAYER1_STATE+2)
   if (PARSER_STATE > VC1_VIDEO_read_PIC_LAYER1_STATE) {
      switch (PARSER_STATE) {
      case PICTYPE:
         goto VC1_VIDEO_read_PIC_LAYER1_PICTYPE;
      case PIC_LAYER1_END:
         goto VC1_VIDEO_read_PIC_LAYER1_END;
      }
   }

#ifdef VC1_VIDEO_TRACE
   VC1_ACTIVATE_TRACE
   VC1_TRACE_BEG_BLK
   printf("<PIC_LAYER %u>\n", ((VC1_frame_cnt & 0x0FFFFFFF)-1));
   VC1_TRACE_END_BLK
#endif
   BIT_SHIFT_BYTE = 8;
   DATA_BYTE      = *(DATA_PTR);

   if (VC1_SEQ_FRAMEINTERP_FLAGp(seq)) {
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      if (DATA_BYTE & 0x80)
         printf("  FrameInterp\n");
      VC1_TRACE_END_BLK
#endif
      BIT_SHIFT_BYTE++;
      DATA_BYTE <<= 1;
   }

#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("  FrameCnt = %hu\n", (u16)(DATA_BYTE >> 6));
   VC1_TRACE_END_BLK
#endif
   BIT_SHIFT_BYTE += 2;

   pic->type = 0x00;
   if (VC1_SEQ_RANGE_REDp(seq)) {
      if (DATA_BYTE & 0x20) {
         pic->type |= 0x08;
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("  RangeRed_Frm\n");
         VC1_TRACE_END_BLK
#endif
      }
      BIT_SHIFT_BYTE++;
   }

   ALIGN0_DATA_BYTE(PICTYPE);
VC1_VIDEO_read_PIC_LAYER1_PICTYPE:
#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("  PicType = ");
   VC1_TRACE_END_BLK
#endif
   GET_LSB;
   if (DATA_BYTE >= 0x80) {
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("P\n");
      VC1_TRACE_END_BLK
#endif
      pic->type    |= VC1_P_FRAME;
      bit_shift_inc = 1;
   }
   else if (!VC1_SEQ_MAXBFRAMESp(seq)) {
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("I\n");
      VC1_TRACE_END_BLK
#endif
      pic->type    |= VC1_I_FRAME;
      bit_shift_inc = 1;
   }
   else {
      if (DATA_BYTE >= 0x40) {
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("I\n");
         VC1_TRACE_END_BLK
#endif
         pic->type |= VC1_I_FRAME;
      }
      else {
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("B or BI\n");
         VC1_TRACE_END_BLK
#endif
         pic->type |= VC1_B_FRAME;
      }
      bit_shift_inc = 2;
   }
   ALIGN_DATA_BYTE(PIC_LAYER1_END, bit_shift_inc);

VC1_VIDEO_read_PIC_LAYER1_END:
   PARSER_STATE      = VC1_VIDEO_read_PIC_LAYER2_STATE;
   seq->payload_len -= (u32)(DATA_PTR - seq->payload);
   seq->payload      = DATA_PTR;
   seq->bit_shift    = BIT_SHIFT_BYTE;
   seq->data         = DATA_BYTE;
   return;

VC1_VIDEO_read_PIC_LAYER1_STOP:
   seq->bit_shift   = BIT_SHIFT_BYTE;
   seq->data        = DATA_BYTE;
   seq->payload_len = 0;

#undef PICTYPE
#undef PIC_LAYER1_END
}


/* Read the second part of the PIC_LAYER data.
      INPUT: seq - Pointer to a SEQ_LAYER structure.
             pic - Pointer to a PIC_LAYER structure. */
void VC1_VIDEO_read_PIC_LAYER2(SEQ_LAYER *seq, PIC_LAYER *pic)
{
   const u8 *ptr       = seq->payload,
            *end_pos   = seq->payload + seq->payload_len;
   u8        bit_shift = seq->bit_shift,
             data      = seq->data,
             bit_shift_inc;

#undef  STOP_PARSING
#define STOP_PARSING    goto VC1_VIDEO_read_PIC_LAYER_STOP

#define BUF_FULL        (VC1_VIDEO_read_PIC_LAYER2_STATE+ 1)
#define PQ_INDEX        (VC1_VIDEO_read_PIC_LAYER2_STATE+ 2)
#define MV_RANGE        (VC1_VIDEO_read_PIC_LAYER2_STATE+ 3)
#define MV_MODE         (VC1_VIDEO_read_PIC_LAYER2_STATE+ 4)
#define MV_MODE2        (VC1_VIDEO_read_PIC_LAYER2_STATE+ 5)
#define LUMA_SCALE      (VC1_VIDEO_read_PIC_LAYER2_STATE+ 6)
#define LUMA_SHIFT      (VC1_VIDEO_read_PIC_LAYER2_STATE+ 7)
#define MV_TYPE         (VC1_VIDEO_read_PIC_LAYER2_STATE+ 8)
#define MV_TYPE_BP      (VC1_VIDEO_read_PIC_LAYER2_STATE+ 9)
#define MB_DIRECT       (VC1_VIDEO_read_PIC_LAYER2_STATE+10)
#define MB_SKIP         (VC1_VIDEO_read_PIC_LAYER2_STATE+11)
#define DQUANT          (VC1_VIDEO_read_PIC_LAYER2_STATE+12)
#define PQUANT_ALT      (VC1_VIDEO_read_PIC_LAYER2_STATE+13)
#define TT_FRM          (VC1_VIDEO_read_PIC_LAYER2_STATE+14)
#define TRANS_TAB       (VC1_VIDEO_read_PIC_LAYER2_STATE+15)
#define PIC_LAYER_END   (VC1_VIDEO_read_PIC_LAYER2_STATE+16)
   if (PARSER_STATE > VC1_VIDEO_read_PIC_LAYER2_STATE) {
      switch (PARSER_STATE) {
      case BUF_FULL:
         goto VC1_VIDEO_read_PIC_LAYER_BUF_FULL;
      case PQ_INDEX:
         goto VC1_VIDEO_read_PIC_LAYER_PQ_INDEX;
      case MV_RANGE:
         goto VC1_VIDEO_read_PIC_LAYER_MV_RANGE;
      case MV_MODE:
         goto VC1_VIDEO_read_PIC_LAYER_MV_MODE;
      case MV_MODE2:
         goto VC1_VIDEO_read_PIC_LAYER_MV_MODE2;
      case LUMA_SCALE:
         goto VC1_VIDEO_read_PIC_LAYER_LUMA_SCALE;
      case LUMA_SHIFT:
         goto VC1_VIDEO_read_PIC_LAYER_LUMA_SHIFT;
      case MV_TYPE:
         goto VC1_VIDEO_read_PIC_LAYER_MV_TYPE;
      case MV_TYPE_BP:
         goto VC1_VIDEO_read_PIC_LAYER_MV_TYPE_BP;
      case MB_DIRECT:
         goto VC1_VIDEO_read_PIC_LAYER_MB_DIRECT;
      case MB_SKIP:
         goto VC1_VIDEO_read_PIC_LAYER_MB_SKIP;
      case DQUANT:
         goto VC1_VIDEO_read_PIC_LAYER_DQUANT;
      case PQUANT_ALT:
         goto VC1_VIDEO_read_PIC_LAYER_PQUANT_ALT;
      case TT_FRM:
         goto VC1_VIDEO_read_PIC_LAYER_TT_FRM;
      case TRANS_TAB:
         goto VC1_VIDEO_read_PIC_LAYER_TRANS_TAB;
      case PIC_LAYER_END:
         goto VC1_VIDEO_read_PIC_LAYER_END;
      }
   }

   if (pic->type >= VC1_B_FRAME) {
      GET_LSB;
      pic->tab = VC1_BFRACTION_TAB;
      VC1_PARSE_1BYTE_VLC(pic->tab)
      bit_shift_inc = *(pic->tab+1);
      if (*(pic->tab+2) == 0xFF)
         pic->type     |= VC1_BI_FRAME;
      else {
         pic->Bfraction = *(pic->tab+2);
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("  Bfraction = %hu/%hu\n",
                (u16)(pic->Bfraction >> 4),
                (u16)(pic->Bfraction & 0x0F));
         VC1_TRACE_END_BLK
#endif
      }
      ALIGN_DATA_BYTE(BUF_FULL, bit_shift_inc);
   }

VC1_VIDEO_read_PIC_LAYER_BUF_FULL:
   if (pic->type & VC1_I_FRAME) {
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      GET_LSB;
      printf("  BufferFull = %hu\n", (u16)(DATA_BYTE >> 1));
      VC1_TRACE_END_BLK
#endif
      ALIGN_DATA_BYTE(PQ_INDEX, 7);
   }

VC1_VIDEO_read_PIC_LAYER_PQ_INDEX:
   GET_LSB;
   bit_shift_inc = 5;
   pic->Pquant   = DATA_BYTE >> 2;
   if (DATA_BYTE >= 0x48) {
      pic->Pquant &= 0xFE;
      pic->type   |= 0x01;    /* LowMotion flag */
   }
   else
      bit_shift_inc++;

   if (!VC1_EPT_QUANTIZERp(seq)) {
      if (DATA_BYTE < 0x48)
         pic->type   |= 0x04; /* uniform quantizer */
      else if (DATA_BYTE < 0xE8)
         pic->Pquant -= 6;
      else if (DATA_BYTE < 0xF8)
         pic->Pquant -= 4;
   }
   else if (VC1_EPT_QUANTIZERp(seq) == 0x08) {
      if (DATA_BYTE & (0x80 >> bit_shift_inc))
         pic->type |= 0x04;   /* uniform quantizer */
      bit_shift_inc++;
   }

#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("  PQ_Index = %hu\n"
          "  Pquant = %hu",
          (u16)(DATA_BYTE   >> 3),
          (u16)(pic->Pquant >> 1));
   if (pic->Pquant & 0x01)
      printf(".5");
   printf("\n  %suniform quantizer\n",
          ((VC1_PIC_UNIFORM_QUANTp(pic)) ? "" : "non-"));
   VC1_TRACE_END_BLK
#endif
   ALIGN_DATA_BYTE(MV_RANGE, bit_shift_inc);

VC1_VIDEO_read_PIC_LAYER_MV_RANGE:
   if (VC1_EPT_EXT_MVp(seq)) {
      GET_LSB;
      if (DATA_BYTE < 0x80) {
         pic->MVsize   = 0x78;
         bit_shift_inc =    1;
      }
      else if (DATA_BYTE < 0xC0) {
         pic->MVsize   = 0x89;
         bit_shift_inc =    2;
      }
      else {
         pic->MVsize   = (DATA_BYTE < 0xE0) ? 0x9B : 0xAC;
         bit_shift_inc =    3;
      }
      ALIGN_DATA_BYTE(MV_MODE, bit_shift_inc);
   }
   else
      pic->MVsize = 0x78;

/** MISSING MULTI-RES STUFF HERE **/

VC1_VIDEO_read_PIC_LAYER_MV_MODE:
   pic->VST = 0;
   if (!(pic->type & VC1_I_FRAME)) {
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("  MVrange  = %ux%u\n",
             (1U << (pic->MVsize & 0x0F)),
             (1U << (pic->MVsize >>   4)));
      VC1_TRACE_END_BLK
#endif
      GET_LSB;
      if (pic->type < VC1_B_FRAME) {
         pic->tab = (pic->Pquant > 24) ?
                    VC1_MVMODE0_TAB : VC1_MVMODE1_TAB;
         VC1_PARSE_1BYTE_VLC(pic->tab)
         bit_shift_inc = *(pic->tab+1);
         pic->MVmode   = *(pic->tab+2);
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("  MVmode = 0x%02hX\n", pic->MVmode);
         VC1_TRACE_END_BLK
#endif
         ALIGN_DATA_BYTE(MV_MODE2, bit_shift_inc);

VC1_VIDEO_read_PIC_LAYER_MV_MODE2:
         if (pic->MVmode == VC1_MVMODE_INT_COMP) {
            GET_LSB;
            pic->tab = (pic->Pquant > 24) ?
                       VC1_MVMODE2_TAB : VC1_MVMODE3_TAB;
            VC1_PARSE_1BYTE_VLC(pic->tab)
            bit_shift_inc  = *(pic->tab+1);
            pic->MVmode   |= *(pic->tab+2);
            ALIGN_DATA_BYTE(LUMA_SCALE, bit_shift_inc);

VC1_VIDEO_read_PIC_LAYER_LUMA_SCALE:
            GET_LSB;
            pic->Yscale = DATA_BYTE >> 2;
            ALIGN_DATA_BYTE(LUMA_SHIFT, 6);
VC1_VIDEO_read_PIC_LAYER_LUMA_SHIFT:
            GET_LSB;
            pic->Yshift = DATA_BYTE >> 2;
#ifdef VC1_VIDEO_TRACE
            VC1_TRACE_BEG_BLK
            printf("  MVmode2 = 0x%02hX\n"
                   "  LumaScale = %hu\n"
                   "  LumaShift = %hu\n",
                   (u16)pic->MVmode,
                   (u16)pic->Yscale,
                   (u16)pic->Yshift);
            VC1_TRACE_END_BLK
#endif
            ALIGN_DATA_BYTE(MV_TYPE, 6);
         }

VC1_VIDEO_read_PIC_LAYER_MV_TYPE:
         if (pic->MVmode & VC1_MVMODE_MIXED_MV) {
            seq->payload_len -= (u32)(DATA_PTR - seq->payload);
            seq->payload      = DATA_PTR;
            seq->bit_shift    = BIT_SHIFT_BYTE;
            seq->data         = DATA_BYTE;
            seq->BPstate      = 0;
#ifdef VC1_VIDEO_TRACE
            VC1_TRACE_BEG_BLK
            printf("  MVtype");
            VC1_TRACE_END_BLK
#endif
VC1_VIDEO_read_PIC_LAYER_MV_TYPE_BP:
            if (!VC1_VIDEO_read_BITPLANE(seq, pic->MVtype)) {
               PARSER_STATE = MV_TYPE_BP;
               return;
            }
            else if (seq->BPstate == VC1_BITPLANE_MODE_RAW)
               pic->BPraw |=  VC1_BP_RAW_MVTYPE;
            else
               pic->BPraw &= ~VC1_BP_RAW_MVTYPE;
#ifdef VC1_VIDEO_TRACE
            VC1_TRACE_BEG_BLK
            printf("\n");
            VC1_TRACE_END_BLK
#endif
         }
         else {
            seq->payload_len -= (u32)(DATA_PTR - seq->payload);
            seq->payload      = DATA_PTR;
            seq->bit_shift    = BIT_SHIFT_BYTE;
            seq->data         = DATA_BYTE;
         }
      }
      else {
         pic->MVmode  = (DATA_BYTE >= 0x80) ?
                        VC1_MVMODE_1MV :
                        VC1_MVMODE_1MV_HALFPIX_BI;
         seq->BPstate = 0;
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("  MVmode = 0x%02hX\n"
                "  MBdirect",
                pic->MVmode);
         VC1_TRACE_END_BLK
#endif
         ALIGN_DATA_BYTE(MB_DIRECT, 1);

         seq->payload_len -= (u32)(DATA_PTR - seq->payload);
         seq->payload      = DATA_PTR;
         seq->bit_shift    = BIT_SHIFT_BYTE;
         seq->data         = DATA_BYTE;
VC1_VIDEO_read_PIC_LAYER_MB_DIRECT:
         if (!VC1_VIDEO_read_BITPLANE(seq, pic->MBdirect)) {
            PARSER_STATE = MB_DIRECT;
            return;
         }
         else if (seq->BPstate == VC1_BITPLANE_MODE_RAW)
            pic->BPraw |=  VC1_BP_RAW_MBDIRECT;
         else
            pic->BPraw &= ~VC1_BP_RAW_MBDIRECT;
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("\n");
         VC1_TRACE_END_BLK
#endif
      }

      seq->BPstate = 0;
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("  MBskip");
      VC1_TRACE_END_BLK
#endif
VC1_VIDEO_read_PIC_LAYER_MB_SKIP:
      if (!VC1_VIDEO_read_BITPLANE(seq, pic->MBskip)) {
         PARSER_STATE = MB_SKIP;
         return;
      }
      else if (seq->BPstate == VC1_BITPLANE_MODE_RAW)
         pic->BPraw |=  VC1_BP_RAW_MBSKIP;
      else
         pic->BPraw &= ~VC1_BP_RAW_MBSKIP;
      DATA_PTR       = seq->payload;
      BIT_SHIFT_BYTE = seq->bit_shift;
      DATA_BYTE      = seq->data;

      GET_LSB;
      pic->mv_cbp_tab = DATA_BYTE;
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("\n  MVtable  = %hu\n"
               "  CBPtable = %hu\n",
             (u16)VC1_PIC_MV_TABLEp(pic),
             (u16)VC1_PIC_CBP_TABLEp(pic));
      VC1_TRACE_END_BLK
#endif
      ALIGN_DATA_BYTE(DQUANT, 4);

VC1_VIDEO_read_PIC_LAYER_DQUANT:
      if (seq->entrypt_quant >= 0x40) {
         if (seq->entrypt_quant >= 0x80)
            pic->dquant = 0x04;
         else {
            GET_LSB;
            if (DATA_BYTE < 0x80) {
               pic->dquant   = 0x00;
               bit_shift_inc = 1;
               if (pic->Pquant >= 16)
                  pic->type |= 0x02;   /* ACmode3_Tab */
               goto VC1_VIDEO_read_PIC_LAYER_DQUANT_DONE;
            }
            else {
               pic->dquant   = DATA_BYTE | 0x04;
               if (DATA_BYTE >= 0xE0) {      /* All Macroblocks */
                  bit_shift_inc = 4;
                  if (DATA_BYTE < 0xF0)
                     goto VC1_VIDEO_read_PIC_LAYER_DQUANT_DONE;
               }
               else if (DATA_BYTE >= 0xA0)   /* Single/Double Edge */
                  bit_shift_inc = 5;
               else                          /* All 4 Edges */
                  bit_shift_inc = 3;
               ALIGN_DATA_BYTE(PQUANT_ALT, bit_shift_inc);
            }
         }

VC1_VIDEO_read_PIC_LAYER_PQUANT_ALT:
         GET_LSB;
         if (DATA_BYTE >= 0xE0) {
            bit_shift_inc   = 8;
            pic->Pquant_alt = (DATA_BYTE << 1) & 0x3F;
         }
         else {
            bit_shift_inc   = 3;
            pic->Pquant_alt = ((DATA_BYTE >> 4) & 0xFE) + (pic->Pquant & 0xFE) + 2;
         }
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("  Pquant_alt = %hu\n",
                (u16)(pic->Pquant_alt >> 1));
         VC1_TRACE_END_BLK
#endif

VC1_VIDEO_read_PIC_LAYER_DQUANT_DONE:
#ifdef VC1_VIDEO_TRACE
         VC1_TRACE_BEG_BLK
         printf("  Dquant_InFrame = ");
         if (VC1_PIC_DQUANT_INFRAMEp(pic)) {
            printf("y\n"
                   "  DquantFrm = %c\n"
                   "  DquantProfile = ",
                   ((VC1_PIC_DQUANT_FRMp(pic)) ? 'y' : 'n'));
            switch (VC1_PIC_DQUANT_PROFILEp(pic)) {
            case VC1_DQUANT_PROFILE_4EDGES:
               printf("All 4 Edges\n");
               break;
            case VC1_DQUANT_PROFILE_2EDGES:
               printf("Double Edge\n"
                      "  DquantEdge = 0x%02hX\n",
                      (u16)(VC1_PIC_DQUANT_EDGEp(pic)));
               break;
            case VC1_DQUANT_PROFILE_1EDGE:
               printf("Single Edge\n"
                      "  DquantEdge = 0x%02hX\n",
                      (u16)(VC1_PIC_DQUANT_EDGEp(pic)));
               break;
            case VC1_DQUANT_PROFILE_ALL_MBS:
               printf("All Macroblocks\n"
                      "  DquantBIlevel = %c\n",
                      ((VC1_PIC_DQUANT_BILEVELp(pic)) ? 'y' : 'n'));
            }
         }
         else
            printf("n\n");
         VC1_TRACE_END_BLK
#endif
         ALIGN_DATA_BYTE(TT_FRM, bit_shift_inc);
      }
      else {
         pic->dquant = 0x00;
         if (pic->Pquant >= 16)
            pic->type |= 0x02;   /* ACmode3_Tab */
      }

VC1_VIDEO_read_PIC_LAYER_TT_FRM:
      if (VC1_EPT_VSTRANSFORMp(seq)) {
         GET_LSB;
         if (DATA_BYTE >= 0x80) {
            bit_shift_inc = 3;
            pic->VST      = (DATA_BYTE >> 5);
            if (pic->VST == VC1_VST_LEVEL_FRAME)
               pic->VST |= VC1_VST_CODED_8x8_ALL;
#ifdef VC1_VIDEO_TRACE
            VC1_TRACE_BEG_BLK
            printf("  VST_Level = Frame\n"
                   "  VST_Type  = %hd\n", (u16)VC1_VST_TYPEp(pic));
            VC1_TRACE_END_BLK
#endif
         }
         else {
            bit_shift_inc = 1;
            pic->VST      = VC1_VST_LEVEL_BLOCK;
         }
         ALIGN_DATA_BYTE(TRANS_TAB, bit_shift_inc);
      }
   }
   else if (pic->Pquant >= 16)
      pic->type |= 0x02;   /* ACmode3_Tab */

VC1_VIDEO_read_PIC_LAYER_TRANS_TAB:
   GET_LSB;
   if (DATA_BYTE >= 0x80) {
      pic->trans_tab = (DATA_BYTE >= 0xC0) ? 0x08 : 0;
      bit_shift_inc  = 2;
   }
   else {
      pic->trans_tab = VC1_PIC_LOW_MOTIONp(pic) ? 0x18 : 0x10;
      bit_shift_inc  = 1;
   }
   BIT_SHIFT_BYTE += bit_shift_inc;
   DATA_BYTE     <<= bit_shift_inc;
#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("  TransAC1_table = %hu (",
          (u16)VC1_PIC_TRANS_AC1p(pic));
   if (VC1_PIC_TRANS_AC1p(pic) < 2)
      printf("%hu)", (u16)(VC1_PIC_TRANS_AC1p(pic) + 1));
   else
      printf("0)");
   printf("  Mode3_table = %hu\n", (u16)VC1_AC_MODE3_TABp(pic));
   VC1_TRACE_END_BLK
#endif

   if (pic->type & VC1_I_FRAME) {
      if (DATA_BYTE >= 0x80) {
         pic->trans_tab |= (DATA_BYTE >= 0xC0) ? 0x02 : 0;
         bit_shift_inc   = 2;
      }
      else {
         pic->trans_tab |= VC1_PIC_LOW_MOTIONp(pic) ? 0x06 : 0x04;
         bit_shift_inc   = 1;
      }
      BIT_SHIFT_BYTE += bit_shift_inc;
      DATA_BYTE     <<= bit_shift_inc;
#ifdef VC1_VIDEO_TRACE
      VC1_TRACE_BEG_BLK
      printf("  TransAC2_table = %hu (",
             (u16)VC1_PIC_TRANS_AC2p(pic));
      if (VC1_PIC_TRANS_AC2p(pic) < 2)
         printf("%hu)\n", (u16)(VC1_PIC_TRANS_AC2p(pic) + 1));
      else
         printf("0)\n");
      VC1_TRACE_END_BLK
#endif
   }

   if (DATA_BYTE >= 0x80)
      pic->trans_tab |= 0x01;
#ifdef VC1_VIDEO_TRACE
   VC1_TRACE_BEG_BLK
   printf("  TransDC_table  = %hu\n",
          (u16)(VC1_PIC_TRANS_DCp(pic)));
   VC1_TRACE_END_BLK
#endif

   ALIGN_DATA_BYTE(PIC_LAYER_END, 1);

VC1_VIDEO_read_PIC_LAYER_END:
   PARSER_STATE      = (u32)VC1_PIC_TYPEp(pic) << 24;
   seq->payload_len -= (u32)(DATA_PTR - seq->payload);
   seq->payload      = DATA_PTR;
   seq->bit_shift    = BIT_SHIFT_BYTE;
   seq->data         = DATA_BYTE;
   return;

VC1_VIDEO_read_PIC_LAYER_STOP:
   seq->bit_shift   = BIT_SHIFT_BYTE;
   seq->data        = DATA_BYTE;
   seq->payload_len = 0;

#undef BUF_FULL
#undef PQ_INDEX
#undef MV_RANGE
#undef MV_MODE
#undef MV_MODE2
#undef LUMA_SCALE
#undef LUMA_SHIFT
#undef MV_TYPE
#undef MV_TYPE_BP
#undef MB_DIRECT
#undef MB_SKIP
#undef DQUANT
#undef PQUANT_ALT
#undef TT_FRM
#undef TRANS_TAB
#undef PIC_LAYER_END
}

