/***************************************************************************
                         SEQ_LAYER.c  -  description
                             -------------------
   Stuff needed for handling the VC1 sequence layer.
                             -------------------
    begin                : Tue May 23 2006
    copyright            : (C) 2006 by Dennis Lew
    email                : dennis_L76@yahoo.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*#define VC1_VIDEO_TRACE 2*/
#ifdef VC1_VIDEO_TRACE
# include <stdio.h>
#endif

#include "SEQ_LAYER.h"
#include "bitstream.h"

#if (VC1_VIDEO_TRACE > 1)
static u8 ss[4];
#endif

#define DATA_PTR        ptr
#define END_DATA_PTR    end_pos
#define PARSER_STATE    seq->state


/* Read the SEQ_STRUCT data.
      INPUT: seq - Pointer to a SEQ_LAYER structure. */
void VC1_VIDEO_read_SEQ_STRUCT(SEQ_LAYER *seq)
{
   const u8 *ptr     = seq->payload,
            *end_pos = seq->payload + seq->payload_len;

#undef  STOP_PARSING
#define STOP_PARSING    goto VC1_VIDEO_read_SEQ_STRUCT_END

   while (PARSER_STATE < 4) {
#if (VC1_VIDEO_TRACE > 1)
      ss[PARSER_STATE] = *DATA_PTR;
#endif
      switch (PARSER_STATE) {
      case 0:
         seq->profile        = (*DATA_PTR & 0xF0) | VC1_COLOUR_FORMAT_420;
         seq->postproc       = *DATA_PTR << 4;
         seq->framerate_num  = (u16)VC1_SEQ_FRAMERATE_PPp(seq);
         seq->framerate_den  = 1;
#ifdef VC1_VIDEO_TRACE
         printf("<SEQ_STRUCT START>\n"
                "   profile    = 0x%02hX\n"
                "   level      = 0x%02hX\n"
                "   framerate  = %hufps\n",
                (u16)VC1_SEQ_PROFILEp(seq),
                (u16)VC1_SEQ_LEVELp(seq),
                seq->framerate_num);
#endif
         break;
      case 1:
         seq->postproc      |= *DATA_PTR >> 4;
         seq->entrypt_flags  = *DATA_PTR & 0x08;
         seq->flags          = (*DATA_PTR >> 2) & 0x01;
#ifdef VC1_VIDEO_TRACE
         printf("   bitrate    = %huKbps\n",
                (u16)VC1_SEQ_BITRATE_PPp(seq) * 8);
         if (VC1_EPT_LOOPFILTERp(seq))
            printf("   loopfilter\n");
         if (VC1_SEQ_MULTIRESp(seq))
            printf("   multires\n");
#endif
         break;
      case 2:
         seq->entrypt_flags |= (*DATA_PTR >> 5) & 0x06;
         seq->entrypt_quant  = (*DATA_PTR << 2) & 0xE0;
         seq->entrypt_quant |= (*DATA_PTR << 1) & 0x0C;
#ifdef VC1_VIDEO_TRACE
         if (VC1_EPT_FASTUV_MCp(seq))
            printf("   fastUV_MC\n");
         if (VC1_EPT_EXT_MVp(seq))
            printf("   ext_MV\n");
         printf("   dquant     = 0x%02hX\n",
                (u16)VC1_EPT_DQUANTp(seq));
         if (VC1_EPT_VSTRANSFORMp(seq))
            printf("   VStransform\n");
         if (VC1_SEQ_OVERLAPp(seq))
            printf("   overlap\n");
         if (VC1_SEQ_SYNC_MARKERSp(seq))
            printf("   sync_markers\n");
#endif
         break;
      case 3:
         seq->entrypt_quant |= *DATA_PTR >> 7;
         seq->entrypt_quant |= (*DATA_PTR << 1) & 0x18;
         seq->flags         |= (*DATA_PTR << 2) & 0x08;
         if (*DATA_PTR & 0x70)
            seq->profile |= 0x01;
         else
            seq->profile &= 0xFE;
#ifdef VC1_VIDEO_TRACE
         if (VC1_SEQ_RANGE_REDp(seq))
            printf("   range_reduce\n");
         printf("   maxBframes = %hu\n"
                "   quantizer  = 0x%02hX\n",
                (u16)(*DATA_PTR & 0x70) >> 4,
                (u16)VC1_EPT_QUANTIZERp(seq));
         if (VC1_SEQ_FRAMEINTERP_FLAGp(seq))
            printf("   frame_interp\n");
# if (VC1_VIDEO_TRACE > 1)
         printf("   BYTES[0x%02hX 0x%02hX 0x%02hX 0x%02hX]\n",
                (u16)ss[0], (u16)ss[1], (u16)ss[2], (u16)ss[3]);
# endif
         printf("<SEQ_STRUCT END>\n");
#endif
      }
      PARSER_STATE++;
      NEXT_BYTE(PARSER_STATE)
   }

   seq->payload_len -= (u32)(DATA_PTR - seq->payload);
   seq->payload      = DATA_PTR;
   PARSER_STATE      = VC1_VIDEO_read_PIC_LAYER1_STATE;
   return;

VC1_VIDEO_read_SEQ_STRUCT_END:
   seq->payload_len = 0;
}
